#========================================================================================
# package      DE_EPAGES::Core::API::SystemCommand
# description  This Module starts an external process.
# state        public
#==============================================================================
package DE_EPAGES::Core::API::SystemCommand;
use base qw( Exporter );
use strict;

our @EXPORT_OK = qw(
    ShellCommand
    SystemCommand
    RunCommandsInDir
    UnpackTar
    PackTar
);

use Config;
use Cwd qw(realpath);
use File::Basename qw(dirname);
use File::Spec;

use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Core::API::TestParameter qw( TestString TestHash TestArray );
use DE_EPAGES::Core::API::File qw (RunOnGlob RunOnDirectory DeleteFile
                                   CreateDirectory);

#ERRORDEFINITION
sub ErrorDefinition () {
    return {
        'CommandError' => 'System command "#Command" failed: #ErrorMessage.',
        'CommandFailed' => 'System command "#Command" failed with exit code #ExitCode.',
        'CommandTerminated' => 'System command "#Command" terminated with signal #Signal.',
        'FileMissing' => 'File does not exist: #FileName',
        'SuffixNotSupported' => 'File has void suffix: #FileName',
    }
}
#ERRORDEFINITION END


#========================================================================================
# function     SystemCommand
# state        public
#----------------------------------------------------------------------------------------
# syntax       SystemCommand( $CommandLine, $hOptions );
# syntax       SystemCommand( $aCommandLine, $hOptions );
# syntax       $Output = SystemCommand( $CommandLine, $hOptions );
# example      SystemCommand( ['java', '-version'], { 'Print' => 1 });
# example      $Output = SystemCommand( "perl -v", { 'Capture' => 1 });
#----------------------------------------------------------------------------------------
# description  Executes a system command and throws an error if the command does not
#               terminate with exit code 0.
#               Possible error codes are:
#               <ul>
#                 <li>CommandError - command did not start</li>
#                 <li>CommandTerminated - command was terminated by signal</li>
#                 <li>CommandFailed - command did not start finish with exit code 0</li>
#               </ul>
#----------------------------------------------------------------------------------------
# input        $CommandLine | command with paramters | string
# input        $aCommandLine | command with paramters, first array element is the executable | ref.array
# input        $hOptions | (optional) keys are:
#                   <ul>
#                     <li>Print - print the command line before execution - boolean</li>
#                     <li>Capture - captures STDOUT and returns it as string</li>
#                     <li>Warning - does not thow an error but a warning</li>
#                   </ul>
#                    | ref.hash
# return       $Output | command output (from STDOUT), only if Capture is true | string
#========================================================================================
sub SystemCommand($;$) {
    my ($CommandLine, $hOptions) = @_;

    if( ref $CommandLine ) {
        TestArray('aCommandLine', $CommandLine);
    }
    else {
        TestString('CommandLine', $CommandLine);
    }

    if( defined $hOptions ) {
        TestHash('hOptions', $hOptions);
    }
    else {
        $hOptions = {};
    }

    if( $hOptions->{'Print'} ) {
        print _JoinCommandLine($CommandLine). "\n";
    }

    local $! = undef;
    my ( $Result, $Output);

    if( $hOptions->{'Capture'} ) {
        my $CommandString = _JoinCommandLine($CommandLine);
        $Output = `$CommandString`;
        $Result = $?;
    } else {
        $Result = ref $CommandLine ? system @$CommandLine : system $CommandLine;
    }

    ## no critic qw(ProhibitMagicNumbers)
    if( 0 != $Result ) {
        my $CommandString = _JoinCommandLine($CommandLine);
        if ($Result == -1 || ($Config{'osname'} eq 'MSWin32' && $Result == 256 && $!)) {
            my $ErrorCode = 'CommandError';
            my $ErrorParams = { 'Command' => $CommandString, 'ErrorMessage' => $! };
            Warning($ErrorCode, $ErrorParams) if $hOptions->{'Warning'};
            Error($ErrorCode, $ErrorParams) unless $hOptions->{'Warning'};
        }
        elsif ($Result & 127) {
            my $ErrorCode = 'CommandTerminated';
            my $ErrorParams = { 'Command' => $CommandString, 'Signal' => ($Result & 127) };
            Warning($ErrorCode, $ErrorParams) if $hOptions->{'Warning'};
            Error($ErrorCode, $ErrorParams) unless $hOptions->{'Warning'};
        }
        else {
            my $ErrorCode = 'CommandFailed';
            my $ErrorParams = { 'Command' => $CommandString, 'ExitCode' => ($Result >> 8) };
            Warning($ErrorCode, $ErrorParams) if $hOptions->{'Warning'};
            Error($ErrorCode, $ErrorParams) unless $hOptions->{'Warning'};
        }
    }
    ## use critic

    return $Output;
}

sub _JoinCommandLine {
    my ($CommandLine) = @_;
    return ref $CommandLine ? join(' ', @$CommandLine) : $CommandLine
}

#========================================================================================
# function     ShellCommand
# state        public
#----------------------------------------------------------------------------------------
# syntax       ShellCommand( $CommandLine, $hOptions );
# syntax       ShellCommand( $aCommandLine, $hOptions );
# syntax       $Output = ShellCommand( $CommandLine, $hOptions );
# example      ShellCommand( ['java', '-version'], { 'Print' => 1 });
# example      $Output = ShellCommand( "perl -v", { 'Capture' => 1 });
#----------------------------------------------------------------------------------------
# description  Executes a bash command and throws an error if the command does not
#               terminate with exit code 0.
#               Possible error codes are:
#               <ul>
#                 <li>CommandError - command did not start</li>
#                 <li>CommandTerminated - command was terminated by signal</li>
#                 <li>CommandFailed - command did not start finish with exit code 0</li>
#               </ul>
#----------------------------------------------------------------------------------------
# input        $CommandLine | command with paramters | string
# input        $aCommandLine | command with paramters, first array element is the executable | ref.array
# input        $hOptions | (optional) keys are:
#                   <ul>
#                     <li>Print - print the command line before execution - boolean</li>
#                     <li>Capture - captures STDOUT and returns it as string</li>
#                     <li>Warning - does not thow an error but a warning</li>
#                   </ul>
#                    | ref.hash
# return       $Output | command output (from STDOUT), only if Capture is true | string
#========================================================================================
sub ShellCommand($;$) {
    my ($CommandLine, $hOptions) = @_;
    if (ref ( $CommandLine )) {
        $CommandLine = join(' ', map { sprintf '\"%s\"', $_} @$CommandLine);
    } else {
        $CommandLine =~ s/\"/\\"/g; # escape double quotes
    }

    return SystemCommand(qq^/bin/bash -c "$CommandLine"\n^, $hOptions);
}

#========================================================================================
# function     RunCommandsInDir
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Status = RunCommandsInDir($Dir, $Args, $Output)
#----------------------------------------------------------------------------------------
# description  Execute files ending in *.sh, *.cmd or *.bat in $Dir with $Args
#----------------------------------------------------------------------------------------
# input        $Dir | directory containing files | string
# input        $Args | additional arguments for SystemCommand() | ref.array
# input        $Output | write output into string | ref.string
# return       $Status | 0 - success ; else - failue | integer
#========================================================================================
sub RunCommandsInDir($$;$) {
    my ($Dir, $Args, $Output) = @_;
    my $Result = 0;

    return $Result if ! -d $Dir;

    RunOnGlob($Dir, '*', sub {
        next unless -x "$Dir/$_";
        next unless -f "$Dir/$_";
        next unless /\.(sh|cmd|bat)$/;

        my $CommandLine = "$Dir/$_";
        $CommandLine .= ' '. join(' ', @$Args) if defined $Args;

        eval {
            if (defined $Output) {
                $$Output .= SystemCommand($CommandLine, {Print => 1, Capture => 1});
            } else {
                SystemCommand($CommandLine, {Print => 1});
            }
        };
        if (ExistsError()) {
            my $Error = GetError();
            Warning( $Error );
            print $Error->message, "\n";
            if ($Error->code eq 'CommandError') {
                $Result = -1;
            } elsif ($Error->code eq 'CommandTerminated') {
                $Result = $Error->{Vars}{Signal};
            } elsif ($Error->code eq 'CommandFailed') {
                $Result = $Error->{Vars}{ExitCode};
            }
            last;
        }
    });
    return $Result;
}

#========================================================================================
# function     UnpackTar
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aPackedFiles = UnpackTar($Package, $Directory, $hFlags);
# example      $aPackedFiles = UnpackTar($Package); # unpack into getcwd()
#               SetAccessRights($_) foreach @$aPackedFiles; # and set access rights
#               # beware: if the package only contains files, missing dirs are created
#               # automatically and access rights for this dirs must be  set as well!
# example      $aPackedFiles = UnpackTar($Package, $Directory); # unpack into $Directory
#----------------------------------------------------------------------------------------
# description  Unpack $Package (.tar/.tar.gz/.tgz) into $Directory (default: getcwd()).
#----------------------------------------------------------------------------------------
# input        $Package   | path to .tar/.tar.gz/.tgz package | string
# input        $Directory | put $Package files into $Directory (default: .) | string
# input        $hFlags    | (optional) keys are:
#                   <ul>
#                     <li>Print - print the command line before execution - boolean</li>
#                     <li>Warning - does not thow an error but a warning</li>
#                   </ul>
#                          | ref.hash
# return       $aPackedFiles | array of files in package | ref.array
#========================================================================================
sub UnpackTar ($;$$) {
    my ($Package, $Directory, $hFlags) = @_;

    Error('FileMissing',{'FileName' => $Package}) unless -f $Package;
    Error('SuffixNotSupported',{'FileName' => $Package})
        unless $Package =~ /\.(tgz|tar(\.gz)?)/;

    $Directory //= '.';
    # realpath doesn't work with MSWin32
    $Package = realpath($Package) if $^O ne 'MSWin32';
    # File::Spec::Unix->rel2abs() neither
    $Package = File::Spec->rel2abs($Package);
    $Package =~ s,\\,/,go if $^O eq 'MSWin32';

    CreateDirectory($Directory);

    my $Files;
    my $Options = $^O eq 'MSWin32' ? '' : ' --no-same-owner';
    $hFlags->{Capture} = 1;
    if ($Package =~ /\.(tgz|tar\.gz)/) {
        RunOnDirectory(
            $Directory,
            sub {$Files = SystemCommand("gzip -cd \"$Package\" | tar -ivx $Options", $hFlags)}
            );
    } else {
        RunOnDirectory(
            $Directory,
            sub {$Files = SystemCommand("tar -ivxf \"$Package\" $Options", $hFlags)}
            );
    }
    return [split(/\n/,$Files)];
}

#========================================================================================
# function     PackTar
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aPackedFiles = PackTar($Package, $Directory, $aFiles, $hFlags);
# example      PackTar($Package); # pack '*' (in getcwd()) into $Package
# example      PackTar($Package, $Directory); # chdir $Directory and pack '*' into $Package
# example      PackTar($Package, undef, ['a*']); # pack files starting with 'a' into $Package
#----------------------------------------------------------------------------------------
# description  Pack $aFiles (files/dirs/globs) from $Directory into $Package (.tar/.tar.gz/.tgz)
#----------------------------------------------------------------------------------------
# input        $Package   | path to .tar/.tar.gz/.tgz package | string
# input        $Directory | put files from $Directory into $Package (default: '.') | string
# input        $aFiles    | put $aFiles (files/dirs/globs) into $Package (default: ['*'])
#                            beware: '*' does not include files starting with '.'! | ref.array
# input        $hFlags  | (optional) keys are:
#                   <ul>
#                     <li>Print - print the command line before execution - boolean</li>
#                     <li>Warning - does not thow an error but a warning</li>
#                   </ul>
#                        | ref.hash
# return       $aPackedFiles | array of files in package | ref.array
#========================================================================================
sub PackTar ($;$$$) {
    my ($Package, $Directory, $aFiles, $hFlags) = @_;

    Error('SuffixNotSupported',{'FileName' => $Package})
        unless $Package =~ /\.(tgz|tar(\.gz)?)/;

    $aFiles //= ['*'];
    $Directory //= '.';
    # realpath doesn't work with MSWin32
    $Package = realpath($Package) if $^O ne 'MSWin32';
    # File::Spec::Unix->rel2abs() neither
    $Package = File::Spec->rel2abs($Package);
    $Package =~ s,\\,/,go if $^O eq 'MSWin32';

    CreateDirectory(dirname($Package));

    my $Files;
    # tar/gzip do not accept backslashes as path delimiters
    my $PackDirs = join(' ', map {s,\\,/,go if $^O eq 'MSWin32'; $_} @$aFiles);
    $hFlags->{Capture} = 1;
    if ($Package =~ /\.(tgz|tar\.gz)/ and $^O eq 'MSWin32') {
        # for win32, tar -zcvf does not work (and capture output with pipe doesn't work as well)
        RunOnDirectory(
            $Directory,
            sub {
                $Files = SystemCommand("tar -cvf \"$Package.tmp.$$\" $PackDirs", $hFlags);
                SystemCommand("gzip -c9 \"$Package.tmp.$$\"> \"$Package\"");
                DeleteFile("$Package.tmp.$$");
            }
            );
    } elsif ($Package =~ /\.(tgz|tar\.gz)/) {
        RunOnDirectory(
            $Directory,
            sub {$Files = SystemCommand("tar zcvf \"$Package\" $PackDirs", $hFlags)}
            );
    } else {
        RunOnDirectory(
            $Directory,
            sub {$Files = SystemCommand("tar -cvf \"$Package\" $PackDirs", $hFlags)}
            );
    }
    return [split(/\n/,$Files)];
}

1;
