use strict;
use Test::More tests => 33;
use Test::Exception;
use File::Basename qw( basename );

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Installer::API::Constants qw(
    STOPPED
    PENDING
    RUNNING
    DENIED
    MISSING
);
use DE_EPAGES::Installer::API::Service qw(
    StopService
    StartService
    ExistsService
    StatusService
    StatusUnixService
    EpagesCustomizedMysqlDaemon
    EpagesCustomizedHttpdDaemon
);

my %MAP_STAT = (
    STOPPED() => 'STOPPED',
    PENDING() => 'PENDING',
    RUNNING() => 'RUNNING',
    DENIED()  => 'DENIED ',
    MISSING() => 'MISSING',
    );

sub getService {
    my ($Status, $Name, $From, $To) = @_;

    my $Stat = ref($From) eq 'HASH' ? $From->{$Name} : $From;
    my $ServiceStatus = "${Status}Service('$Name') : $MAP_STAT{$Stat}";
    if ($To) {
        $Stat = ref($To) eq 'HASH' ? $To->{$Name} : $To;
        $ServiceStatus .= " -> $MAP_STAT{$Stat}";
    }
    return $ServiceStatus. "\n";
}

RunScript( Sub => sub {
    my ($StatX, $StatY);

    $StatX = StatusService('cron');
    ok($StatX->{'cron'} != MISSING, getService('Status', 'cron', $StatX));

    foreach my $Service (qw(appserver reqrouter webserver mongodb
                            solr-server solr-server-slave epagesj-server)) {
        my $ServiceExists = ExistsService($Service);
      SKIP: {
          skip("Service not installed: $Service", 1) unless $ServiceExists;
        }
        $StatX = StatusService($Service, {verbose => 1, dieIfMissing => 1});
        if ($StatX->{$Service} == STOPPED) {
            StartService($Service, {verbose => 1});
            $StatY = StatusService($Service, {verbose => 1, dieIfMissing => 1});
            ok($StatY->{$Service} == RUNNING, getService('Start', $Service, $StatX, $StatY));
            StopService($Service, {verbose => 1});
            $StatX = StatusService($Service, {verbose => 1, dieIfMissing => 1});
            ok($StatX->{$Service} == STOPPED, getService('Stop', $Service, $StatY, $StatX));
        } else {
            StopService($Service, {verbose => 1});
            $StatY = StatusService($Service, {verbose => 1, dieIfMissing => 1});
            ok($StatY->{$Service} == STOPPED, getService('Stop', $Service, $StatX, $StatY));
            StartService($Service, {verbose => 1});
            $StatX = StatusService($Service, {verbose => 1, dieIfMissing => 1});
            ok($StatX->{$Service} == RUNNING, getService('Start', $Service, $StatY, $StatX));
        }
    }

    my %ServiceCluster = (
        'epages'    => [qw(appserver reqrouter)],
        'epagesj'   => [qw(mongodb solr-server epagesj-server)],
        );
    if (ExistsService('solr-server-slave')) {
        push @{$ServiceCluster{epagesj}}, 'solr-server-slave';
    } else {
      SKIP: {
          skip('Service not installed: solr-server-slave', 2);
        }
    }

    foreach (sort keys %ServiceCluster) {
        my $ServiceExists = ExistsService($_);
      SKIP: {
          skip("Service not installed: $_",
               2 * scalar @{$ServiceCluster{$_}}) unless $ServiceExists;
        }
        my $Origin = StatusService($_, {verbose => 1, dieIfMissing => 1});
        StopService($_, {verbose => 1});
        $StatX = StatusService($_, {verbose => 1, dieIfMissing => 1});
        foreach my $Service (@{$ServiceCluster{$_}}) {
            ok($StatX->{$Service} == STOPPED, "$_: ".
               getService('Stop', $Service, $Origin, $StatX));
        }
        StartService($_, {verbose => 1});
        $StatY = StatusService($_, {verbose => 1, dieIfMissing => 1});
        foreach my $Service (@{$ServiceCluster{$_}}) {
            ok($StatY->{$Service} == RUNNING, "$_: ".
               getService('Start', $Service, $StatX, $StatY));
            StopService($Service,{quiet => 1}) if $Origin->{$Service} == STOPPED;
        }
    }

    foreach (qw(httpd mysqld)) {
        my $RunScript = /httpd/ ? EpagesCustomizedHttpdDaemon()
            : EpagesCustomizedMysqlDaemon;
      SKIP: {
          skip("epages $_ server not installed", 1) unless $RunScript;
        }
        my $Service = basename($RunScript);
        ok($Service, "Valid service name for $RunScript.");

        $StatX = StatusUnixService($Service, {verbose => 1});
        ok($StatX != MISSING, getService('StatusUnix', $Service, $StatX));
    }

    ##no critic qw(ProtectPrivateSubs)
    my $NonexistentService = DE_EPAGES::Installer::API::Service::_GetDaemon('xxx');
    ok( !($NonexistentService), 'Return false for non-existent service by default.');
    dies_ok {
        DE_EPAGES::Installer::API::Service::_GetDaemon('xxx', {'dieIfMissing' => 1});
    } 'Dies with non-existent service on request.';
    ## use critic
});
