#========================================================================================
# package      DE_EPAGES::MolliePayments::t::unit::PaymentStatusService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for UI::Provider
#========================================================================================
package DE_EPAGES::MolliePayments::t::unit::PaymentStatusService;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::Mockify;
use Test::Mockify::Matcher qw ( HashRef Object String Number );
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;
use DE_EPAGES::MolliePayments::API::Constants qw (
    LINE_ITEM_PAYMENT_MOLLIE_PAYMENTS
    PAYMENT_METHOD_MOLLIE_PAYMENTS
    PAYMENT_STATUS_FAILED
    PAYMENT_STATUS_PENDING
    PAYMENT_STATUS_PAID
);

use strict;

use constant MOLLIE_ID => 'tr_d0b0E3EA3v';
use constant ORDER_STATUS_OPEN => 'open';
use constant EPAGES_ORDER_NUMBER => '1009';

sub test {
    my $self = shift;

    $self->test_handleStatusOnPaymentWebhook_errors();
    $self->test_handleStatusOnPaymentWebhook_success();

    $self->test_handleStatusOnOrderWebhook_errors();
    $self->test_handleStatusOnOrderWebhook_success();

    $self->test_handleLineItemPaymentStatusChange_pending_paid_order();
    $self->test_handleLineItemPaymentStatusChange_pending_failed_order();
    $self->test_handleLineItemPaymentStatusChange_undef_failed_order();
    $self->test_handleLineItemPaymentStatusChange_undef_failed_basket();
    $self->test_handleLineItemPaymentStatusChange_undef_paid_order();
    $self->test_handleLineItemPaymentStatusChange_undef_paid_basket();
    $self->test_handleLineItemPaymentStatusChange_undef_pending_order();
    $self->test_handleLineItemPaymentStatusChange_undef_pending_basket();
    $self->test_handleLineItemPaymentStatusChange_empty_failed_order();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub test_handleStatusOnPaymentWebhook_errors {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
        'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_PAID
        }]
    };
    my $SUT = $self->_createSUT(undef, $self->_createMolliePaymentsClient($hOrderResponse));
    my $WrongLineItemPayment = $self->_createOrder();
    throws_ok(
        sub { $SUT->handleStatusOnPaymentWebhook($WrongLineItemPayment, MOLLIE_ID); },
        qr/Wrong LineItemPayment class/,
        $SubTestName . ' - Error gets thrown when Wrong LineItemPayment object'
    );

    $SUT = $self->_createSUT(undef, $self->_createMolliePaymentsClient($hOrderResponse));
    throws_ok(
        sub { $SUT->handleStatusOnPaymentWebhook($self->_createLineItemPaymentMolliePayments(), undef); },
        qr/WebhookId/,
        $SubTestName . ' - Error gets thrown when WebhookPaymentId is undef'
    );

    $SUT = $self->_createSUT(undef, $self->_createMolliePaymentsClient($hOrderResponse));
    throws_ok(
        sub { $SUT->handleStatusOnPaymentWebhook($self->_createLineItemPaymentMolliePayments(PAYMENT_STATUS_FAILED), MOLLIE_ID); },
        qr/already has a definitive status: failed/,
        $SubTestName . ' - Error gets thrown when LineItemPayment has status failed'
    );

    return;
}

sub test_handleStatusOnPaymentWebhook_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
        'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_PAID
        }]
    };

    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(PAYMENT_STATUS_PENDING);
    my $SUT = $self->_createSUT(undef, $self->_createMolliePaymentsClient($hOrderResponse));
    lives_ok(
        sub { $SUT->handleStatusOnPaymentWebhook($LineItemPayment, MOLLIE_ID);},
        "$SubTestName - test if handleStatusOnPaymentWebhook can be called without errors"
    );

    ok(WasCalled($SUT, '_getMollieOrderById'),
        "$SubTestName - _getMollieOrderById in handleStatusOnPaymentWebhook was called."
    );

    ok(WasCalled($SUT, 'handleLineItemPaymentStatusChange'),
        "$SubTestName - handleLineItemPaymentStatusChange in handleStatusOnPaymentWebhook was called."
    );

    is_deeply(  GetParametersFromMockifyCall( $SUT, 'handleLineItemPaymentStatusChange' ),
                [ $LineItemPayment, $hOrderResponse ],
                "$SubTestName - Methods withProfileId was called with correct hOrderResponse"
    );

    is($LineItemPayment->get('TransID'),
        MOLLIE_ID,
        "$SubTestName - LineItemPayment TransID was correctly updated."
    );

    return;
}

sub test_handleStatusOnOrderWebhook_errors {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
        'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_PAID
        }]
    };
    my $SUT = $self->_createSUT(undef, $self->_createMolliePaymentsClient($hOrderResponse));
    my $WrongLineItemPayment = $self->_createOrder();
    throws_ok(
        sub { $SUT->handleStatusOnOrderWebhook($WrongLineItemPayment, MOLLIE_ID); },
        qr/Wrong LineItemPayment class/,
        $SubTestName . ' - Error gets thrown when Wrong LineItemPayment object'
    );

    $SUT = $self->_createSUT(undef, $self->_createMolliePaymentsClient($hOrderResponse));
    throws_ok(
        sub { $SUT->handleStatusOnOrderWebhook($self->_createLineItemPaymentMolliePayments(), undef); },
        qr/WebhookId/,
        $SubTestName . ' - Error gets thrown when WebhookOrderId is undef'
    );

    $SUT = $self->_createSUT(undef, $self->_createMolliePaymentsClient($hOrderResponse));
    throws_ok(
        sub { $SUT->handleStatusOnOrderWebhook($self->_createLineItemPaymentMolliePayments(PAYMENT_STATUS_FAILED), MOLLIE_ID); },
        qr/already has a definitive status: failed/,
        $SubTestName . ' - Error gets thrown when LineItemPayment has status failed'
    );

    return;
}

sub test_handleStatusOnOrderWebhook_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
        'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_PAID
        }]
    };

    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(PAYMENT_STATUS_PENDING);
    my $SUT = $self->_createSUT(undef, $self->_createMolliePaymentsClient($hOrderResponse));
    lives_ok(
        sub { $SUT->handleStatusOnOrderWebhook($LineItemPayment, MOLLIE_ID);},
        "$SubTestName - test if handleStatusOnOrderWebhook can be called without errors"
    );

    ok(WasCalled($SUT, '_getMollieOrderById'),
        "$SubTestName - _getMollieOrderById in handleStatusOnOrderWebhook was called."
    );

    ok(WasCalled($SUT, 'handleLineItemPaymentStatusChange'),
        "$SubTestName - handleLineItemPaymentStatusChange in handleStatusOnOrderWebhook was called."
    );

    is_deeply(  GetParametersFromMockifyCall( $SUT, 'handleLineItemPaymentStatusChange' ),
                [ $LineItemPayment, $hOrderResponse ],
                "$SubTestName - Methods withProfileId was called with correct hOrderResponse"
    );

    is($LineItemPayment->get('TransID'),
        MOLLIE_ID,
        "$SubTestName - LineItemPayment TransID was correctly updated."
    );

    return;
}

sub test_handleLineItemPaymentStatusChange_pending_paid_order {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
    'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_PAID
        }]
    };

    my $MockedRefundsService = $self->_createMolliePaymentsRefundsService();

    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(PAYMENT_STATUS_PENDING);
    my $MockedMollieClient = $self->_createMolliePaymentsClient($hOrderResponse);
    my $MockedOrdersRequest = $self->_createOrdersRequest();

    my $SUT = $self->_createSUT($MockedRefundsService, $MockedMollieClient, $MockedOrdersRequest);
    my $StatusResult;
    lives_ok(
        sub { $StatusResult = $SUT->handleLineItemPaymentStatusChange($LineItemPayment, $hOrderResponse);},
        "$SubTestName - test if handleLineItemPaymentStatusChange can be called without errors"
    );

    is($StatusResult,
        PAYMENT_STATUS_PAID,
        "$SubTestName - $StatusResult was correctly returned."
    );

    is($LineItemPayment->get('TransStatus'),
        PAYMENT_STATUS_PAID,
        "$SubTestName - LineItemPayment was correctly updated."
    );

    my $PaymentTransTime = $LineItemPayment->get('TransTime');
    is(isValid($PaymentTransTime),
        1,
        "$SubTestName - LineItemPayment TransTime was correctly updated."
    );

    is($LineItemPayment->container()->parent()->get('PaidOn'),
        $PaymentTransTime,
        "$SubTestName - Order PaidOn was correctly updated."
    );

    ok((not WasCalled($SUT, '_cancelMollieOrderById')),
        "$SubTestName - _cancelMollieOrderById on paid status was not called."
    );

    ok(WasCalled($MockedRefundsService, 'updateLineItemRefundAmounts'),
        "$SubTestName - updateLineItemRefundAmounts was called when paid status."
    );

    return;
}

sub test_handleLineItemPaymentStatusChange_pending_failed_order {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
    'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_FAILED
        }]
    };

    my $MockedRefundsService = $self->_createMolliePaymentsRefundsService();
    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(PAYMENT_STATUS_PENDING);
    my $MockedMollieClient = $self->_createMolliePaymentsClient($hOrderResponse);
    my $MockedOrdersRequest = $self->_createOrdersRequest();
    
    my $SUT = $self->_createSUT($MockedRefundsService, $MockedMollieClient, $MockedOrdersRequest);
    my $StatusResult;
    lives_ok(
        sub { $StatusResult = $SUT->handleLineItemPaymentStatusChange($LineItemPayment, $hOrderResponse);},
        "$SubTestName - test if handleLineItemPaymentStatusChange can be called without errors"
    );

    is($StatusResult,
        PAYMENT_STATUS_FAILED,
        "$SubTestName - $StatusResult was correctly returned."
    );

    is($LineItemPayment->get('TransStatus'),
        PAYMENT_STATUS_FAILED,
        "$SubTestName - LineItemPayment TransStatus was correctly updated to failed");

    my $PaymentTransTime = $LineItemPayment->get('TransTime');
    ok(defined($PaymentTransTime),
        "$SubTestName - LineItemPayment TransTime is undef."
    );

    is($LineItemPayment->container()->parent()->get('PaidOn'),
        undef,
        "$SubTestName - Order PaidOn is undef."
    );

    ok(WasCalled($SUT, '_cancelMollieOrderById'),
        "$SubTestName - _cancelMollieOrderById on failed status was called."
    );

    ok((not WasCalled($MockedRefundsService, 'updateLineItemRefundAmounts')),
        "$SubTestName - updateLineItemRefundAmounts was not called when failed status."
    );

    is_deeply(  GetParametersFromMockifyCall( $MockedOrdersRequest, 'withOrderId' ),
                [ MOLLIE_ID ],
                "$SubTestName - Method withOrderId was called with correct Order id"
    );

    return;
}

sub test_handleLineItemPaymentStatusChange_undef_paid_basket {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
    'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_PAID
        }]
    };

    my $MockedRefundsService = $self->_createMolliePaymentsRefundsService();
    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(undef, my $IsBasket = 1);
    my $MockedMollieClient = $self->_createMolliePaymentsClient($hOrderResponse);
    my $MockedOrdersRequest = $self->_createOrdersRequest();
    
    my $SUT = $self->_createSUT($MockedRefundsService, $MockedMollieClient, $MockedOrdersRequest);
    my $StatusResult;
    lives_ok(
        sub { $StatusResult = $SUT->handleLineItemPaymentStatusChange($LineItemPayment, $hOrderResponse);},
        "$SubTestName - test if handleLineItemPaymentStatusChange can be called without errors"
    );

    is($StatusResult,
        PAYMENT_STATUS_PAID,
        "$SubTestName - $StatusResult was correctly returned."
    );

    is($LineItemPayment->get('TransStatus'),
        PAYMENT_STATUS_PAID,
        "$SubTestName - LineItemPayment was correctly updated."
    );

    my $PaymentTransTime = $LineItemPayment->get('TransTime');
    ok(defined($PaymentTransTime),
        "$SubTestName - LineItemPayment TransTime is undef."
    );

    is($LineItemPayment->container()->parent()->get('PaidOn'),
        undef,
        "$SubTestName - Order PaidOn is undef."
    );

    ok(!WasCalled($SUT, '_cancelMollieOrderById'),
        "$SubTestName - _cancelMollieOrderById on paid status was NOT called."
    );

    ok(WasCalled($MockedRefundsService, 'updateLineItemRefundAmounts'),
        "$SubTestName - updateLineItemRefundAmounts was called when paid status."
    );

    ok(!WasCalled($MockedMollieClient, 'doPutRequest'),
        "$SubTestName - Mollie Order Number was NOT updated on a basket."
    );
}

sub test_handleLineItemPaymentStatusChange_undef_paid_order {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
    'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_PAID
        }]
    };

    my $MockedRefundsService = $self->_createMolliePaymentsRefundsService();
    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(undef);
    my $MockedMollieClient = $self->_createMolliePaymentsClient($hOrderResponse);
    my $MockedOrdersRequest = $self->_createOrdersRequest();
    
    my $SUT = $self->_createSUT($MockedRefundsService, $MockedMollieClient, $MockedOrdersRequest);
    my $StatusResult;
    lives_ok(
        sub { $StatusResult = $SUT->handleLineItemPaymentStatusChange($LineItemPayment, $hOrderResponse);},
        "$SubTestName - test if handleLineItemPaymentStatusChange can be called without errors"
    );

    is($StatusResult,
        PAYMENT_STATUS_PAID,
        "$SubTestName - $StatusResult was correctly returned."
    );

    is($LineItemPayment->get('TransStatus'),
        PAYMENT_STATUS_PAID,
        "$SubTestName - LineItemPayment was correctly updated."
    );

    my $PaymentTransTime = $LineItemPayment->get('TransTime');
    is(isValid($PaymentTransTime),
        1,
        "$SubTestName - LineItemPayment TransTime was correctly updated."
    );

    is($LineItemPayment->container()->parent()->get('PaidOn'),
        $PaymentTransTime,
        "$SubTestName - Order PaidOn was correctly updated."
    );

    ok(( not WasCalled($SUT, '_cancelMollieOrderById')),
        "$SubTestName - _cancelMollieOrderById on paid status was not called."
    );

    ok(WasCalled($MockedRefundsService, 'updateLineItemRefundAmounts'),
        "$SubTestName - updateLineItemRefundAmounts was called when paid status."
    );

    return;
}

sub test_handleLineItemPaymentStatusChange_empty_failed_order {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
    'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_FAILED
        }]
    };

    my $MockedRefundsService = $self->_createMolliePaymentsRefundsService();
    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments('');
    my $MockedMollieClient = $self->_createMolliePaymentsClient($hOrderResponse);
    my $MockedOrdersRequest = $self->_createOrdersRequest();
    
    my $SUT = $self->_createSUT($MockedRefundsService, $MockedMollieClient, $MockedOrdersRequest);
    my $StatusResult;
    lives_ok(
        sub { $StatusResult = $SUT->handleLineItemPaymentStatusChange($LineItemPayment, $hOrderResponse);},
        "$SubTestName - test if handleLineItemPaymentStatusChange can be called without errors"
    );

    is($StatusResult,
        PAYMENT_STATUS_FAILED,
        "$SubTestName - $StatusResult was correctly returned."
    );

    is($LineItemPayment->get('TransStatus'),
        PAYMENT_STATUS_FAILED,
        "$SubTestName - LineItemPayment TransStatus was correctly updated."
    );

    is($LineItemPayment->get('TransID'),
        undef,
        "$SubTestName - LineItemPayment TransID was correctly updated."
    );

    my $PaymentTransTime = $LineItemPayment->get('TransTime');
    ok(defined($PaymentTransTime),
        "$SubTestName - LineItemPayment TransTime is undef."
    );

    is($LineItemPayment->container()->parent()->get('PaidOn'),
        undef,
        "$SubTestName - Order PaidOn was correctly updated."
    );

    ok(WasCalled($SUT, '_cancelMollieOrderById'),
        "$SubTestName - _cancelMollieOrderById on failed status was called."
    );

    ok((not WasCalled($MockedRefundsService, 'updateLineItemRefundAmounts')),
        "$SubTestName - updateLineItemRefundAmounts was not called when failed status."
    );

    return;
}

sub test_handleLineItemPaymentStatusChange_undef_failed_basket {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
    'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_FAILED
        }]
    };

    my $MockedRefundsService = $self->_createMolliePaymentsRefundsService();
    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(undef, my $IsBasket = 1);
    my $MockedMollieClient = $self->_createMolliePaymentsClient($hOrderResponse);
    my $MockedOrdersRequest = $self->_createOrdersRequest();
    
    my $SUT = $self->_createSUT($MockedRefundsService, $MockedMollieClient, $MockedOrdersRequest);
    my $StatusResult;
    lives_ok(
        sub { $StatusResult = $SUT->handleLineItemPaymentStatusChange($LineItemPayment, $hOrderResponse);},
        "$SubTestName - test if handleLineItemPaymentStatusChange can be called without errors"
    );

    is($StatusResult,
        PAYMENT_STATUS_FAILED,
        "$SubTestName - $StatusResult was correctly returned."
    );

    is($LineItemPayment->get('TransStatus'),
        PAYMENT_STATUS_FAILED,
        "$SubTestName - LineItemPayment TransStatus was correctly updated to failed."
    );

    my $PaymentTransTime = $LineItemPayment->get('TransTime');
    ok(defined($PaymentTransTime),
        "$SubTestName - LineItemPayment TransTime is undef."
    );

    is($LineItemPayment->container()->parent()->get('PaidOn'),
        undef,
        "$SubTestName - Order PaidOn was correctly updated."
    );

    ok(WasCalled($SUT, '_cancelMollieOrderById'),
        "$SubTestName - _cancelMollieOrderById on failed status was called."
    );

    ok((not WasCalled($MockedRefundsService, 'updateLineItemRefundAmounts')),
        "$SubTestName - updateLineItemRefundAmounts was not called when failed status."
    );

    
    ok(!WasCalled($MockedMollieClient, 'doPutRequest'),
        "$SubTestName - Mollie Order Number was NOT updated on a basket."
    );

    return;
}

sub test_handleLineItemPaymentStatusChange_undef_failed_order {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
    'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_FAILED
        }]
    };

    my $MockedRefundsService = $self->_createMolliePaymentsRefundsService();
    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(undef);
    my $MockedMollieClient = $self->_createMolliePaymentsClient($hOrderResponse);
    my $MockedOrdersRequest = $self->_createOrdersRequest();
    
    my $SUT = $self->_createSUT($MockedRefundsService, $MockedMollieClient, $MockedOrdersRequest);
    my $StatusResult;
    lives_ok(
        sub { $StatusResult = $SUT->handleLineItemPaymentStatusChange($LineItemPayment, $hOrderResponse);},
        "$SubTestName - test if handleLineItemPaymentStatusChange can be called without errors"
    );

    is($StatusResult,
        PAYMENT_STATUS_FAILED,
        "$SubTestName - $StatusResult was correctly returned."
    );

    is($LineItemPayment->get('TransStatus'),
        PAYMENT_STATUS_FAILED,
        "$SubTestName - LineItemPayment TransStatus was correctly updated to failed."
    );

    my $PaymentTransTime = $LineItemPayment->get('TransTime');
    ok(defined($PaymentTransTime),
        "$SubTestName - LineItemPayment TransTime is undef."
    );

    is($LineItemPayment->container()->parent()->get('PaidOn'),
        undef,
        "$SubTestName - Order PaidOn was correctly updated."
    );

    ok(WasCalled($SUT, '_cancelMollieOrderById'),
        "$SubTestName - _cancelMollieOrderById on failed status was called."
    );

    ok((not WasCalled($MockedRefundsService, 'updateLineItemRefundAmounts')),
        "$SubTestName - updateLineItemRefundAmounts was not called when failed status."
    );

    return;
}

sub test_handleLineItemPaymentStatusChange_undef_pending_basket {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
    'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_PENDING
        }]
    };

    my $MockedRefundsService = $self->_createMolliePaymentsRefundsService();
    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(undef, my $IsBasket = 1);
    my $MockedMollieClient = $self->_createMolliePaymentsClient($hOrderResponse);
    my $MockedOrdersRequest = $self->_createOrdersRequest();
    
    my $SUT = $self->_createSUT($MockedRefundsService, $MockedMollieClient, $MockedOrdersRequest);
    my $StatusResult;
    lives_ok(
        sub { $StatusResult = $SUT->handleLineItemPaymentStatusChange($LineItemPayment, $hOrderResponse);},
        "$SubTestName - test if handleLineItemPaymentStatusChange can be called without errors"
    );

    is($StatusResult,
        PAYMENT_STATUS_PENDING,
        "$SubTestName - $StatusResult was correctly returned."
    );

    is($LineItemPayment->get('TransStatus'),
        PAYMENT_STATUS_PENDING,
        "$SubTestName - LineItemPayment was correctly updated."
    );

    my $PaymentTransTime = $LineItemPayment->get('TransTime');
    ok(defined($PaymentTransTime),
        "$SubTestName - LineItemPayment TransTime is undef."
    );

    is($LineItemPayment->container()->parent()->get('PaidOn'),
        undef,
        "$SubTestName - Order PaidOn was correctly updated."
    );

    ok(( not WasCalled($SUT, '_cancelMollieOrderById')),
        "$SubTestName - _cancelMollieOrderById on failed status was not called."
    );


    ok((not WasCalled($MockedRefundsService, 'updateLineItemRefundAmounts')),
        "$SubTestName - updateLineItemRefundAmounts was not called when pending status."
    );

    ok(!WasCalled($MockedMollieClient, 'doPutRequest'),
        "$SubTestName - Mollie Order Number was NOT updated on a basket."
    );

    return;
}

sub test_handleLineItemPaymentStatusChange_undef_pending_order {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
    'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_PENDING
        }]
    };

    my $MockedRefundsService = $self->_createMolliePaymentsRefundsService();
    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(undef);
    my $MockedMollieClient = $self->_createMolliePaymentsClient($hOrderResponse);
    my $MockedOrdersRequest = $self->_createOrdersRequest();
    
    my $SUT = $self->_createSUT($MockedRefundsService, $MockedMollieClient, $MockedOrdersRequest);
    my $StatusResult;
    lives_ok(
        sub { $StatusResult = $SUT->handleLineItemPaymentStatusChange($LineItemPayment, $hOrderResponse);},
        "$SubTestName - test if handleLineItemPaymentStatusChange can be called without errors"
    );

    is($StatusResult,
        PAYMENT_STATUS_PENDING,
        "$SubTestName - $StatusResult was correctly returned."
    );

    is($LineItemPayment->get('TransStatus'),
        PAYMENT_STATUS_PENDING,
        "$SubTestName - LineItemPayment was correctly updated."
    );

    my $PaymentTransTime = $LineItemPayment->get('TransTime');
    ok(defined($PaymentTransTime),
        "$SubTestName - LineItemPayment TransTime is undef."
    );

    is($LineItemPayment->container()->parent()->get('PaidOn'),
        undef,
        "$SubTestName - Order PaidOn was correctly updated."
    );

    ok(( not WasCalled($SUT, '_cancelMollieOrderById')),
        "$SubTestName - _cancelMollieOrderById on failed status was not called."
    );


    ok((not WasCalled($MockedRefundsService, 'updateLineItemRefundAmounts')),
        "$SubTestName - updateLineItemRefundAmounts was not called when pending status."
    );

    return;
}


#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($MolliePaymentsRefundsService, $MolliePaymentsClient, $OrderRequest) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::MolliePayments::API::PaymentStatusService');

    $Mockify->mock('_LogPayment')->when(String(), HashRef())->thenReturnUndef();

    $Mockify->spy('_getMollieOrderById')
        ->when(Object('DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments'));

    $Mockify->spy('_cancelMollieOrderById')
        ->when(Object('DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments'));

    $Mockify->spy('handleLineItemPaymentStatusChange')
        ->when(Object('DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments'), HashRef());

    $Mockify->mock('_createMollieClient')
        ->when(Object('DE_EPAGES::MolliePayments::API::Requests::Orders'))
        ->thenReturn($MolliePaymentsClient);

    $Mockify->mock('_createRefundsService')
        ->when()->thenReturn($MolliePaymentsRefundsService // $self->_createMolliePaymentsRefundsService());

    $Mockify->mock('_createOrdersRequest')
        ->when(HashRef())->thenReturn($OrderRequest // $self->_createOrdersRequest());

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createMolliePaymentsClient {
    my $self = shift;
    my ($hResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::MolliePayments::API::MolliePaymentsClient');


    $Mockify->mock('doGetRequest')->when()->thenReturn($hResponse);

    $Mockify->mock('doDeleteRequest')->when()->thenReturn($hResponse);

    $Mockify->mock('doPutRequest')->when()->thenReturn($hResponse);

    $Mockify->mock('withTestMode')->when(Number())->thenReturn($Mockify->getMockObject());

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createMolliePaymentsRefundsService {
    my $self = shift;
    my ($hResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::MolliePayments::API::MolliePaymentsRefundsService');

    $Mockify->mock('updateLineItemRefundAmounts')
        ->when(Object('DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments'), HashRef())
        ->thenReturnUndef();

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createOrdersRequest {
    my $self = shift;
    my ($hResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::MolliePayments::API::Requests::Orders');

    $Mockify->spy('withOrderId')->when(String());

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createOrder {
    my $self = shift;

    my $hAttributes = {
        'ID' => 1234,
        'instanceOf' => 'CustomerOrder',
        'PaidOn' => undef,
        'Alias' => EPAGES_ORDER_NUMBER
    };
    my $MockifyOrder = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes
    );

    return $MockifyOrder->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createBasket {
    my $self = shift;

    my $hAttributes = {
        'ID' => 1234,
        'instanceOf' => 'Basket',
        'PaidOn' => undef,
        'Alias' => EPAGES_ORDER_NUMBER
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::Basket',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createLineItemContainer {
    my $self = shift;
    my ($isBasket) = @_;

    my $hAttributes = {};
    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes
    );

    my $Parent = $isBasket ? $self->_createBasket() : $self->_createOrder();
    $MockifyContainer->addMockWithReturnValue('parent', $Parent);

    return $MockifyContainer->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createLineItemPaymentMolliePayments {
    my $self = shift;
    my ($TransStatus, $isBasket) = @_;

    my $hAttributes = {
        'PaymentMethod' => $self->_createPaymentMethod(),
        'instanceOf' => LINE_ITEM_PAYMENT_MOLLIE_PAYMENTS,
        'TransStatus' => $TransStatus,
        'MollieOrderId' => MOLLIE_ID,
        'TransTime' => undef,
        'TransID' => undef,
        'LineItemContainer' => $self->_createLineItemContainer($isBasket),
        'IsTestTransaction' => 1,
        'ID' => 21348
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments', $hAttributes
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createPaymentMethod {
    my $self = shift;

    my $hAttributes = {
        'ID' => 123,
        'instanceOf' => PAYMENT_METHOD_MOLLIE_PAYMENTS
    };
    my $MockifyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments', $hAttributes
    );

    return $MockifyPaymentMethod->getMockObject();
}

__PACKAGE__->runScript();
