#==============================================================================
# package      DE_EPAGES::PayPalExpress::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::PayPalExpress::UI::Shop;
use base DE_EPAGES::Order::UI::Shop;

use strict;
use feature qw (switch);

use Data::Dumper qw ( Dumper );
use HTTP::Request;
use LWP::UserAgent;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( UsePackage fcmp );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByGUID
    LoadObjectByGUID
    LoadRootObject
);
use DE_EPAGES::Object::API::Table::Language qw ( GetInfoLanguage );
use DE_EPAGES::Object::API::Table::Locale qw ( GetInfoLocale );
use DE_EPAGES::Order::UI::Basket;
use DE_EPAGES::ShopMailType::API::ShopMailType qw ( GetShopMailType );
use DE_EPAGES::ThirdPartyConfig::API::ThirdPartyConfig qw ( GetDefaultTimeOut );

use DE_EPAGES::Core::API::Constants qw(PERL516);
BEGIN {
    if ($] < PERL516) {
        # use old packages for old perl
        UsePackage('Net::INET6Glue');
    }
}


#==============================================================================
sub PayPalExpressIPN {
    my $self = shift;
    my ($Servlet) = @_;

    my $FormValues = $Servlet->params();
    LogPayment('PayPalExpress', 'PayPalExpressIPN input values', $FormValues);

    # ignore IPN calls for refunds
    return undef if (defined($FormValues->{'reason_code'}) && $FormValues->{'reason_code'} =~ /refund/i);

    # test the parameters of PayPal, show errors if something is wrong
    my $hValues = TestPayPalExpressParametersIPN($Servlet, $FormValues);
    return undef if not $hValues;
    LogPayment('PayPalExpress', 'PayPalExpressIPN tested values', $hValues);

    my $PaymentLineItem = delete($hValues->{'PaymentLineItem'});

    # EPG-27322: reset IPN processed flag before verify if IPN is sent because of an reversed payment
    $PaymentLineItem->set({'IsIPNProcessed' => 0}) if (defined $hValues->{'TransStatus'} and lc $hValues->{'TransStatus'} eq 'reversed');

    my $Object = $PaymentLineItem->container->parent;

    # EPG-26477: verify IPN message
    return undef unless _verifyPayPalExpressIPN($Servlet->request, $PaymentLineItem);

    # EPG-26457: prevent creating an order if transaction status isn't positive and order creation occurs only if payment is successful
    my $DoPaymentOnBasket = $PaymentLineItem->get('PaymentMethod')->get('DoPaymentOnBasket');
    return undef if ($DoPaymentOnBasket and not lc $hValues->{'TransStatus'} ~~ [qw(completed pending created processed)]);

    # update the paymentlineitem
    $PaymentLineItem->set($hValues);

    my $Order = $Object->instanceOf('CustomerOrder') ? $Object : DE_EPAGES::Order::UI::Basket->doBasket2Order($Servlet, $Object);

    $Servlet->vars('ViewAction','View');
    $Servlet->vars('ViewObjectID',$Order->id);
    $Servlet->registerSession($Order->get('User'),0 );

    # reset cached language id etc.
    $Servlet->locale(1);
    $Servlet->languageID(1);
    $Servlet->currencyID(1);

    # set order status
    $self->_SetTransStatus($Order, lc($hValues->{'TransStatus'})) if $hValues->{'TransStatus'};

    # set IPN processed flag
    $PaymentLineItem->set({'IsIPNProcessed' => 1});
    return;
}

#==============================================================================
sub _verifyPayPalExpressIPN {
    my ($Request, $PaymentLineItem) = @_;

    my $PaymentMethod = $PaymentLineItem->get('PaymentMethod');
    my $PaymentMethodAlias = $PaymentMethod->alias;

    # cancel if transaction was processed before
    if ($PaymentLineItem->get('IsIPNProcessed')) {
        LogPayment('PayPalExpress', "Verify IPN: reject IPN message for payment method $PaymentMethodAlias because it's already processed", {});
        return 0;
    }

    # resend IPN message
    my $URL = LoadRootObject()->get($PaymentMethod->get('IsInTestMode') ? 'PayPalExpress_SandboxURL' : 'PayPalExpress_ProductionURL');

    # tell PayPal we want to verify that IPN message
    my $Query = $Request->content . '&cmd=_notify-validate';

    LogPayment('PayPalExpress', "Verify IPN: query to send to $URL for payment method $PaymentMethodAlias", $Query);

    my $VerifyRequest = HTTP::Request->new('POST', $URL);
    my $UserAgent = LWP::UserAgent->new();
    $UserAgent->timeout(GetDefaultTimeOut);
    $UserAgent->agent('ePages/6 ' . $UserAgent->agent());
    $VerifyRequest->content_type('application/x-www-form-urlencoded');
    $VerifyRequest->header('Host' => 'www.paypal.com');
    $VerifyRequest->content($Query);

    my $Response = $UserAgent->request($VerifyRequest);

    # check verification response
    if ($Response->is_success && ($Response->content eq 'VERIFIED')) {
        LogPayment('PayPalExpress', "Verify IPN: response from $URL for payment method $PaymentMethodAlias", $Response->content());
        return 1;
    }

    eval { Error('IPN_VERIFICATION_FAILURE', { 'Response' => Dumper($Response->content()) }); };
    if (ExistsError()) {
        LogError(GetError());
    }

    # verification not successful
    return 0;
}

#==============================================================================
sub TestPayPalExpressParametersIPN {
    my ($Servlet, $FormValues) = @_;
    my $Shop = $Servlet->object;

    my ($check, $transactionID, $transactionStatus);
    eval {
        ($check, $transactionID, $transactionStatus) = _doValidation($Shop, $FormValues);
    };
    if (ExistsError()) {
        Warning( GetError() );
        return undef;
    }

    my $PaymentLineItem = LoadObjectByGUID($FormValues->{'PaymentGUID'} );
    my $Now = GetCurrentDBHandle()->currentDateTime;

    my %Values = (
        'PaymentLineItem' => $PaymentLineItem,
        'TransTimeMod'    => $Now,
        'IsSecure'        => defined($FormValues->{'payer_status'}) && lc($FormValues->{'payer_status'}) eq 'verified' ?  1 : 0,
    );

    if (defined($FormValues->{'amt'}) && defined($FormValues->{'cc'})) {
        $Values{'Amount'}     = $FormValues->{'amt'};
        $Values{'CurrencyID'} = $FormValues->{'cc'};
    }
    elsif (defined($FormValues->{'mc_gross'}) && defined($FormValues->{'mc_currency'})) {
        $Values{'Amount'}     = $FormValues->{'mc_gross'};
        $Values{'CurrencyID'} = $FormValues->{'mc_currency'};
    }
    elsif (defined($FormValues->{'mc_gross_1'}) && defined($FormValues->{'mc_currency'})) {
        # should not happen as _doValidation copies the value from mc_gross_1 to mc_gross
        $Values{'Amount'}     = $FormValues->{'mc_gross_1'};
        $Values{'CurrencyID'} = $FormValues->{'mc_currency'};
    }

    if ($check && defined($transactionStatus)) {
        $Values{'TransStatus'} = $transactionStatus;
    }
    elsif (!$PaymentLineItem->get('TransStatus')) {
        $Values{'TransStatus'} = 'NotValidatedYet';
    }
    $Values{'TransID'}    = $transactionID if defined $transactionID;
    $Values{'TransTime'}  = $Now unless defined $PaymentLineItem->get('TransTime');
    $Values{'PayerEMail'} = $FormValues->{'payer_email'} if defined $FormValues->{'payer_email'};

    return \%Values;
}

#==============================================================================
sub _doValidation {
    my ($Shop, $FormValues) = @_;
    my ($check, $transactionID, $transactionStatus);

    my $PaymentLineItemGUID = $FormValues->{'PaymentGUID'};
    Error('BadRedirect_NoPaymentGUID', {
        'Name'       => 'PaymentGUID',
        'Reason'     => 'BadRedirect_NoPaymentGUID',
    }) unless (defined $PaymentLineItemGUID && ExistsObjectByGUID($PaymentLineItemGUID));
    my $PaymentLineItem = LoadObjectByGUID($FormValues->{'PaymentGUID'});

    Error('BadRedirect_ShopIDMisMatch', {
        'Name'       => 'ShopID',
        'Reason'     => 'BadRedirect_ShopIDMisMatch'
    }) if ($PaymentLineItem->getSite()->id != $Shop->id );

    my $Container = $PaymentLineItem->get('LineItemContainer');
    my $GrandTotal = $Container->get('GrandTotal');
    my $CurrencyID = $Container->get('CurrencyID');

    if (defined($FormValues->{'amt'}) && defined($FormValues->{'cc'})) {
        if (fcmp($GrandTotal, $FormValues->{'amt'}) != 0 || $CurrencyID ne $FormValues->{'cc'}) {
            LogPayment('PayPalExpress', 'BadRedirect_AmountMisMatch for payment line item '
                             . $PaymentLineItem->id . ' (' . $PaymentLineItem->pathString .'): '
                             . "amount received=$FormValues->{'amt'} $FormValues->{'cc'}, "
                             . "expected=$GrandTotal $CurrencyID",
                             {});

            Error( 'BadRedirect_AmountMisMatch', {
                'Name'       => 'amount',
                'Reason'     => 'BadRedirect_AmountMisMatch'
            });
        } else {
            $check = 1;
            $transactionID = $FormValues->{'tx'};
            $transactionStatus = $FormValues->{'st'};
        }
    }

    if ((defined($FormValues->{'mc_gross'}) || defined($FormValues->{'mc_gross_1'})) && defined($FormValues->{'mc_currency'})) {
        $FormValues->{'mc_gross'} //= $FormValues->{'mc_gross_1'};
        if (fcmp($GrandTotal, $FormValues->{'mc_gross'}) != 0 || $CurrencyID ne $FormValues->{'mc_currency'}) {
            LogPayment('PayPalExpress', 'BadRedirect_AmountMisMatch for payment line item '
                             . $PaymentLineItem->id . ' (' . $PaymentLineItem->pathString .'): '
                             . "amount received=$FormValues->{'mc_gross'} $FormValues->{'mc_currency'}, "
                             . "expected=$GrandTotal $CurrencyID",
                             {});

            Error( 'BadRedirect_AmountMisMatch', {
                'Name'       => 'amount',
                'Reason'     => 'BadRedirect_AmountMisMatch',
            });
        } else {
            $check = 1;
            $transactionID = $FormValues->{'txn_id'};
            $transactionStatus = $FormValues->{'payment_status'};
        }
    }

    return ($check, $transactionID, $transactionStatus);
}

#==============================================================================
sub _SetTransStatus {
    my $self = shift;
    my ($Order, $TransStatus) = @_;

    my $now = GetCurrentDBHandle()->currentDateTime;
    given(lc($TransStatus)) {
        when ('pending') {
            $Order->set({'PendingOn' => $now});
        }
        when ('completed') {
            $Order->set({
                'PaidOn'    => $now,
                'PendingOn' => undef
            });
        }
        when ('denied') {
            $Order->set({'CancelledOn' => $now});
            _SendPayerNotification($Order);
        }
        when ('reversed') {
            # EPG-27322: remove PaidOn status if payment was reversed
            $Order->set({'PaidOn' => undef});
        }
    }
    return;
}

#==============================================================================
sub _SendPayerNotification {
    my ($Order) = @_;

    my $PaymentLineItem = $Order->container->get('Payment');
    return unless $PaymentLineItem;

    my $Recipient;
    my $BillingAddress = $Order->get('BillingAddress');
    $Recipient = $BillingAddress->get('EMail') if $BillingAddress;
    $Recipient ||= $PaymentLineItem->get('PayerEMail');
    return unless $Recipient;

    my $Shop = $PaymentLineItem->getSite();
    my $MailType = GetShopMailType( $Shop, 'PayPalExpressPaymentDeclined', 1);
    return unless $MailType;

    my %Vars = (
        'Alias'           => $Order->alias,
        'PaymentLineItem' => $PaymentLineItem->tleHash,
    );

    my %MailParams = (
        'HTMLAllowed' => 1,
        'TO'          => $Recipient,
    );
    $MailParams{'FROM'} = $Shop->get('MailFrom') unless $MailType->get('FROM');

    my $Locale = $Shop->get('LocaleID');
    my %RegionalParams = (
        'Locale'   => $Locale,
        'Language' => GetInfoLanguage( GetInfoLocale($Locale)->{'LanguageID'} )->{'Code2'},
    );
    $MailType->send( $Shop, \%MailParams, \%RegionalParams, \%Vars );
}

#========================================================================================
# function     TransactionPaymentName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Name = $Handler->TransactionPaymentName();
#----------------------------------------------------------------------------------------
# description  return PayPalExpress as string. Name of PaymentType for search/view
#                               transactions.
#----------------------------------------------------------------------------------------
# return       $Name | name of payment type | string
#========================================================================================
sub TransactionPaymentName { 'PayPalExpress' }

#========================================================================================
sub PayPalExpressGiroPaySuccess { # show order page
    my $self = shift;
    my ($Servlet) = @_;

    my $guid = $Servlet->params->{'ppeguid'};
    Error('NO_GUID') unless defined $guid;

    LogPayment('PayPalExpress', 'guid', $guid);
    Error('NO_PAYMENTLINEITEM') unless ExistsObjectByGUID($guid);
    my $pli = LoadObjectByGUID($guid);

    my $PaymentObject = $pli->container->parent;
    Error('NO_ORDER') unless $PaymentObject->instanceOf('CustomerOrder');

    # show order confirmation page
    $Servlet->vars('ObjectID', $PaymentObject->id);
    $Servlet->vars('ViewAction', 'View');

    return;
}

#========================================================================================
sub PayPalExpressGiroPayCancel {
    my $self = shift;
    my ($Servlet) = @_;

    my $guid = $Servlet->params->{'ppeguid'};
    Error('NO_GUID') unless defined $guid;

    LogPayment('PayPalExpress', 'guid', $guid);
    Error('NO_PAYMENTLINEITEM') unless ExistsObjectByGUID($guid);
    my $pli = LoadObjectByGUID($guid);

    my $PaymentObject = $pli->container->parent;
    $Servlet->vars('ObjectID', $PaymentObject->id);
    if ($PaymentObject->instanceOf('Basket')) { # show basket page again, but this should not happen
        $Servlet->vars('ViewAction', 'View');
    }
    else { # we have an order, show cancel page
        $Servlet->vars('ViewAction', 'ViewPayPalExpressCancel');
    }

    return;
}

1;
