use Test::More;
use strict;
use Time::HiRes qw( gettimeofday tv_interval );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Url qw( UrlEncodeValue );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath );
use DE_EPAGES::WebInterface::API::FakeRequest;

my $SHOP_ALIAS = 'DemoShop';
my $SHOPPATH = "/Shops/$SHOP_ALIAS";

sub LoginToshop {
    my ($FakeRequest, $Shop) = @_;

    my $ScriptURL = $Shop->get('WebServerScriptAdminUrlSSL');
    my $URL = $ScriptURL . '?' . join('&', (
        'ObjectPath='.UrlEncodeValue($Shop->pathString),
        'ChangeAction=SaveLoginForm',
        'Login=admin',
        'Password=admin'
    ));

    my $Response = $FakeRequest->get($URL);
    my $RC = $Response->code;
    is( $RC, 200, "shop login: $URL -> RC=$RC" );

    # grep URL with sec-Token inside from content (LWP::UserAgent would do this aswell ...)
    # i.e.: <base href="https://qa-vm-lin64-1.intern.epages.de/epages/Store.admin/de_DE/">
    if( $Response->content =~ /<base\s+href=(["'])(.*?)\1/x ) { # '"
        my $BaseURL = $2;
        like( $BaseURL, qr/^\Q$ScriptURL\E\/sec/, "session URL: $BaseURL");
        return $BaseURL;
    }
    else {
        die "base href=\"...\": not found in content\n";
    }
}

sub DoSmartSearch {
    my ($FakeRequest, $Shop, $SessionURL, $SearchString) = @_;

    my $UrlParams = join('&', (
        'ViewAction=JSONSmartSearchProductAlias',
        'ObjectID='.$Shop->id,
        'SearchString='.$SearchString
    ));

    my $Response = $FakeRequest->get(
        $SessionURL . '?' . $UrlParams,
        [
            'content-type' => 'application/x-www-form-urlencoded; charset=UTF-8',
            'Accept' => 'text/x-json',
        ]
    );

    my $RC = $Response->code;
    my $CT = $Response->header('content-type');
    return ($RC, $CT);
}

sub TestProductSmartSearch {
    my ($FakeRequest, $Shop, $SessionURL) = @_;
    foreach my $SearchString ( qw(be_)) {
        my $t0= [gettimeofday];
        my ($RC, $CT) = DoSmartSearch($FakeRequest, $Shop, $SessionURL, $SearchString);
        ok(($RC == 200 and $CT =~ /\bjson\b/), "search '$SearchString': RC=$RC Content-Type=$CT");
        my $t1= [gettimeofday];
        my $elapsed = tv_interval ($t0, $t1);
      SKIP: {
          skip( 'Forced skip of timing tests.', 1 ) if $ENV{'SKIP_TIMING'};
          cmp_ok( $elapsed, '<', 0.2, sprintf('duration of 1 search: %0.2fs (< 0.2s)', $elapsed) );
        }
    }
}

sub MassTestProductSmartSearch {
    my ($FakeRequest, $Shop, $SessionURL) = @_;

    my @SearchQueryPool = qw(be be_ be_4040 be_40401 be_404010100 be_40402 cg cg_ js js_32 md_49417110 md_494171101);

    my $t0= [gettimeofday];
#DB::enable_profile();
    my ($RC, $CT);
    for (0..200) {
        ($RC, $CT) = DoSmartSearch($FakeRequest, $Shop, $SessionURL, $SearchQueryPool[rand @SearchQueryPool]);
        last if $RC != 200 or $CT !~ /\bjson\b/;
    }
    ok(($RC == 200 and $CT =~ /\bjson\b/), "search '".join(' ', @SearchQueryPool).
       "': RC=$RC Content-Type=$CT");
#DB::disable_profile();
    my $t1= [gettimeofday];
    my $elapsed = tv_interval ($t0, $t1);
  SKIP: {
      skip( 'Forced skip of timing tests.', 1 ) if $ENV{'SKIP_TIMING'};
      cmp_ok( $elapsed, '<', 10, sprintf('duration of 200 searches: %0.2fs (< 10s)', $elapsed) );
    }
}

sub Main {

    if( ExistsObjectByPath($SHOPPATH) ) {
        my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;

        my $Shop = LoadObjectByPath($SHOPPATH);
        ok( $Shop->id, 'load demoshop: ' . $Shop->id );
        my $SessionURL = LoginToshop($FakeRequest, $Shop);

        # user clicks on product list and clicks on a product
        LoadObjectByPath($SHOPPATH)->child('Products');
        LoadObjectByPath($SHOPPATH)->child('Products')->child('be_40401');

        TestProductSmartSearch($FakeRequest, $Shop, $SessionURL);
        MassTestProductSmartSearch($FakeRequest, $Shop, $SessionURL);
    } else {
      SKIP: {
          skip 'no DemoShop installed', 8;
        };
    }
    return;
}


RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});

done_testing();
