#========================================================================================
# package      DE_EPAGES::Saferpay2::UI::PaymentMethodSaferpay2
# base         DE_EPAGES::Payment::UI::PaymentMethod
# state        private
#----------------------------------------------------------------------------------------
# description  UI Controller for payment method
#========================================================================================
package DE_EPAGES::Saferpay2::UI::PaymentMethodSaferpay2;
use base DE_EPAGES::ExternalPayment::UI::PaymentMethodExternalPayment;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath Transaction );
use DE_EPAGES::Payment::API::Table::ShippingPaymentMap qw ( InsertShippingPaymentMap );
use DE_EPAGES::Payment::API::Table::ShopPaymentLogo qw ( ExistsShopPaymentLogo InsertShopPaymentLogo );
use DE_EPAGES::Payment::API::Shop qw ( GetShopPaymentLogos );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );

use DE_EPAGES::Saferpay2::API::Constants qw (
    @SPLIT_GATEWAY_METHODS
    $SPLIT_GATEWAY_METHODS_MAXROWS
    PAYMENT_TYPE_SAFERPAY2
);
use DE_EPAGES::Saferpay2::API::SaferpayService;
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use constant ERROR_TRANSACTION_NOT_FOUND => 'TRANSACTION_NOT_FOUND';

use strict;


#========================================================================================
# function     ViewSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller to view settings on the MBO settings page.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub ViewSettings {
    my $self = shift;
    my ($Servlet) = @_;

    $self->SUPER::View($Servlet);
    $self->ViewSettingsMultiPayment($Servlet, 'PaymentMethodSaferpay2', 'SplitGatewayMethod');

    return;
}



#========================================================================================
# function     SaveSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       SaveSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller to save settings on the MBO settings page.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub SaveSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();
    my $Form = $Servlet->form();
    my $SplitGatewayMethod = $Form->formValues()->{'SplitGatewayMethod'}[0];
    my $Shop = $PaymentMethod->getSite();
    my $Folder = $Shop->folder('PaymentMethods');


    my $hFormValues = $Form->form($PaymentMethod, 'Saferpay2Settings');
    $Servlet->vars('GetMerchantStatusError', 0);
    unless ($self->_checkCredentials($hFormValues)) {
        $Servlet->vars('GetMerchantStatusError', 1);
        $Form->addFormError({
            'Form' => 'Saferpay2Settings',
            'Reason' => 'CredentialsValidationError',
            'Name' => 'CredentialsValidationError' });
        $Form->executeFormError();
        return;
    }

    if (isValid($SplitGatewayMethod)) {
        my $aSplityGatewayMethod = $self->GetSplitGatewayMethods($Servlet);
        my %hSPLIT_GATEWAY_METHODS = map { ($_->{'ID'}) => $_ } @$aSplityGatewayMethod;
        my $aliasMethod = $hSPLIT_GATEWAY_METHODS{$SplitGatewayMethod}->{'Name'};
        my $LogoName = $hSPLIT_GATEWAY_METHODS{$SplitGatewayMethod}->{'LogoName'};

        if (not isValid($aliasMethod)) {
            $Form->addFormError({
                'Form'  => 'PaymentMethodSaferpay2',
                'Reason'=> 'OBJECT_EXISTS',
                'Name'  => $SplitGatewayMethod
            });
            $Form->executeFormError;
        }

        # Rewrite the name and the alias of the payment method
        $PaymentMethod->set({
            'Name' => $aliasMethod,
            'Alias' => $self->_getUniqueAliasForPayment($Folder, $aliasMethod)
        });

        $PaymentMethod->SetNameForAllLanguages($aliasMethod);

        # Set Method Logo
        my $hLogoData = $self->_getPaymentLogoData($Shop, $LogoName);
        if (isValid($hLogoData->{'PaymentLogoPath'})) {
            $PaymentMethod->set({ 'InternalImagePath' => $hLogoData->{'PaymentLogoPath'} });
        }
    }
    $self->Save($Servlet);
}

sub _checkCredentials {
    my $self = shift;
    my ($hFormValues) = @_;

    my $hValues = {
        MerchantName => $hFormValues->{'MerchantName'},
        MerchantPassword => $hFormValues->{'MerchantPassword'},
        CustomerId => $hFormValues->{'CustomerId'},
        TerminalId => $hFormValues->{'TerminalId'},
        IsInTestMode => $hFormValues->{'IsInTestMode'},
    };

    my $SaferpayService = DE_EPAGES::Saferpay2::API::SaferpayService->new(undef, $hValues);
    $self->_LogPayment('DoAssertCall Request to validate the credentials.', { 'MerchantName' => $hFormValues->{'MerchantName'} });
    my $hResponse = $SaferpayService->DoAssertCall("ValidateCredentialsInvalidToken");
    my $CredentialsChecked = (isValid($hResponse->{'ErrorName'}) &&
        $hResponse->{'ErrorName'} eq ERROR_TRANSACTION_NOT_FOUND) ? 1 : 0;

    return $CredentialsChecked;
}


sub _LogPayment {
    my $self = shift;
    my ($Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment(PAYMENT_TYPE_SAFERPAY2, $Name, $DebugData, $Section, $Filter);
}
#========================================================================================
# function     GetSplitGatewayMethods
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aSplitGatewayMethods = GetSplitGatewayMethods( );
#----------------------------------------------------------------------------------------
# description  It provides the multi-payment structure of grouped payment methods.
#----------------------------------------------------------------------------------------
# return       $aSplitGatewayMethods | hash with split gateway methods | ref.array
#========================================================================================
sub GetSplitGatewayMethods {
    my $self = shift;
    my ($Servlet) = @_;

    my @SplitGatewayMethodsWithTranslations = @SPLIT_GATEWAY_METHODS;
    my $LanguageID = $Servlet->params('LanguageID');
    foreach my $hSplitGatewayMethod (@SplitGatewayMethodsWithTranslations) {
        if (!$hSplitGatewayMethod->{'isGroup'}) {
            my $PaymentTypeMethod = LoadObjectByPath('/PaymentTypes/Saferpay2/PaymentTypeMethods/SaferpayPaymentGroup/' . $hSplitGatewayMethod->{'ID'});
            $hSplitGatewayMethod->{'Translated_Name'} = $PaymentTypeMethod->get('Name', $LanguageID) // $hSplitGatewayMethod->{'ID'};
        }
    }
    return \@SplitGatewayMethodsWithTranslations;
}

sub GetSplitGatewayMethodsMaxRow {
    my $self = shift;

    return $SPLIT_GATEWAY_METHODS_MAXROWS;
}

#========================================================================================
# function     GetSplitGatewayMethodsDialogOneColumn
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DialogOneColumn = $self->GetSplitGatewayMethodsDialogOneColumn( );
#----------------------------------------------------------------------------------------
# description  Provides the dialog columns formatting. When true the dialog is shown
#               in one column, when false on four columns
#----------------------------------------------------------------------------------------
# return       $DialogOneColumn | If true dialog one column, false four columns | boolean
#========================================================================================
sub GetSplitGatewayMethodsDialogOneColumn {
    my $self = shift;

    return 1;
}


#==============================================================================
sub _checkNewPaymentMethods{
    my $self = shift;
    my ($Servlet, $PaymentMethod, $aSelectedSplitGatewayMethods, $hSplitGatewyMethods) = @_;

    my $Shop = $PaymentMethod->getSite();
    my $Form = $Servlet->form();
    my $ExecuteError = 0;

    if (not scalar @$aSelectedSplitGatewayMethods) {
        $Form->addFormError({
            'Form'  => 'PaymentMethod',
            'Reason'=> 'NOTHING_SELECTED',
            'Name'  => 'NOTHING_SELECTED'
        });
    }

    my $LeftPaymentMethods = $Shop->featureCurrentValue('PaymentMethods') + scalar(@$aSelectedSplitGatewayMethods) - $Shop->featureMaxValue('PaymentMethods');
    if ( $LeftPaymentMethods >= scalar(@$aSelectedSplitGatewayMethods) ){
        $ExecuteError = 1;
        $Form->addFormError({
            'Form'  => 'PaymentMethod',
            'Reason'=> 'FEATURE_LIMIT_EXCEEDED',
            'Name'  => 'FEATURE_LIMIT_EXCEEDED'
        });
    }
    if ( $ExecuteError == 1 ){
        $Form->executeFormError();
    }

    return $LeftPaymentMethods;
}


#========================================================================================
# function     NewPaymentMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       NewPaymentMethod( $Servlet, $PaymentMethod, $SplitGatewayMethod, $MethodName, $LogoName );
#----------------------------------------------------------------------------------------
# description  It creates a new payment method.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
# input        $PaymentMethod | payment method object | object
# input        $SplitGatewayMethod | payment method object | object
# input        $MethodName | method name | string
# input        $LogoName | logo name | string
# return       $NewPaymentMethod | payment method | object
#========================================================================================
sub NewPaymentMethod {
    my $self = shift;
    my ( $Servlet, $PaymentMethod, $SplitGatewayMethod, $MethodName, $LogoName ) = @_;

    my $Shop = $PaymentMethod->getSite();
    my $PaymentType = LoadObjectByPath('/PaymentTypes/Saferpay2');

    return unless defined $SplitGatewayMethod && $SplitGatewayMethod =~ /\S/;

    my $hLogoData = $self->_getPaymentLogoData($Shop, $LogoName);

    my $NewPaymentMethod;
    eval {
        Transaction( sub {
            my $hCommonCredentials = $PaymentMethod->get([ qw(MerchantName MerchantPassword CustomerId TerminalId IsInTestMode) ]);
            $NewPaymentMethod = $PaymentType->newPaymentMethod($Shop, {
                'MerchantName'       => $hCommonCredentials->{'MerchantName'},
                'MerchantPassword'   => $hCommonCredentials->{'MerchantPassword'},
                'CustomerId'         => $hCommonCredentials->{'CustomerId'},
                'TerminalId'         => $hCommonCredentials->{'TerminalId'},
                'SplitGatewayMethod' => $SplitGatewayMethod,
                'Alias'              => $MethodName,
                'Position'           => 9999,
                'Shop'               => $Shop,
                'IsInTestMode'       => $hCommonCredentials->{'IsInTestMode'},
            });

            # activate on all shipping methods
            InsertShippingPaymentMap({
                'ShippingMethodID'  => $_->id,
                'PaymentMethodID'   => $NewPaymentMethod->id()
            }) foreach @{$Shop->get('ShippingMethods')};

            # insert new Logos
            if ($hLogoData->{'PaymentTypeLogoID'}) {
                InsertShopPaymentLogo({
                    'ShopID'            => $Shop->id,
                    'PaymentTypeLogoID' => $hLogoData->{'PaymentTypeLogoID'},
                }) unless ExistsShopPaymentLogo({
                    'ShopID'            => $Shop->id,
                    'PaymentTypeLogoID' => $hLogoData->{'PaymentTypeLogoID'},
                });
            }

            if ($hLogoData->{'PaymentLogoPath'}) {
                $NewPaymentMethod->set({ 'InternalImagePath' => $hLogoData->{'PaymentLogoPath'} });
            }
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'FeatureLimitExceededGeneral') {
            $Servlet->vars('FeatureMaxValueReachedGeneral',1);
        }
        elsif ($Error->code  eq 'FeatureLimitExceeded'){
            $Servlet->vars('FeatureMaxValueReached', $Error->vars);
        }
        else {
            Error($Error);
        }
    }

    return $NewPaymentMethod;
}


#========================================================================================
# function     NewMultiPaymentMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       NewMultiPaymentMethod( $Servlet );
#----------------------------------------------------------------------------------------
# description  Payment method replication with a different method.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub NewMultiPaymentMethod {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;
    my $PaymentMethod = $Servlet->object();
    my $Values = $Servlet->form->formValues();

    my $aSplitGatewayMethods = $self->GetSplitGatewayMethods($Servlet);

    # Go through the payment methods array and generate a hash with key = ID and value the payment method.
    my %hSplitGatewayMethods = map { ($_->{'ID'}) => $_ } @$aSplitGatewayMethods;

    # We build an array with the selected methods in the settings page
    my @aSelectedSplitGatewayMethods = map { /^aMultiPaymentMethod_(.*)/x } grep { /^aMultiPaymentMethod_/x } keys %$Values;

    # We check the selected methods
    my $LeftPaymentMethods = $self->_checkNewPaymentMethods($Servlet, $PaymentMethod, \@aSelectedSplitGatewayMethods, \%hSplitGatewayMethods);

    if ( $LeftPaymentMethods > 0 ) {
        splice(@aSelectedSplitGatewayMethods, -1 * $LeftPaymentMethods);
        $Form->addFormError({
            'Form'  => 'PaymentMethod',
            'Reason'=> 'FEATURE_LIMIT_EXCEEDED',
            'Name'  => 'FEATURE_LIMIT_EXCEEDED'
        });
    }

    # We create the diferent selected payment method and add them to an array
    my @newPaymentMethods;
    my $Shop = $PaymentMethod->getSite();
    my $Folder = $Shop->folder('PaymentMethods');
    my $aliasMethod;

    foreach my $SplitGatewayMethod (@aSelectedSplitGatewayMethods) {
        #Check if the alias for payment method already exists, before rewriting
        $aliasMethod = ($hSplitGatewayMethods{$SplitGatewayMethod}->{'UIName'} // $hSplitGatewayMethods{$SplitGatewayMethod}->{'Name'});
        my $NewPaymentMethod = $self->NewPaymentMethod(
            $Servlet,
            $PaymentMethod,
            $SplitGatewayMethod,
            $self->_getUniqueAliasForPayment($Folder, $aliasMethod),
            $hSplitGatewayMethods{$SplitGatewayMethod}->{'LogoName'}
        );
        $NewPaymentMethod->SetNameForAllLanguages($SplitGatewayMethod);
        push(@newPaymentMethods, $NewPaymentMethod);
    }

    $PaymentMethod->getSite()->child('PaymentMethods')->renumberChildren;
    $Servlet->vars('NewPaymentMethods',\@newPaymentMethods);
    $Form->executeFormError();

    return;
}

#==============================================================================
sub _getUniqueAliasForPayment{
    my $self = shift;
    my ($Folder, $aliasMethod) = @_;

    if ($Folder->existsChild($aliasMethod)) {
        my $i = 1;
        my $newAliasMethod = $aliasMethod . '_' .  $i;
        while ($Folder->existsChild($newAliasMethod)) {
            $newAliasMethod =  $aliasMethod . '_' .  ++$i;
        }
        $aliasMethod = $newAliasMethod;
    }
    return $aliasMethod;
}


sub _getPaymentLogoData {
    my $self = shift;
    my ($Shop, $LogoName) = @_;

    my $aLogos = GetShopPaymentLogos($Shop, LoadObjectByPath('/PaymentTypes/Saferpay2'));
    my $hLogoData = {};
    foreach my $hLogo (@$aLogos) {
        if ($LogoName eq $hLogo->{'PaymentLogo'}->alias()) {
            $hLogoData->{'PaymentTypeLogoID'} = $hLogo->{'PaymentTypeLogo'}->id;
            $hLogoData->{'PaymentLogoPath'} = $hLogo->{'PaymentLogo'}->get('Image');
        }
    }

    return $hLogoData;
}

1;