#========================================================================================
# package      DE_EPAGES::ShopMigration::API::CategoryProductMapsCommon
# state        public
#----------------------------------------------------------------------------------------
# description  common functions for exporting/importing category product maps
#========================================================================================
package DE_EPAGES::ShopMigration::API::CategoryProductMapsCommon;
use base qw(Exporter);

use strict;
our @EXPORT_OK = qw(
    GetCategoryProductMaps
    SetCategoryProductMaps
    GetCategoryProductMapData
    SetCategoryProductMapData
    GetCategoryProductMapIDs
);

use DE_EPAGES::Core::API::TestParameter qw(TestString);
use DE_EPAGES::ShopMigration::API::Constants qw(CATEGORYPRODUCTMAPS_ATTRIBUTES CATEGORYPRODUCTMAPS_OBJECTATTRIBUTES);
use DE_EPAGES::Core::API::Error qw(Error);
use DE_EPAGES::Object::API::Factory qw(LoadClassByAlias LoadObject);

#========================================================================================
# function     GetCategoryProductMaps
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ahCategoryProductMaps = GetCategoryProductMaps($Shop);
#----------------------------------------------------------------------------------------
# description  Fetches all categoryproductmaps
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
#----------------------------------------------------------------------------------------
# return       $ahCategoryProductMaps | see GetCategoryProductMapData() | reference to array of hashes
#========================================================================================
sub GetCategoryProductMaps {
    my ($Shop) = @_;

    my $Class = LoadClassByAlias('CategoryProductMap');
    my @Items;
    my $aCategoryProductMapIDs = GetCategoryProductMapIDs($Shop);
    foreach my $CategoryProductMapID (@$aCategoryProductMapIDs) {
        my $CategoryProductMap = $Class->loadObject($CategoryProductMapID);
        my $hInfo = GetCategoryProductMapData($Shop, $CategoryProductMap);
        push @Items, $hInfo if (keys %$hInfo);
    }
    return \@Items;
}

#========================================================================================
# function     GetCategoryProductMapIDs
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ahCategoryProductMaps = GetCategoryProductMapIDs($Shop);
#----------------------------------------------------------------------------------------
# description  Fetches all categoryproductmaps as a list of IDs
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
#----------------------------------------------------------------------------------------
# return       $aItems | list of CategoryProductMap object IDs | ref array integer
#========================================================================================
sub GetCategoryProductMapIDs {
    my ($Shop) = @_;
    my @Items;
    my $CategoryClass = LoadClassByAlias('Category');
    my $BaseCategory = $Shop->get('Categories');
    _categoryProductMaps($CategoryClass, $BaseCategory, \@Items);
    return \@Items;
}

#========================================================================================
# function     _categoryProductMaps
# state        private
#----------------------------------------------------------------------------------------
# syntax       _categoryProductMaps($CategoryClass, $BaseCategory, \@Items;
#----------------------------------------------------------------------------------------
# description  Fetches recursively the categoryproductmaps ids for all categories
#----------------------------------------------------------------------------------------
# input        $CategoryClass | class of category | object
# input        $Category | starting category for recursion | object
# input        $aItems | list of CategoryProductMap object ids | ref array integer
#========================================================================================
sub _categoryProductMaps {
    my ($CategoryClass, $Category, $aItems) = @_;

    push @$aItems, map {$_->id} @{$Category->get('CategoryProductMaps')};
    my $aChildObjectIDs = $Category->findChildren;
    foreach my $ChildID (@$aChildObjectIDs) {
        my $ChildCategory = LoadObject($ChildID);
        _categoryProductMaps($CategoryClass, $ChildCategory, $aItems) if $ChildCategory->instanceOf($CategoryClass);
    }
    return;
}

#========================================================================================
# function     GetCategoryProductMapData
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $hCategoryProductMap = GetCategoryProductMapData($Shop, $CategoryProductMap);
#----------------------------------------------------------------------------------------
# description  Constructs a hash with the exportable fields for a single CategoryProductMap
#----------------------------------------------------------------------------------------
# input        $Shop | shop | shop object
# input        $CategoryProductMap | categoryproductmap | object
#----------------------------------------------------------------------------------------
# return       $hCategoryProductMap | hash with following fields
#               <ul>
#                   <li>Attributes
#                       <ul>
#                           <li>Category                        - string (path of category)</li>
#                           <li>Product                         - string (path of product)</li>
#                           <li>Position                        - integer</li>
#                           <li>PositionCategory                - integer</li>
#                       </ul>
#                   </li>
#               </ul>| reference to hash
#========================================================================================
sub GetCategoryProductMapData {
    my ($Shop, $CategoryProductMap) = @_;

    my $Attributes = [ @{CATEGORYPRODUCTMAPS_ATTRIBUTES()} ];
    my $ObjectAttributes = [ @{CATEGORYPRODUCTMAPS_OBJECTATTRIBUTES()} ];
    my $hValues = { Attributes => $CategoryProductMap->get($Attributes) };
    foreach my $AttributeName (@$ObjectAttributes) {
        my $Value = $hValues->{Attributes}{$AttributeName};
        next unless defined $Value;
        $hValues->{Attributes}{$AttributeName} = $Shop->relativePath($Value);
    }
    return $hValues;
}

#========================================================================================
# function     SetCategoryProductMaps
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetCategoryProductMaps($Shop, $ahCategoryProductMaps);
#----------------------------------------------------------------------------------------
# description  sets attributes on CategoryProductMaps as specified in the $ahCategoryProductMaps
#----------------------------------------------------------------------------------------
# input        $Shop | shop | shop object
# input        $ahCategoryProductMaps | see GetCategoryProductMaps() | reference to array of hashes
#========================================================================================
sub SetCategoryProductMaps {
    my ($Shop, $ahCategoryProductMaps) = @_;

    foreach my $hCategoryProductMap (@$ahCategoryProductMaps) {
        SetCategoryProductMapData($Shop, $hCategoryProductMap);
    }
}

#========================================================================================
# function     SetCategoryProductMapData
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetCategoryProductMapData($Shop, $hCategoryProductMap);
#----------------------------------------------------------------------------------------
# description  inserts or sets attributes on a CategoryProductMap as specified in the hash $hCategoryProductMap
#----------------------------------------------------------------------------------------
# input        $Shop | shop | shop object
# input        $hCategoryProductMap | see GetCategoryProductMaps() | reference to array of hashes
# return       $CategoryProductMap | updated/inserted category product map | object
#========================================================================================
sub SetCategoryProductMapData {
    my ($Shop, $hCategoryProductMap) = @_;

    _analyzeAttributesHash($Shop, $hCategoryProductMap);
    my $CategoryProductMap = $hCategoryProductMap->{'CategoryProductMap'};
    if (defined $CategoryProductMap) {
        $CategoryProductMap->set($hCategoryProductMap->{'Attributes'});
    } else {
        $hCategoryProductMap->{'CategoryProductMapClass'}->insertObject($hCategoryProductMap->{'Attributes'});
    }
    return;
}

#========================================================================================
# function     _analyzeAttributesHash
# state        private
#----------------------------------------------------------------------------------------
# syntax       _analyzeAttributesHash($hLineData);
#----------------------------------------------------------------------------------------
# description  Analyzes the fields in the parsed $hLineData
#               converts pathes to objects
#----------------------------------------------------------------------------------------
# input        $Shop | shop | shop object
# input        $hLineData | Hash with Values | ref hash string
#========================================================================================
sub _analyzeAttributesHash {
    my ($Shop, $hLineData) = @_;

    my ($Product, $Category);
    my $ProductPath = $hLineData->{'Attributes'}{'Product'};
    my $CategoryPath = $hLineData->{'Attributes'}{'Category'};;
    $ProductPath =~ s/\s+$|^\s+//g;
    if ($ProductPath eq '') {
        Error('CATALOG_EMPTY_PRODUCT', { 'Attribute' => 'Product', 'Value' => $ProductPath, 'Category' => $CategoryPath });
    } else {
        $Product = _loadShopObject($Shop, $ProductPath, 'Product');
        $hLineData->{'Attributes'}{'Product'} = $Product;
    }

    # it is not tested at MBO, but at CSV import ?
    ## test if we have unity and try to set an own category to a subproduct
    #if($Shop->get('IsUnityRegistered') and $Product->get('SuperProduct')){
    #    $DoImport = 0;
    #    return;
    #}
    $CategoryPath =~ s/\s+$|^\s+//g;
    if ($CategoryPath eq '') {
        Error('CATALOG_EMPTY_CATEGORY', { 'Attribute' => 'Category', 'Value' => $CategoryPath, 'Product' => $ProductPath });
    } else {
        $Category = _loadShopObject($Shop, $CategoryPath, 'Category');
        $hLineData->{'Attributes'}{'Category'} = $Category;
    }

    my $Class = LoadClassByAlias('CategoryProductMap');
    # exists category product map?
    my $aMaps = $Class->getObjects('ByCategoryIDProductID', [ $Category->id, $Product->id ]);
    my $CategoryProductMap = scalar @$aMaps ? $aMaps->[0] : undef;

    $hLineData->{'CategoryProductMapClass'} = $Class;
    $hLineData->{'CategoryProductMap'} = $CategoryProductMap;
    return;
}

#========================================================================================
# function     _loadShopObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = _loadShopObject( $ParentObject, $Path, $ClassAlias )
#----------------------------------------------------------------------------------------
# description  loads the object specified by object path and verifies that it exists,
#               belongs to the given parent object and is an instance of a certain class.
#               Returns undef if path doesn't exist or object doesn't belong to the same shop.
#               Returns the object if all tests passed.
#----------------------------------------------------------------------------------------
# input        $ParentObject | parent object | shop object
# input        $Path | object path to load | string
# input        $ClassAlias | alias of a class (optional) | string
# return       $Object | object | object
#========================================================================================
sub _loadShopObject {
    my ($ParentObject, $Path, $ClassAlias) = @_;

    TestString('Path', $Path);
    my $ObjectID = $ParentObject->findPath($Path);
    Error('UNKNOWN_OBJECT' => {
        'Value' => $Path,
    }) unless defined $ObjectID;

    # get object
    my $ObjectByPath = LoadObject($ObjectID);
    # check if object belongs to the current shop
    Error('FORBIDDEN_OBJECT' => {
        'Value' => $Path,
        'Class' => $ClassAlias,
    }) if (!_checkShopObject($ParentObject->getSite(), $ObjectByPath, $ClassAlias));
    return $ObjectByPath;
}

#========================================================================================
# function     _checkShopObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       _checkShopObject( $Shop, $Object, $ClassAlias )
#----------------------------------------------------------------------------------------
# description  Checks if an object belongs to a site (shop) and
#               is instance of a certain class. Returns 0 if not.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | shop object
# input        $Object | test object | object
# input        $ClassAlias | alias of a class (optional) | string
# return       $OK | true if check was ok | boolean
#========================================================================================
sub _checkShopObject {
    my ($Shop, $Object, $ClassAlias) = @_;

    if ($Object->getSite()->id != $Shop->id) {
        return 0;
    }
    if ($ClassAlias and not $Object->instanceOf($ClassAlias)) {
        return 0;
    }
    return 1;
}

1;
