#========================================================================================
# package      DE_EPAGES::ShopMigration::API::HTMLSanitizer
# state        public
#----------------------------------------------------------------------------------------
# description  sanitizes a given chunk of HTML for the scribe editor the same way
#               the javascript snippet in the MBO does
#               (see DE_EPAGES/JQuery/Data/Public/lib/scribe-loader.js)
#
#               You can either use the functional interface by calling:
#                 my $HTML = SanitizeHTMLForScribe($HTML);
#               or the OO interface by using an intermediate object:
#                 my $Sanitizer = DE_EPAGES::ShopMigration::API::HTMLSanitizer->new();
#                 my $HTML = $Sanitizer->sanitize($HTML);
#========================================================================================
package DE_EPAGES::ShopMigration::API::HTMLSanitizer;
use base qw(Exporter);

use 5.010;
use strict;
use DE_EPAGES::Core::API::TestParameter qw(TestString);
use HTML::Restrict;

our @EXPORT_OK = qw(
    SanitizeHTMLForScribe
);

#========================================================================================
# function     SanitizeHTMLForScribe
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $HTML = SanitizeHTMLForScribe($Text);
#----------------------------------------------------------------------------------------
# description  strips unwanted HTML tags and attributes from the string $Text
#----------------------------------------------------------------------------------------
# input        $Text | a chunk of HTML | string
# return       $HTML | a sanitized chunk of HTML | string
#========================================================================================
sub SanitizeHTMLForScribe {
    my ($Text) = @_;

    state $Processor = HTML::Restrict->new(rules => _getScribeFilterRules());
    return fixLineBreakTags($Processor->process($Text));
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Sanitizer = DE_EPAGES::ShopMigration::API::HTMLSanitizer->new( %options )
#----------------------------------------------------------------------------------------
# description  Creates a new HTMLSanitizer object
#----------------------------------------------------------------------------------------
# input        %options | (optional) parameters for the  $Sanitizer object | hash
# return       $Sanitizer | HTMLSanitizer object | object
#========================================================================================
sub new {
    my $class = shift;
    my %options = @_;

    $options{Processor} = HTML::Restrict->new(rules => _getScribeFilterRules());
    return bless {
        %options,
    }, $class;
}

#========================================================================================
# function     sanitze
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $HTML = $Sanitizer->sanitze( $Text )
#----------------------------------------------------------------------------------------
# description  strips unwanted HTML tags and attributes from the string $Text
#----------------------------------------------------------------------------------------
# input        $Text | a chunk of HTML | string
# return       $HTML | a sanitized chunk of HTML | string
#========================================================================================
sub sanitze {
    my $self = shift;
    my ($Text) = @_;
    TestString('Text', $Text);

    my $Processor = $self->{Processor};
    return fixLineBreakTags($Processor->process($Text));
}

# HTML::Restrict turns the better readable <br /> into <br> which we reverse here
sub fixLineBreakTags {
    my ($Text) = @_;
    $Text =~ s,<br>,<br />,g if defined $Text;
    return $Text;
}

#========================================================================================
# function     _getScribeFilterRules
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getScribeFilterRules( $Response );
#----------------------------------------------------------------------------------------
# description  returns the filtering rules for the sanitizer
#----------------------------------------------------------------------------------------
# return       $Result | Hash of filter rules | ref_hash
#========================================================================================
sub _getScribeFilterRules {
    return {
        p           => [ { style => qr{^text-align:[^;]+;$} }, qw( align ) ],
        h1          => [ { style => qr{^text-align:[^;]+;$} }, qw( align ) ],
        h2          => [ { style => qr{^text-align:[^;]+;$} }, qw( align ) ],
        h3          => [ { style => qr{^text-align:[^;]+;$} }, qw( align ) ],
        h3          => [ { style => qr{^text-align:[^;]+;$} }, qw( align ) ],
        ul          => [ { style => qr{^text-align:[^;]+;$} }, qw( align ) ],
        ol          => [ { style => qr{^text-align:[^;]+;$} }, qw( align ) ],
        li          => [ { style => qr{^text-align:[^;]+;$} }, qw( align ) ],
        blockquote  => [ { style => qr{^text-align:[^;]+;$} }, qw( align ) ],
        span        => [ { style => qr{^color:[^;]+;$} }, ],
        strong      => [],
        b           => [],
        em          => [],
        i           => [],
        u           => [],
        a           => [ qw( href ) ],
        br          => [],
    }
}

1;