#========================================================================================
# package       DE_EPAGES::ShopMigration::API::JSON
# state         public
# base
#----------------------------------------------------------------------------------------
# description  Provides methods for creating and parsing JSON files.
#               The EncodeJSONArray() and DecodeJSONArray() functions are meant
#               to output and parse JSONLines (see http://jsonlines.org/)
#               The DecodeJSON and EncodeJSON are just wrapper functions for
#               the standard JSON package.
#               All function arguments and return values are
#               assumed to be native UTF-8 perl strings.
#----------------------------------------------------------------------------------------
# example      $hValues = DecodeJSON( '{"key1":"value1", "key2":"value"}' );
# example      $String  = EncodeJSON( { key1 => "value1", key2 => "value2" }, { 'pretty' => '1' } );
# example      my $JsonAPI = DE_EPAGES::ShopMigration::API::JSON->new;
#               my $fh = FileHandle->new;
#               $fh->open( $FileName, '<:encoding(utf-8)' ) or die "Can't read $FileName: $!";
#               while( my $ahFields = $JsonAPI->getline($fh) ) {
#                   # process record
#               }
#========================================================================================
package DE_EPAGES::ShopMigration::API::JSON;
use base qw(Exporter);

use strict;
use Encode qw(decode_utf8);
use IO::Scalar;
use JSON;

use DE_EPAGES::Core::API::TestParameter qw(TestArray TestHash);
use DE_EPAGES::Core::API::Error qw(Error ExistsError GetError);

our @EXPORT_OK = qw(
    DecodeJSON
    EncodeJSON
    DecodeJSONArray
    EncodeJSONArray
);

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'GET_LINE_FAILED' => '#Message',
    }
}
#ERRORDEFINITION

#========================================================================================
# function     DecodeJSON
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahFields = DecodeJSON( $Line, $hOptions );
# syntax       $ahFields = DecodeJSON( $Line );
#----------------------------------------------------------------------------------------
# description  Decodes one JSON formatted line and returns the parsed fields.
#----------------------------------------------------------------------------------------
# input        $Line | JSON formatted string | string
# input        $hOptions | see <function new> | ref.hash.string
# return       $ahFields | parsed fields | ref.hash.string
#========================================================================================
sub DecodeJSON($;$) {
    my ($Line, $hOptions) = @_;
    $hOptions = {} unless defined $hOptions;
    $Line = '' unless defined $Line;
    TestHash('hOptions', $hOptions);

    return __PACKAGE__->new($hOptions)->decode($Line);
}

#========================================================================================
# function     EncodeJSON
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Line = EncodeJSON( $ahFields, $hOptions );
# syntax       $Line = EncodeJSON( $ahFields );
#----------------------------------------------------------------------------------------
# description  encodes a hash of fields to one JSON formatted line.
#----------------------------------------------------------------------------------------
# input        $ahFields | hash with fields | ref.hash.string
# input        $hOptions | (optional) see <function new> | ref.hash.string
# return       $Line | JSON formatted string | string
#========================================================================================
sub EncodeJSON($;$) {
    my ($ahFields, $hOptions) = @_;
    $hOptions = {} unless defined $hOptions;
    TestHash('ahFields', $ahFields);
    TestHash('hOptions', $hOptions);

    return __PACKAGE__->new($hOptions)->encode($ahFields);
}


#========================================================================================
# function     DecodeJSONArray
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahFields = DecodeJSONArray( $Content, $hOptions );
# syntax       $ahFields = DecodeJSONArray( $Content );
# example      $ahFields = DecodeJSONArray( "{"Key":"Value1"}\n{"Key":"Value2"}\n" );
#----------------------------------------------------------------------------------------
# description  Decodes a list of json strings into an list of hashes
#----------------------------------------------------------------------------------------
# input        $Content | JSON fragments separated by newlines | string
# input        $hOptions | (optional) see <function new>.
#                           The expected line-separator is "\n".
#                         | ref.hash.string
# return       $ahFields | parsed fields | ref.array.hash.string
#========================================================================================
sub DecodeJSONArray($;$) {
    my ($Content, $hOptions) = @_;
    $hOptions = {} unless defined $hOptions;
    $hOptions->{'eol'} = "\n" unless exists $hOptions->{'eol'};
    TestHash('hOptions', $hOptions);

    my $Converter = DE_EPAGES::ShopMigration::API::JSON->new($hOptions);
    my $fh = IO::Scalar->new(\$Content);
    my @Lines;
    while (my $hFields = $Converter->getline($fh)) {
        push @Lines, $hFields;
    }

    return \@Lines;
}

#========================================================================================
# function     EncodeJSONArray
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Line = EncodeJSONArray( $ahFields, $hOptions );
# syntax       $Line = EncodeJSONArray( $ahFields );
#----------------------------------------------------------------------------------------
# description  Converts an array of hashes to JSON newline formatted string.
#----------------------------------------------------------------------------------------
# input        $ahFields | array of hashes
#                         | ref.array.hash.string
# input        $hOptions | see <function new>
#                           The default line-separator is "\n".
#                         | ref.hash.string
# return       $Line | CSV formatted string | string
#========================================================================================
sub EncodeJSONArray($;$) {
    my ($ahFields, $hOptions) = @_;
    $hOptions = {} unless defined $hOptions;
    $hOptions->{'line_sep'} = "\n" unless exists $hOptions->{'line_sep'};
    TestArray('ahFields', $ahFields);
    TestHash('hOptions', $hOptions);

    my $Converter = DE_EPAGES::ShopMigration::API::JSON->new($hOptions);

    return join($hOptions->{'line_sep'}, map {$Converter->encode($_)} @$ahFields);
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $JSON = DE_EPAGES::ShopMigration::API::JSON->new( $hOptions );
# syntax       $JSON = DE_EPAGES::ShopMigration::API::JSON->new;
# example      $JSON = DE_EPAGES::ShopMigration::API::JSON->new( { 'pretty' => '1' } );
#----------------------------------------------------------------------------------------
# description  Create a JSON parser and formatter object.
#----------------------------------------------------------------------------------------
# input        $hOptions | (optional) format options, see also JSON .
#                       <ul>
#                           <li>canonical       - sorting keys (default: '1')</li>
#                           <li>utf8            - encode to utf8 byte stream & decode from utf8 byte stream (default: 0)</li>
#                           <li>latin1          - encode will use escape characters for ordinal values over 255 (default: 0)</li>
#                           <li>ascii           - encode will use escape characters for ordinal values over 126  (default: 0)</li>
#                           <li>pretty          - enable/disable all of indent/space_before/space_after flags (default: 0)</li>
#                           <li>indent          - if true encode will use a multiline format with newlines (default: 0)</li>
#                           <li>space_before    - add extra space before the separator ":" on encoding (default: 0)</li>
#                           <li>space_after     - add extra space after the separator ":" on encoding (default: 0)</li>
#                           <li>relaxed         - allow some non-standard relaxation on input in decode (default: 0)</li>
#                           <li>allow_nonref    - allow encode of a non-reference into its corresponding string (default: 0)</li>
#                           <li>allow_unknown   - encode non-encodable values (e.g. filehandles) to JSON null (default: 0)</li>
#                           <li>convert_blessed - use blessed values TO_JSON method and use resulting scalar for encode (default:1) </li>
#                       </ul>
# return       $JSON | JSON parser and formatter | object
#========================================================================================
sub new {
    my ($class, $hOptions) = @_;

    #create json converter with default options
    my $Converter = JSON->new();

    # allow only documented options
    foreach my $key (qw(
        canonical utf8 latin1 ascii indent space_before space_after relaxed
        allow_nonref allow_unknown convert_blessed pretty
    )) {
        next unless exists $hOptions->{$key};
        $Converter->$key($hOptions->{$key});
    }
    return bless {
        Converter => $Converter
    }, $class;
}

#========================================================================================
# function     converter
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Converter = $JSON->converter;
#----------------------------------------------------------------------------------------
# description  Returns the underlying JSON object.
#----------------------------------------------------------------------------------------
# return       $Converter | instance of JSON | object
#========================================================================================
sub converter {
    return shift->{Converter};
}

#========================================================================================
# function     decode
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahFields = JSON->decode( $Line );
#----------------------------------------------------------------------------------------
# description  Parses one JSON formatted line and returns the parsed fields.
#----------------------------------------------------------------------------------------
# input        $Line | JSON formatted string | string
# return       $ahFields | parsed fields | ref.hash.string
#========================================================================================
sub decode {
    my $self = shift;
    my ($Line) = @_;

    utf8::upgrade($Line) unless utf8::is_utf8($Line);
    my $Converter = $self->converter;
    my $ahFields = $Converter->decode($Line);

    return $ahFields;
}

#========================================================================================
# function     encode
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Line = $JSON->encode( $hFields );
#----------------------------------------------------------------------------------------
# description  Encodes a hash of fields to one JSON formatted line.
#----------------------------------------------------------------------------------------
# input        $hFields | parsed fields | ref.hash.string
# return       $Line | JSON formatted string | string
#========================================================================================
sub encode {
    my $self = shift;
    my ($hFields) = @_;
    TestHash('hFields', $hFields);

    my $Converter = $self->converter;
    my $json_text = $Converter->encode($hFields);

    return decode_utf8($json_text);
}

##========================================================================================
## function     getline
## state        public
##----------------------------------------------------------------------------------------
## syntax       $ahFields = $JSON->getline( $FileHandle );
##----------------------------------------------------------------------------------------
## description  Reads one record from a JSON file.
##               Returns the parsed fields or undef on end of file.
##----------------------------------------------------------------------------------------
## input        $FileHandle | file handle | IO::Handle object
## return       $hFields | parsed line, undef on EOF | ref.hash.string
##========================================================================================
sub getline {
    my $self = shift;
    my ($FileHandle) = @_;

    my $hFields;
    my $Line = $FileHandle->getline();

    if (!defined $Line) {
        $hFields = undef;
    }
    else {
        eval {
            $hFields = $self->converter->decode($Line);
        };
        if (ExistsError()) {
            my $Error = GetError();
            my $Message = $Error->message;
            $Message =~ s/ at \S+ line \d+\b.*//; # remove code line
            Error('GET_LINE_FAILED', {
                'Message' => $Message,
                $self->formatSpec,
            });
        }
    }
    return $hFields;
}

#========================================================================================
# function     formatSpec
# state        public
#----------------------------------------------------------------------------------------
# syntax       %Format = $JSON->formatSpec;
#----------------------------------------------------------------------------------------
# description  Returns the current format specification.
#----------------------------------------------------------------------------------------
# return       %Format | current format specificationm, see <function new> | hash.string
#========================================================================================
sub formatSpec {
    my $self = shift;

    my $Converter = $self->converter;
    return(
        'canonical'       => $Converter->get_canonical,
        'utf8'            => $Converter->get_utf8,
        'ascii'           => $Converter->get_ascii,
        'latin1'          => $Converter->get_latin1,
        'indent'          => $Converter->get_indent,
        'space_before'    => $Converter->get_space_before,
        'space_after'     => $Converter->get_space_after,
        'relaxed'         => $Converter->get_relaxed,
        'allow_nonref'    => $Converter->get_allow_nonref,
        'allow_unknown'   => $Converter->get_allow_unknown,
        'allow_blessed'   => $Converter->get_allow_blessed,
        'convert_blessed' => $Converter->get_convert_blessed,
    );
}

1;
