#========================================================================================
# package      DE_EPAGES::ShopMigration::API::JSONLines::ShopObjectExportImport;
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements the high level interface for import and export of
#               objects to JSONLines files
#========================================================================================
package DE_EPAGES::ShopMigration::API::JSONLines::ShopObjectExportImport;

use strict;
use DE_EPAGES::Core::API::Error qw(Error);
use DE_EPAGES::Core::API::PerlTools qw(UsePackage);

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'JSON_SHOP_UNDEFINED' => 'To call this function a shop must be set before',
    };
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver=DE_EPAGES::ShopMigration::API::JSONLines::ShopObjectExportImport->new($Shop);
#----------------------------------------------------------------------------------------
# example      $Driver=DE_EPAGES::ShopMigration::API::JSONLines::ShopObjectExportImport->new($Shop);
#----------------------------------------------------------------------------------------
# description  Creates a new json export import driver.
#----------------------------------------------------------------------------------------
# input        $Shop    | shop object  | object
# input        $DirName | directory name | string
# return       $Driver  | import/export driver | object
#========================================================================================
sub new {
    my $class = shift;
    my ($Shop, $DirName) = @_;
    my %params = ('Shop' => $Shop, 'DirName' => $DirName);
    my $self = bless \%params, $class;
    return $self;
}

#========================================================================================
# function     exportDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->exportDriver();
#----------------------------------------------------------------------------------------
# description  Return driver package.
#----------------------------------------------------------------------------------------
# return       $DriverPackage  | package to driver | string
#========================================================================================
sub exportDriver {
    my $self = shift;
    die 'please implement this function';
}

#========================================================================================
# function     importDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->importDriver();
#----------------------------------------------------------------------------------------
# description  Return driver package.
#----------------------------------------------------------------------------------------
# return       $DriverPackage  | package to driver | string
#========================================================================================
sub importDriver {
    my $self = shift;
    die 'please implement this function';
}

#========================================================================================
# function     exportJSONData
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->exportJSONData($DirName, $aItems);
#----------------------------------------------------------------------------------------
# description  simple wrapper for exportJSONLines to call for a single shop
#----------------------------------------------------------------------------------------
# input        $DirName  | export directory | string
# input        $aItems   | list of exportable items (objectids) | ref.array
# return       $Exporter | exporter  | object
#========================================================================================
sub exportJSONData {
    my $self = shift;
    my ($DirName, $aItems) = @_;

    $aItems //= $self->getExportItems();
    my $Shop = $self->shop();
    Error('JSON_SHOP_UNDEFINED') unless defined $Shop;
    UsePackage($self->exportDriver);
    my $Driver = $self->exportDriver->new(Shop => $Shop, 'DirName' => $DirName);
    $Driver->exportJSONLines(undef, undef, undef, undef, $aItems);
    return $Driver;
}

#========================================================================================
# function     importJSONData
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Importer->importJSONData($DirName);
#----------------------------------------------------------------------------------------
# description  simple wrapper for importJSONLines to call for a single shop
#----------------------------------------------------------------------------------------
# input        $DirName  | import directory | string
# return       $Importer | importer    | object
#========================================================================================
sub importJSONData {
    my $self = shift;
    my ($DirName) = @_;

    my $Shop = $self->shop();
    Error('JSON_SHOP_UNDEFINED') unless defined $Shop;
    UsePackage($self->importDriver);
    my $Driver = $self->importDriver->importJSONLines(undef, undef, { Shop => $Shop, DirName => $DirName });
    return $Driver;
}

#========================================================================================
# function     shop
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->shop;
# syntax       $self->shop($Shop);
#----------------------------------------------------------------------------------------
# description  returns or sets the shop object for the im/exporter (used in subclasses)
#----------------------------------------------------------------------------------------
# input        $Shop | new shop | object
# return       $Shop | shop object | object
#========================================================================================
sub shop {
    my $self = shift;
    my ($Shop) = @_;
    if (@_ == 1) {
        $self->{Shop} = $Shop;
    }
    return $self->{Shop};
}

#========================================================================================
# function     getExportItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getExportItems()
#----------------------------------------------------------------------------------------
# description  Creates a list of exportable items (list of objectids)
#----------------------------------------------------------------------------------------
# return       $aItems | list of objectids | ref.array
#========================================================================================
sub getExportItems {
    my $self = shift;

    die 'please implement this function';
}


1;
