#========================================================================================
# package      DE_EPAGES::ShopMigration::API::MailTypesCommon
# state        public
#----------------------------------------------------------------------------------------
# description common functions for exporting/importing mailtypes
#========================================================================================
package DE_EPAGES::ShopMigration::API::MailTypesCommon;
use base qw(Exporter);

use strict;
use feature 'switch';
use DE_EPAGES::ShopMigration::API::ShopCommonData qw(GetActiveShopLanguages);
our @EXPORT_OK = qw(
    GetShopMailTypes
    SetShopMailTypes
    SetShopMailTypeData
    GetShopMailTypeData
);

use DE_EPAGES::Core::API::PerlTools qw(ElementOf);
use DE_EPAGES::Object::API::Factory qw(LoadRootObject);
use DE_EPAGES::Object::API::Language qw(GetPKeyLanguageByCode);
use DE_EPAGES::ShopMigration::API::Constants qw(
    MAILTYPE_ATTRIBUTES_FOR_EXPORT
    MAILTYPE_ATTRIBUTES_FOR_IMPORT
    MAILTYPE_LOCALIZEDATTRIBUTES_FOR_EXPORT
    MAILTYPE_LOCALIZEDATTRIBUTES_FOR_IMPORT
    MAILTYPE_SCRIBEATTRIBUTES_FOR_EXPORT
    MAILTYPE_BOOLEAN_ATTRIBUTES_FOR_RESET
    MAILTYPE_LOCALIZEDATTRIBUTES_FOR_RESET
);
use DE_EPAGES::ShopMailType::API::ShopMailType qw(GetShopMailType); ##no critic (RequireCorrectImports)
use DE_EPAGES::ShopMigration::API::HTMLSanitizer qw(SanitizeHTMLForScribe);

#========================================================================================
# function     GetShopMailTypes
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ahMailTypes = GetShopMailTypes($Shop);
#----------------------------------------------------------------------------------------
# description  Filter for those Mailtypes that we can actually use in the PrepShop
#----------------------------------------------------------------------------------------
# input        $Shop | BaseShop (epages6) | object
#----------------------------------------------------------------------------------------
# return       $ahMailTypes | see GetShopMailTypeData() | reference to array of hashes
#========================================================================================
sub GetShopMailTypes {
    my ($Shop) = @_;

    my @Items;
    my $ShopMailTypes = $Shop->get('ShopMailTypes');
    foreach my $ShopMailType (sort {$a->alias cmp $b->alias} @$ShopMailTypes) {
        my $hInfo = _getShopMailTypeInfo($Shop, $ShopMailType);
        push @Items, $hInfo if (keys %$hInfo);
    }
    return \@Items;
}

#========================================================================================
# function     GetShopMailTypeData
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $hMailType = GetShopMailTypeData($Shop, $ShopMailType);
#----------------------------------------------------------------------------------------
# description  Constructs a hash with the exportable fields for a single shopmailtype
#               returns empty hash if ShopMailType is not exportable
#----------------------------------------------------------------------------------------
# input        $Shop | BaseShop (epages6) | object
# input        $ShopMailType | shopmailtype | object
#----------------------------------------------------------------------------------------
# return       $hMailType | hash with following fields
#               <ul>
#                   <li>Attributes
#                       <ul>
#                           <li>Alias                        - string</li>
#                           <li>IsActive                     - boolean</li>
#                           <li>TO                           - string</li>
#                           <li>FROM                         - string</li>
#                           <li>CC                           - string</li>
#                           <li>BCC                          - string</li>
#                           <li>OrderDocumentPdfAttachment   - 0</li>
#                           <li>AttachCustomerInformationPDF - 0</li>
#                           <li>UnityAttachLegalDocumentsPDF - array of string ( if $Shop is PrepShop only)</li>
#                       </ul>
#                   </li>
#                   <li>LocalizedAttributes
#                       <ul>
#                           <li>LanguageCode2                    - en|de|..</li>
#                               <ul>
#                                   <li>Subject                  - string</li>
#                                   <li>Header                   - string</li>
#                                   <li>AdditionalText           - undef</li>
#                               </ul>
#                           </li>
#                       </ul>
#                   </li>
#               </ul>| reference to hash
#========================================================================================
sub GetShopMailTypeData {
    my ($Shop, $ShopMailType) = @_;

    return _getShopMailTypeInfo($Shop, $ShopMailType);
}

sub _getShopMailTypeInfo {
    my ($Shop, $ShopMailType) = @_;

    my %Info = ();
    my $ActiveLanguages = GetActiveShopLanguages($Shop);
    if ($ShopMailType->get('MailTypeTemplate')->get('IsForUnity') or ($ShopMailType->alias eq 'NewPassword')) {
        # get baseattributes, reset some attributes to default in Prep2Base step (not used in now)
        my $Attributes = [ @{MAILTYPE_ATTRIBUTES_FOR_EXPORT()} ];
        $Info{'Attributes'} = $ShopMailType->get($Attributes);
        foreach my $FieldName (@{MAILTYPE_BOOLEAN_ATTRIBUTES_FOR_RESET()}) {
            $Info{'Attributes'}{$FieldName} = 0;
        }
        # convert NewPassword to UnityNewPassword
        $Info{'Attributes'}->{'Alias'} = 'UnityNewPassword' if $Info{'Attributes'}->{'Alias'} eq 'NewPassword';
        $Info{'Attributes'}->{'IsActive'} += 0;
        # localized attributes
        my $LocalizedAttributes = [ @{MAILTYPE_LOCALIZEDATTRIBUTES_FOR_EXPORT()} ];
        foreach my $hLanguage (@$ActiveLanguages) {
            my $hLocalInfo = $ShopMailType->get($LocalizedAttributes, $hLanguage->{'LanguageID'});
            $Info{'LocalizedAttributes'}{$hLanguage->{'Code2'}} = $hLocalInfo;
            foreach my $FieldName (@{MAILTYPE_SCRIBEATTRIBUTES_FOR_EXPORT()}) {
                $hLocalInfo->{$FieldName} = SanitizeHTMLForScribe($hLocalInfo->{$FieldName});
            }
        }
        foreach my $Code2 (map {$_->{'Code2'}} @{LoadRootObject()->get('Languages')}) {
            foreach my $FieldName (@{MAILTYPE_LOCALIZEDATTRIBUTES_FOR_RESET()}) {
                $Info{'LocalizedAttributes'}{$Code2}{$FieldName} = undef;
            }
        }
        # convert ref attributes to strings
        if ($Shop->get('IsMigrationPrepShop') &&
            $ShopMailType->get('MailTypeTemplate')->get('MailTypeTemplateGroup')->alias eq 'OrderStatus') {
            my $LegalDocs = [ map {$_->alias} @{$ShopMailType->get('UnityAttachLegalDocumentsPDF')} ];
            $Info{'Attributes'}->{UnityAttachLegalDocumentsPDF} = $LegalDocs;
        }
    }
    return \%Info;
}

#========================================================================================
# function     SetShopMailTypes
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetShopMailTypes($Shop, $ahMailTypes);
#----------------------------------------------------------------------------------------
# description  sets attributes on shopmailtype as specified in the $ahMailTypes
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# input        $ahMailTypes  | see GetShopMailTypeData() | reference to array of hashes
#========================================================================================
sub SetShopMailTypes {
    my ($Shop, $ahMailTypes) = @_;

    foreach my $hMailType (@$ahMailTypes) {
        SetShopMailTypeData($Shop, $hMailType);
    }
}

#========================================================================================
# function     SetShopMailTypeData
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetShopMailTypeData($Shop, $hMailType);
#----------------------------------------------------------------------------------------
# description  sets attributes on a shopmailtype as specified in the hash $hMailType
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# input        $hMailType    | hash with following fields
#               <ul>
#                   <li>Attributes
#                       <ul>
#                           <li>Alias                        - string</li>
#                           <li>IsActive                     - boolean</li>
#                           <li>TO                           - string</li>
#                           <li>FROM                         - string</li>
#                           <li>CC                           - string</li>
#                           <li>BCC                          - string</li>
#                           <li>OrderDocumentPdfAttachment   - boolean ( if $Shop is BaseShop only)</li>
#                           <li>AttachCustomerInformationPDF - boolean ( if $Shop is BaseShop only)</li>
#                           <li>UnityAttachLegalDocumentsPDF - array of string ( if $Shop is BaseShop only)</li>
#                       </ul>
#                   </li>
#                   <li>LocalizedAttributes
#                       <ul>
#                           <li>LanguageCode2                    - en|de|..</li>
#                               <ul>
#                                   <li>Subject                  - string</li>
#                                   <li>Header                   - string</li>
#                                   <li>AdditionalText           - string ( if $Shop is BaseShop only)</li>
#                               </ul>
#                           </li>
#                       </ul>
#                   </li>
#               </ul>| reference to hash
#========================================================================================
sub SetShopMailTypeData {
    my ($Shop, $hMailType) = @_;

    my $Alias = $hMailType->{Attributes}{Alias};
    my @Attributes = @{MAILTYPE_ATTRIBUTES_FOR_IMPORT()};
    foreach my $Key (keys %{$hMailType->{Attributes}}) {
        die "Invalid Element for Attributes: $Key" unless ElementOf($Key, @Attributes);
    }
    my $MailType = GetShopMailType($Shop, $Alias);
    if (!$Shop->get('IsMigrationPrepShop') && exists $hMailType->{Attributes}{UnityAttachLegalDocumentsPDF}) {
        my $aDocNames = $hMailType->{Attributes}{UnityAttachLegalDocumentsPDF};
        delete $hMailType->{Attributes}{UnityAttachLegalDocumentsPDF};
        foreach my $LegalDoc (grep {$_->alias ne 'Imprint'} @{$Shop->child('LegalPagesUnity')->children}) {
            foreach my $WantedDoc (@$aDocNames) {
                if ($LegalDoc->alias eq $WantedDoc) {
                    push @{$hMailType->{Attributes}{UnityAttachLegalDocumentsPDF}}, $LegalDoc;
                }
            }
        }
    }
    $MailType->set($hMailType->{Attributes}) if $MailType;
    my @LocalizedAttributes = @{MAILTYPE_LOCALIZEDATTRIBUTES_FOR_IMPORT()};
    foreach my $LanguageCode2 (keys %{$hMailType->{LocalizedAttributes}}) {
        foreach my $Key (keys %{$hMailType->{LocalizedAttributes}{$LanguageCode2}}) {
            die "Invalid Element for Attributes: $Key" unless ElementOf($Key, @LocalizedAttributes);
        }
        my $LanguageID = GetPKeyLanguageByCode($LanguageCode2);
        # Silently ignore fields for non-existing languages
        if ($Shop->existsLanguage($LanguageID)) {
            $MailType->set($hMailType->{LocalizedAttributes}{$LanguageCode2}, $LanguageID);
        }
    }
}

1;
