#========================================================================================
# package      DE_EPAGES::ShopMigration::API::MigrationShop
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the functions regarding migration shop and its prepshop.
#========================================================================================
package DE_EPAGES::ShopMigration::API::MigrationShop;
use base Exporter;
use strict;
use 5.010; # Needed because we use state vars

our @EXPORT_OK = qw(
    CreatePrepShop
    ExistsPrepShop
    SetShopFeatureValues
    TransferEpagesUI
    FindAllShopsWithMigrationStatus
    ExportShopDataForMigration
    ImportShopDataForMigration
    PreMigrationImportTasks
    PostMigrationImportTasks
);

use DE_EPAGES::Core::API::Error qw(Error ExistsError GetError);
use DE_EPAGES::Core::API::Log qw(LogError LogDebug);
use DE_EPAGES::Core::API::TestParameter qw(TestObject TestArray TestString);
use DE_EPAGES::Database::API::Config qw(DatabaseConnectionData);
use DE_EPAGES::Database::API::Connection qw(RunOnStore GetCurrentDBHandle);
use DE_EPAGES::Object::API::Domain qw(ConstructUnityDomainName GetUnitySubDomain);
use DE_EPAGES::Object::API::Factory qw(LoadObjectByPath Transaction LoadRootObject LoadObject LoadClassByAlias ExistsObject);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw(ImportObjects);
use DE_EPAGES::Object::API::XML::AttributeImportHandler qw(ImportAttributes);
use DE_EPAGES::Shop::API::Table::ShopFeature qw(GetInfoShopFeaturesByShopID);
use DE_EPAGES::ShopMigration::API::Cartridge;
use DE_EPAGES::ShopMigration::API::Constants qw(
    PREPSHOPS_FOLDER_ALIAS
    @SHOPMIGRATION_PREPSHOP_FEATURES
    %SHOPMIGRATION_SHOPTYPE_MAPPING
    SHOPMIGRATION_INITIAL_IN_PROGRESS
    SHOPMIGRATION_INITAL_PREPARED
    SHOPMIGRATION_FINAL_ALLOW
    SHOPMIGRATION_FINAL_IN_PROGRESS
    SHOPMIGRATION_FINAL_DONE
    EXPORT_IMPORT_ORDER
);
use DE_EPAGES::RemoteSearch::API::RemoteSearchImpl qw(GetOrCreateSearchIndex); ##no critic (RequireCorrectImports)
use DE_EPAGES::ShopMigration::API::ShopCommonData qw(GetBasicShopData SetBasicShopData);
##no critic (RequireCorrectImports)
use DE_EPAGES::ShopMigration::API::JSONLines::ShopDataExportImport qw(ImportShopDataJSON ExportShopDataJSON);
use DE_EPAGES::ShopMigration::API::JSONLines::MailTypesExportImport qw(ImportMailTypesJSON ExportMailTypesJSON);
use DE_EPAGES::ShopMigration::API::JSONLines::LegalPagesExportImport qw(ImportLegalPagesJSON ExportLegalPagesJSON);
use DE_EPAGES::ShopMigration::API::JSONLines::CategoriesExportImport qw(ImportCategoriesJSON ExportCategoriesJSON);
use DE_EPAGES::ShopMigration::API::JSONLines::ProductTypesExportImport qw(ImportProductTypesJSON ExportProductTypesJSON);
use DE_EPAGES::ShopMigration::API::JSONLines::ProductsExportImport qw(ImportProductsJSON ExportProductsJSON);
use DE_EPAGES::ShopMigration::API::JSONLines::CategoryProductMapsExportImport qw(
    ImportCategoryProductMapsJSON ExportCategoryProductMapsJSON
);
use DE_EPAGES::ShopMigration::API::JSONLines::CrossSellingsExportImport qw(ImportCrossSellingsJSON ExportCrossSellingsJSON);
## use critic
use DE_EPAGES::ShopMigration::API::ResetShopHandler qw(ResetBaseShop);
use DE_EPAGES::Trigger::API::Trigger qw(TriggerHook);

our $DELETE_PREP_SHOP = 0;

#ERRORDEFINITION
sub ErrorDefinition() {
    return {
        'NOT_A_BASESHOP'               => 'The shop #Alias is not a base shop.',
        'PREPSHOP_EXISTS'              => 'The shop #Alias has already a prepshop at prepshops folder.',
        'NO_PREPSHOP'                  => 'The shop #Alias has no prepshop.',
        'MIGRATIONSTATUS_NOT_EXISTING' => 'The attribute MigrationStatus does not exists in class shop.',
    };
}
#ERRORDEFINITION END

#========================================================================================
# function     SetShopFeatureValues
# state        intern
#----------------------------------------------------------------------------------------
# syntax       SetShopFeatureValues($Shop, $ahFeatureValues);
#----------------------------------------------------------------------------------------
# description  sets the shopfeatures for shop
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $ahFeatureValues | array of ShopFeatures to set | ref.array.hash.string
#========================================================================================
sub SetShopFeatureValues {
    my ($Shop, $ahFeatureValues) = @_;
    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');
    TestArray('ahFeatureValues', $ahFeatureValues);

    my $Features = LoadObjectByPath('/Features');
    Transaction(sub {
        foreach my $hFeatureValue (@$ahFeatureValues) {
            my $MaxValue = $hFeatureValue->{'MaxValue'};
            my $IsVisible = exists $hFeatureValue->{'IsVisible'} ? $hFeatureValue->{'IsVisible'} : 1;
            my $Feature;
            if (exists $hFeatureValue->{'Alias'}) {
                $Feature = $Features->child($hFeatureValue->{'Alias'}) if ($Features->existsChild($hFeatureValue->{'Alias'}));
            } elsif (exists $hFeatureValue->{'FeatureID'}) {
                $Feature = LoadObject($hFeatureValue->{'FeatureID'}) if ExistsObject($hFeatureValue->{'FeatureID'});
            }
            if (defined $Feature) {
                $Shop->featureMaxValue($Feature, $MaxValue, $IsVisible);
            } elsif ($MaxValue) {
                Error('FeatureNotAvailable', {
                    'FeatureAlias' => $hFeatureValue->{'Alias'},
                    'FeatureID'    => $hFeatureValue->{'FeatureID'},
                });
            }
        }
    });
    return;
}

#========================================================================================
# function     ExistsPrepShop
#----------------------------------------------------------------------------------------
# syntax       $Exists = ExistsPrepShop( $Shop );
#----------------------------------------------------------------------------------------
# description  Returns true if a PrepShop for this Shop exists in the database.
#----------------------------------------------------------------------------------------
# input        $Shop | base shop - shop object | object
# return       $Exists | true if the prep shop exists | boolean
#========================================================================================
sub ExistsPrepShop {
    my ($Shop) = @_;
    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    my $Folder = _getPrepShopFolder();
    if ($Folder->existsChild(_getPrepShopAlias($Shop->alias))) {
        return 1;
    } else {
        return 0;
    }
}

#========================================================================================
# function     CreatePrepShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $PrepShop = CreatePrepShop($Shop);
#----------------------------------------------------------------------------------------
# description  creates a PrepShop if not existing and returns it
#----------------------------------------------------------------------------------------
# input        $Shop | base shop - shop object | object
#----------------------------------------------------------------------------------------
# return       $PrepShop | prep shop - shop object | object
#========================================================================================
sub CreatePrepShop {
    my ($Shop) = @_;

    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');
    if ($Shop->parent->alias ne 'Shops' or $Shop->get('IsUnityRegistered')) {
        Error('NOT_A_BASESHOP', { Alias => $Shop->alias });
    }
    if (ExistsPrepShop($Shop)) {
        Error('PREPSHOP_EXISTS', { Alias => $Shop->alias });
    }
    my $hShopData = GetBasicShopData($Shop);

    $Shop->set({ 'MigrationStatus' => SHOPMIGRATION_INITIAL_IN_PROGRESS });
    my $PrepShopAlias = _getPrepShopAlias($hShopData->{'Attributes'}{'Alias'});
    #use UnitySlug of Base Shop
    my $UnitySlug = GetUnitySubDomain($hShopData->{'Attributes'}{'Alias'});
    #create prepshop
    my $PrepShopFolder = _getPrepShopFolder();
    my $ShopTypeRef = _getShopTypeMappingForShop($Shop);
    my $ahFeatures = _getPrepShopFeaturesByShopTypeRef($ShopTypeRef);

    my $PrepShop = $PrepShopFolder->insertChild('Shop', {
        'Alias'                    => $PrepShopAlias,
        'WebServerScriptNamePart'  => $PrepShopAlias,
        'UnitySlug'                => $UnitySlug,
        'ShopTypeRef'              => $ShopTypeRef,
        'InitialShopWizardStartup' => 0,
        'DomainName'               => ConstructUnityDomainName($hShopData->{'Attributes'}{'Alias'}),
        'HasSSLCertificate'        => 1,
        'IsMigrationPrepShop'      => 1,
        'LocaleID'                 => $hShopData->{'Attributes'}{'LocaleID'}
    });
    SetShopFeatureValues($PrepShop, $ahFeatures);
    _importPrepShopDefaults($PrepShop);
    SetBasicShopData($PrepShop, $hShopData);
    sleep 5;
    eval {$PrepShop->registerUnity;};
    if (ExistsError()) {
        my $Error = GetError();
        #if registerunity does not work, delete prepshop
        $PrepShop->delete;
        Error($Error);
    }
    #set it again because we need to add additional languages again after registerUnity
    SetBasicShopData($PrepShop, $hShopData);

    _removePagesinUI($UnitySlug);

    # Add now domain to pooldb (works only after registerunity with urlschema domain)
    $PrepShop->set({ 'WebServerScriptNamePart' => $PrepShopAlias, 'MigrationShop' => $Shop });

    $Shop->set({ 'MigrationPrepShop' => $PrepShop });
    return $PrepShop;
}

#========================================================================================
# function     TransferEpagesUI
# state        public
#----------------------------------------------------------------------------------------
# syntax       TransferEpagesUI($Shop);
#----------------------------------------------------------------------------------------
# description  assigns epages-ui from prepshop to migrationshop
#               unset IsUnityRegistered from prepshop
#----------------------------------------------------------------------------------------
# input        $Shop | base shop - shop object | object
#========================================================================================
sub TransferEpagesUI {
    my ($Shop) = @_;

    my $PrepShop = $Shop->get('MigrationPrepShop');
    Error('NO_PREPSHOP') unless defined $PrepShop;

    #add unity feature to base shop
    my $Features = LoadObjectByPath('/Features');
    my $Feature = $Features->child('Unity');
    $Shop->featureMaxValue($Feature, 1, 1);

    #do nearly the same like DE_EPAGES/Shop/API/Object/Shop.pm sub registerUnity
    my %Content;
    #add all the things e.g. ShopNotifications, uneditable and prefilled attributes
    TriggerHook('API_BeforeCreateUnityShop', {
        'Object'  => $Shop,
        'Content' => \%Content
    });
    eval {
        #delete locale (this cannot patched here, this only possible for create shops)
        delete $Content{'locale'};
        # send credentials to the Unity database (via now shop app)
        $PrepShop->sendUnityRequest('PATCH', '/shop', undef, \%Content);
    };
    if (ExistsError()) {
        my $Error = GetError();
        #todo:
        # what should happen in this case ?
        # remove the unity feature? some data is already overwritten (uneditable, prefilled attributes) :-(
        LogError($Error);
        Error($Error);
    } else {
        # unassign now shop:
        $PrepShop->set({ 'DomainName' => undef, 'IsUnityRegistered' => 0 });

        #set domainname again to add unity wildcard domain to pooldb
        my $DomainName = $Shop->get('DomainName');
        $Shop->set({
            'DomainName'        => $DomainName,
            'IsUnityRegistered' => 1,
        });
    }
    return;
}


sub _getShopTypeMappingForShop {
    my ($Shop) = @_;

    my $ShopTypeRefFolder = LoadRootObject()->folder('ShopTypeRefs');
    my $ShopTypeRef = $Shop->get('ShopTypeRef');
    my $ShopTypeRefAlias = defined $ShopTypeRef ? $ShopTypeRef->alias : '_default';
    if (exists $SHOPMIGRATION_SHOPTYPE_MAPPING{$ShopTypeRefAlias}) {
        $ShopTypeRefAlias = $SHOPMIGRATION_SHOPTYPE_MAPPING{$ShopTypeRefAlias};
    } else {
        $ShopTypeRefAlias = $SHOPMIGRATION_SHOPTYPE_MAPPING{'_default'};
    }
    return $ShopTypeRefFolder->child($ShopTypeRefAlias);
}

sub _getPrepShopFeaturesByShopTypeRef {
    my ($ShopTypeRef) = @_;
    my $aShopIDs = LoadClassByAlias('Shop')->findObjects('ByShopTypeRefID', [ $ShopTypeRef->id ]);
    my $FirstShopWithShopTypeRef = scalar(@$aShopIDs) ? LoadObject($aShopIDs->[0]) : undef;
    return defined $FirstShopWithShopTypeRef ? GetInfoShopFeaturesByShopID($FirstShopWithShopTypeRef->id) : \@SHOPMIGRATION_PREPSHOP_FEATURES;
}

sub _importPrepShopDefaults {
    my ($PrepShop) = @_;

    my $Cartridge = DE_EPAGES::ShopMigration::API::Cartridge->new;
    my $Directory = join('/', $ENV{EPAGES_CARTRIDGES}, $Cartridge->cartridgeDirectory, 'Data/PrepShopImport.d');
    foreach my $file (qw(BasicMinUnity.xml)) {
        _importXML($Directory . '/' . $file, $PrepShop->pathString);
    }
    return;
}

sub _removePagesinUI {
    my ($UnitySlug) = @_;

    my $Store = LoadRootObject()->get('StoreName');
    my $UIStore = DatabaseConnectionData($Store)->{InstanceUnityUI};
    if (!defined $UIStore || !$UIStore) {
        Error('UI_STORE_NOT_DEFINED', { Store => $Store })
    } else {
        RunOnStore(
            Store => $UIStore,
            Sub   => sub {
                my $dbi = GetCurrentDBHandle();
                #remove default pages (type='blank') from ui db
                my $sth = $dbi->executeStatement(<<SQL, [ $UnitySlug ]);
DELETE FROM pages
WHERE type='blank'
  AND shopId IN (
    SELECT id FROM shops WHERE shopGateID = (
        SELECT id FROM shopGates WHERE slug=?
    )
)
SQL
                LogDebug(sprintf("Deleting in %s.pages [%s]: affected %s rows\n",
                    $UIStore, $UnitySlug, $sth->rows), 1);
            }
        );
    }
    return;
}

sub _importXML {
    my ($FileName, $Path) = @_;
    TestString('FileName', $FileName);
    TestString('Path', $Path);

    if ($FileName =~ /\bTranslation\.(\w+\.)+(\w\w|\w\w-\w\w)\.xml$/) {
        ImportAttributes($FileName, $Path);
    } else {
        ImportObjects($FileName, $Path);
    }
    return;
}

sub _getPrepShopAlias {
    my ($ShopAlias) = @_;
    return GetUnitySubDomain('PrepShop' . $ShopAlias);
}

sub _getPrepShopFolder {
    return LoadRootObject()->child(PREPSHOPS_FOLDER_ALIAS);
}

#========================================================================================
# function     FindAllShopsWithMigrationStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aShops = FindAllShopsWithMigrationStatus($Status);
#----------------------------------------------------------------------------------------
# description  returns an array of shops with the given migration status
#               see DE_EPAGES::ShopMigration::API::Constants for a list of
#               available status
#----------------------------------------------------------------------------------------
# input        $Status | wanted migration status | integer
#----------------------------------------------------------------------------------------
# return       $PrepShop | prep shop - shop object | object
#========================================================================================
sub FindAllShopsWithMigrationStatus {
    my ($Status) = @_;

    my $ExistsAttribute = LoadClassByAlias('Shop')->existsAttribute('MigrationStatus');
    Error('MIGRATIONSTATUS_NOT_EXISTING') unless $ExistsAttribute;
    my $MigrationStatusAttributeID = LoadClassByAlias('Shop')->attribute('MigrationStatus')->id;
    my $dbi = GetCurrentDBHandle();
    my $Query = 'SELECT objectid from integerattribute WHERE attributeid=? and value=?';
    my @Params = ($MigrationStatusAttributeID, $Status);
    my $aaRows = $dbi->execute($Query, \@Params, [ 'Integer' ]);
    return [ map {LoadObject($_->[0])} @$aaRows ];
}

#========================================================================================
# function     ExportShopDataForMigration
# state        public
#----------------------------------------------------------------------------------------
# syntax       ExportShopDataForMigration($Shop, $Directory);
#----------------------------------------------------------------------------------------
# description  exports all the JSON data file for a shop (prepshop or baseshop)
#               to the given directory
#----------------------------------------------------------------------------------------
# input        $Shop      | shop object | object
# input        $Directory | output directory | string
#========================================================================================
sub ExportShopDataForMigration {
    my ($Shop, $Directory) = @_;

    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');
    TestString('Directory', $Directory);
    state $Dispatch = { map {$_, \&{'Export' . $_ . 'JSON'}} @{EXPORT_IMPORT_ORDER()} };
    foreach my $DataType (@{EXPORT_IMPORT_ORDER()}) {
        LogDebug(sprintf("Exporting %s to %s\n", $DataType, $Directory), 1);
        $Dispatch->{$DataType}->($Shop, $Directory);
    }

}

#========================================================================================
# function     ImportShopDataForMigration
# state        public
#----------------------------------------------------------------------------------------
# syntax       ImportShopDataForMigration($Shop, $Directory);
#----------------------------------------------------------------------------------------
# description  imports all the JSON data file for a shop (prepshop or baseshop)
#               from a given directory
#----------------------------------------------------------------------------------------
# input        $Shop      | shop object | object
# input        $Directory | output directory | string
#========================================================================================
sub ImportShopDataForMigration {
    my ($Shop, $Directory) = @_;

    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');
    TestString('Directory', $Directory);
    state $Dispatch = { map {$_, \&{'Import' . $_ . 'JSON'}} @{EXPORT_IMPORT_ORDER()} };
    foreach my $DataType (@{EXPORT_IMPORT_ORDER()}) {
        LogDebug(sprintf("Importing %s from %s\n", $DataType, $Directory), 1);
        $Dispatch->{$DataType}->($Shop, $Directory);
    }
}

#========================================================================================
# function     PreMigrationImportTasks
# state        public
#----------------------------------------------------------------------------------------
# syntax       PreMigrationImportTasks($Shop);
#----------------------------------------------------------------------------------------
# description  runs pre data import tasks necessary depending on whatever phase
#               this shop is in
#----------------------------------------------------------------------------------------
# input        $Shop      | base shop object | object
#========================================================================================
sub PreMigrationImportTasks {
    my ($Shop) = @_;

    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');
    if ($Shop->get('MigrationStatus') == SHOPMIGRATION_INITIAL_IN_PROGRESS) {
        LogDebug('Running [PreMigrationImportTasks] Nothing to do.', 1);

    } elsif ($Shop->get('MigrationStatus') == SHOPMIGRATION_FINAL_ALLOW) {
        LogDebug('Running [PreMigrationImportTasks] MigrationStatus => SHOPMIGRATION_FINAL_IN_PROGRESS', 1);
        $Shop->set({ 'MigrationStatus' => SHOPMIGRATION_FINAL_IN_PROGRESS });

        #delete / reset obsolete data in baseshop
        LogDebug('Running [PreMigrationImportTasks] ResetBaseShop()', 1);
        ResetBaseShop($Shop);

        #transfer epages-ui of prepshop to baseshop -> set IsUnityRegistered
        LogDebug('Running [PreMigrationImportTasks] TransferEpagesUI()', 1);
        TransferEpagesUI($Shop);
    }
}

#========================================================================================
# function     PostMigrationImportTasks
# state        public
#----------------------------------------------------------------------------------------
# syntax       PostMigrationImportTasks($Shop);
#----------------------------------------------------------------------------------------
# description  runs post data import tasks necessary depending on whatever this is a
#               prepshop or transfer shop
#----------------------------------------------------------------------------------------
# input        $Shop      | shop object | object
#========================================================================================
sub PostMigrationImportTasks {
    my ($Shop) = @_;

    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');
    if ($Shop->get('MigrationStatus') == SHOPMIGRATION_INITIAL_IN_PROGRESS) {
        LogDebug('Running [PostMigrationImportTasks] GetOrCreateSearchIndex(PrepShop)->initialize', 1);
        my $PrepShop = $Shop->get('MigrationPrepShop');
        GetOrCreateSearchIndex($PrepShop)->initialize();

        LogDebug('Running [PostMigrationImportTasks] MigrationStatus => SHOPMIGRATION_INITAL_PREPARED', 1);
        $Shop->set({ 'MigrationStatus' => SHOPMIGRATION_INITAL_PREPARED });

    } elsif ($Shop->get('MigrationStatus') == SHOPMIGRATION_FINAL_IN_PROGRESS) {
        LogDebug('Running [PostMigrationImportTasks] GetOrCreateSearchIndex(Shop)->initialize', 1);
        GetOrCreateSearchIndex($Shop)->initialize();

        LogDebug('Running [PostMigrationImportTasks] changeShopType', 1);
        # TODO: change shoptype

        LogDebug('Running [PostMigrationImportTasks] PrepShop->delete', 1);
        my $PrepShop = $Shop->get('MigrationPrepShop');
        $PrepShop->delete if (defined $PrepShop && $DELETE_PREP_SHOP);

        LogDebug('Running [PostMigrationImportTasks] MigrationStatus => SHOPMIGRATION_FINAL_DONE', 1);
        $Shop->set({ 'IsClosed' => 0, 'MigrationStatus' => SHOPMIGRATION_FINAL_DONE });
    }
}

1;