use strict;
use File::Spec::Functions;
use Getopt::Long;
use String::Random;
use Time::HiRes qw(time);
use 5.010; # Needed because we use state vars

use DE_EPAGES::Core::API::Error qw(ExistsError GetError);
use DE_EPAGES::Core::API::Log qw(LogError LogContext);
use DE_EPAGES::Core::API::Script qw(RunScript);
use DE_EPAGES::Core::API::Warning qw(Warning);
use DE_EPAGES::Database::API::Connection qw(RunOnStore);
use DE_EPAGES::Object::API::Factory qw(LoadRootObject);
use DE_EPAGES::ShopMigration::API::Constants qw(
    SHOPMIGRATION_INITAL_FAILED SHOPMIGRATION_FINAL_FAILED
    SHOPMIGRATION_INITIAL_IN_PROGRESS SHOPMIGRATION_FINAL_ALLOW);
use DE_EPAGES::ShopMigration::API::MigrationShop qw(
    ExistsPrepShop
    FindAllShopsWithMigrationStatus CreatePrepShop
    ExportShopDataForMigration ImportShopDataForMigration
    PreMigrationImportTasks PostMigrationImportTasks);
use DE_EPAGES::WebInterface::API::MessageCenter qw(SynchronizeCache);

##no critic qw( RequireCommentForExportedFunctions ProhibitMultiplePackages RequireArgUnpacking ProhibitPackageVars )
# hack log4perl / cspec.E and cspec.G for easier search in ELK
use DE_EPAGES::WebInterface::API::ApplicationServer;
package DE_EPAGES::WebInterface::API::ApplicationServer;
sub currentRequest {return shift;}
sub headers {return shift;}
sub address {return '127.0.0.1';}
sub port {return '0';}
sub header {return @_[0]->{@_[1]};}
our $FakeAppSrv = bless {}, 'DE_EPAGES::WebInterface::API::ApplicationServer';
$DE_EPAGES::WebInterface::API::ApplicationServer::APPLICATION_SERVER = $FakeAppSrv;
# end of hack
## use critic

package main; ##no critic ( ProhibitMultiplePackages )

sub Main {
    local $| = 1;
    my %Options = ();

    my @GetOpt = qw(help passwd=s storename=s shop=s directory=s onlyimport prep final output! printfailed resetfailed);
    usage("Unkown parameter specified.\n") unless GetOptions(\%Options, @GetOpt);
    usage() if $Options{help};
    usage('Missing parameter -storename') unless $Options{storename};
    $Options{directory} //= catdir($ENV{EPAGES_SHARED}, 'Packages', 'shopexport', $Options{storename});
    $Options{prep} = (exists $Options{final} && $Options{final}) ? 0 : 1;

    my $time1 = time;
    my $hCount;
    RunOnStore(
        Store      => $Options{storename},
        DBPassword => $Options{passwd},
        Sub        => sub {
            printFailed() if ($Options{printfailed});
            resetFailed() if ($Options{resetfailed});
            $hCount = migrateShops(\%Options);
        },
    );
    my $time2 = time;
    $hCount->{Left} = $hCount->{Total} - $hCount->{Done} - $hCount->{Failed};
    printf "\n# Statistics #\n" . join('', map {"%-13s: %s\n"} keys %$hCount),
        'Total Shops', $hCount->{Total}, 'Migrated', $hCount->{Done}, 'Failed',
        $hCount->{Failed}, 'Left', $hCount->{Left} if $Options{output};
    printf "need %.3f seconds\n", $time2 - $time1;
    eval {SynchronizeCache();};
    Warning(GetError()) if ExistsError();
}

sub printFailed {
    my $aFailedPrep = FindAllShopsWithMigrationStatus(SHOPMIGRATION_INITAL_FAILED);
    my $aFailedFinal = FindAllShopsWithMigrationStatus(SHOPMIGRATION_FINAL_FAILED);
    if (scalar @$aFailedPrep) {
        printf "Shops in state 'SHOPMIGRATION_INITAL_FAILED':\n%s\n",
            join("\n", map {$_->alias} @$aFailedPrep);
    }
    if (scalar @$aFailedFinal) {
        printf "Shops in state 'SHOPMIGRATION_FINAL_FAILED':\n%s\n",
            join("\n", map {$_->alias} @$aFailedFinal);
    }
    if (!scalar @$aFailedPrep && !scalar @$aFailedFinal) {
        printf "Found no shops in failed migration status in this store.\n",
    }
    exit 0;
}

sub resetFailed {
    my $aFailedPrep = FindAllShopsWithMigrationStatus(SHOPMIGRATION_INITAL_FAILED);
    my $aFailedFinal = FindAllShopsWithMigrationStatus(SHOPMIGRATION_FINAL_FAILED);
    if (scalar @$aFailedPrep) {
        printf "Resetting shops with 'SHOPMIGRATION_INITAL_FAILED':\n";
        for (@$aFailedPrep) {
            my $PrepShop = $_->get('MigrationPrepShop');
            my $DeletedPrepShop = '';
            if ($PrepShop) {
                $DeletedPrepShop = sprintf(' (also PrepShop %s deleted)', $PrepShop->alias);
                $PrepShop->delete;
            }
            $_->set({ MigrationStatus => SHOPMIGRATION_INITIAL_IN_PROGRESS });
            printf "%s%s\n", $_->alias, $DeletedPrepShop;
        }
    }
    if (scalar @$aFailedFinal) {
        printf "Resetting shops with 'SHOPMIGRATION_FINAL_FAILED':\n";
        for (@$aFailedFinal) {
            $_->set({ MigrationStatus => SHOPMIGRATION_FINAL_ALLOW });
            printf "%s\n", $_->alias;
        }
    }
    exit 0;
}

sub migrateShops {
    my ($hOptions) = @_;

    my %Count = (Total => 0, Failed => 0, Done => 0);
    my ($StoreName, $Directory, $OnlyImport, $ShopAlias, $Output) =
        @{$hOptions}{qw(storename directory onlyimport shop output)};
    my $WantedStatus = $hOptions->{prep} ? SHOPMIGRATION_INITIAL_IN_PROGRESS : SHOPMIGRATION_FINAL_ALLOW;
    my $WantedString = $hOptions->{prep} ? 'INITIAL_IN_PROGRESS' : 'FINAL_ALLOW';
    my $aShopList = defined $ShopAlias ?
        findSingleShop($ShopAlias, $WantedStatus, $WantedString) :
        FindAllShopsWithMigrationStatus($WantedStatus);
    $Count{Total} = scalar @$aShopList;
    unless ($Count{Total}) {
        printf("No shops in status '%s' (%s) found!\n", $WantedStatus, $WantedString) if $Output;
        exit 0;
    }
    printf "Running %s for %s shops in directory %s\n", $0, $Count{Total}, $Directory if $Output;
    foreach my $Shop (@$aShopList) {
        eval {
            my ($SiteID, $RequestID) = ($Shop->get('GUID'), generateRequestID());
            $FakeAppSrv->{'x-epages-requestid'} = $RequestID;
            $FakeAppSrv->{'x-epages-site'} = $SiteID;
            LogContext({ 'Shop' => $Shop->alias, 'RequestID' => $RequestID }, sub {
                my $SourceShop = $hOptions->{prep} ? $Shop : $Shop->get('MigrationPrepShop');
                ExportShopDataForMigration($SourceShop, $Directory);
                PreMigrationImportTasks($Shop);
                my $TargetShop = $hOptions->{prep} ? ($OnlyImport ? $Shop->get('MigrationPrepShop') : CreatePrepShop($Shop)) : $Shop;
                ImportShopDataForMigration($TargetShop, $Directory);
                PostMigrationImportTasks($Shop);
            });
        };
        # Set error state on BaseShop in case something goes wrong so we don't try over and over
        if (ExistsError()) {
            my $Error = GetError();
            if ($hOptions->{prep}) {
                $Shop->set({ 'MigrationStatus' => SHOPMIGRATION_INITAL_FAILED });
                my $PrepShop = $Shop->get('MigrationPrepShop');
                $PrepShop->delete if (defined $PrepShop);
            } else {
                $Shop->set({ 'MigrationStatus' => SHOPMIGRATION_FINAL_FAILED, 'IsClosed' => 0 });
            }
            printf "%sMigration for shop %s store %s failed with '%s'\n",
                $hOptions->{prep} ? 'Initial ' : 'Final ', $Shop->alias, $StoreName, $Error->shortMessage;
            LogError($Error);
            $Count{Failed}++;
        } else {
            printf "%sMigration for shop %s store %s succeeded\n",
                $hOptions->{prep} ? 'Initial ' : 'Final ', $Shop->alias, $StoreName;
            $Count{Done}++;
        }
    }
    return \%Count;
}

sub initializeGenerator {
    my $Generator = String::Random->new;
    $Generator->{'A'} = [ @{$Generator->{'C'}}, @{$Generator->{'c'}}, @{$Generator->{'n'}} ];
    return $Generator;
}

# Generates a RequestID of 27 chars which looks like the Apache unique_id
sub generateRequestID {
    state $Generator = initializeGenerator();
    return $Generator->randpattern('AAAAAAAAAAAAAAAAAAAAAAAAAAA');
}

sub findSingleShop {
    my ($ShopAlias, $WantedStatus, $WantedString) = @_;

    my $ShopFolder = LoadRootObject()->child('Shops');
    usage(sprintf('No such shop: %s', $ShopAlias)) unless $ShopFolder->existsChild($ShopAlias);
    if (!$ShopFolder->existsChild($ShopAlias)) {
        printf("Shop is not in the proper  such shop: %s\n", $ShopAlias);
        exit 2;
    }
    my $Shop = $ShopFolder->child($ShopAlias);
    my ($IsStatus, $IsStatusString) = @{$Shop->get([ 'MigrationStatus', 'MigrationStatusString' ])}
        {qw(MigrationStatus MigrationStatusString)};
    if ($IsStatus != $WantedStatus) {
        printf("Shop %s is not in the proper status (Wanted: %s [%s], Is: %s [%s])\n",
            $ShopAlias, $WantedStatus, $WantedString, $IsStatus, $IsStatusString);
        exit 2;
    } elsif (!$Shop->get('IsMigrationShop')) {
        printf("Shop is not marked for migration (IsMigrationShop=0)\n", $ShopAlias);
        exit 2;
    }
    return [ $Shop ];
}

sub usage {
    my ($ErrorNote) = @_;

    print STDERR "\n$ErrorNote\n\n" if $ErrorNote;
    print <<END_USAGE;
Migrate one or multiple (accordingly flagged) shops to either prep stage (prepshops)
or final stage (transfer shops).

Usage:
    perl $0 [options] flags objects

options:
    -passwd      database user password                                    ()
    -shop        shop alias                                                ()
    -directory   output directory                                          (\$EPAGES_SHARED/Packages/shopexport/<Store>)
    -storename   name of store                                             required

flags:
    -all         search for shops with fitting status unless -shop given   (1)
    -help        show the command line options                             ()
    -prep        migrate fitting shop(s) to prep stage                     (1)
    -final       transfer fitting shop(s) to final stage                   (0)
    -onlyimport  do not create a new prepshop, only import                 (0)
    -(no)output  turn on more verbose status output                        (0)
    -printfailed list shops with failed migration status
    -resetfailed reset shops with failed migration status (to try again)

Example:
    perl $0 -storename Store -shop DemoShop -directory /tmp/shopexport
    perl $0 -storename Store -shop DemoShop -directory /tmp/shopexport -prep
    perl $0 -storename Store -shop DemoShop -directory /tmp/shopexport -final
    perl $0 -storename Store -prep -output
    perl $0 -storename Store -final -output
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main,
);

1;
