#========================================================================================
# description      Count the number of shops that are eligible for the migration to
#                   Unity according to the given feature usage.
#========================================================================================

use 5.010;
use strict;
use Getopt::Long;
use Time::HiRes qw(time);
use DE_EPAGES::Core::API::CSV::BaseExportDriver;
use DE_EPAGES::Core::API::Error qw(ExistsError GetError);
use DE_EPAGES::Core::API::Warning qw(Warning);
use DE_EPAGES::Core::API::PerlTools qw(GetCurrentDateTime AddDuration);
use DE_EPAGES::Core::API::Script qw(RunScript);
use DE_EPAGES::Database::API::Connection qw(RunOnStore GetCurrentDBHandle);
use DE_EPAGES::Object::API::Factory qw(LoadObject LoadObjectByPath LoadClassByAlias LoadRootObject);
use DE_EPAGES::Object::API::Table::Locale qw(GetInfoLocale);
use DE_EPAGES::Order::API::Shop qw(GetLatestOrderCreationDate);
use DE_EPAGES::Shop::API::Feature qw(LoadFeatureByAlias);
use DE_EPAGES::ShopMigration::API::Constants qw(SHOPMIGRATION_INITIAL_ALLOW);
use DE_EPAGES::ShopMigration::API::ShopMigrationConfig qw(ShopMigrationConfig);
use DE_EPAGES::WebInterface::API::MessageCenter qw(SynchronizeCache);
use File::Spec::Functions;
use List::MoreUtils qw(uniq);

## no critic qw( Tics::ProhibitLongLines )
my ($Output, $ShowDetail, $MaximumFeatureValuesForShopMigration, $BundleAttributeID, $MaxDepthAttributeID);
my $checkFeatureMethod = {
    Appointments                          => \&checkAppointments,
    CustomForm                            => \&checkShopFeature,
    ProductTypes                          => \&checkShopFeature,
    Variations                            => \&checkVariations,
    CustomerGroups                        => \&checkShopFeature,
    Currencies                            => \&checkShopFeature,
    ForceRegistrationBeforeOrder          => \&checkShopFeature,
    MinOrderAmount                        => \&existsMinOrderAmount,
    hasLastOrderWithinDays                => \&hasLastOrderWithinDays,
    Currencies                            => \&checkShopFeature,
    Avatax                                => sub {
        my ($Options) = @_;
        $Options->{Attribute} = 'TaxMethod';
        return hasShopAttributeDefined($Options);
    },
    PriceLists                            => sub {
        my ($Options) = @_;
        $Options->{Attribute} = 'PriceLists';
        return hasShopFolderWithChildren($Options);
    },
    ForceRegistrationBeforeOrder          => \&checkShopFeature,
    Janolaw                               => sub {
        my ($Options) = @_;
        $Options->{Attribute} = 'JanolawLastUpdateDate';
        return hasShopAttributeDefined($Options);
    },
    ProductBundles                        => \&checkBundleProducts,
    ProductCustomization                  => \&checkProductCustomization,
    ProductDownloads                      => \&checkDownloadProducts,
    NumberFormats                         => sub {
        my ($Options) = @_;
        $Options->{Attribute} = 'NumberFormats';
        return hasShopAttributeDefined($Options);
    },
    DeferedPayments                       => sub {
        my ($Options) = @_;
        $Options->{Attribute} = 'IsHirePurchaseEnabled';
        return hasPaymentsWithBoolAttributeTrue($Options);
    },
    LoginRequired2Order                   => sub {
        my ($Options) = @_;
        $Options->{Attribute} = 'LoginRequired2Order';
        return hasShopBoolAttributeWithValue($Options);
    },
    LoginRequired2InstantlyOrder          => sub {
        my ($Options) = @_;
        $Options->{Attribute} = 'LoginRequired2InstantlyOrder';
        return hasShopBoolAttributeWithValue($Options);
    },
    IsPriceOnlyForRegisteredUser          => sub {
        my ($Options) = @_;
        $Options->{Attribute} = 'IsPriceOnlyForRegisteredUser';
        return hasShopBoolAttributeWithValue($Options);
    },
    MaxCategoryDepth                      => \&checkMaxCategoryDepth,
    LastMerchantLogin                     => \&checkLastMerchantLogin,
    ActiveDeliveryMethodCustomerGroups    => sub {
        my ($Options) = @_;
        $Options->{Attribute} = 'ShippingMethods';
        $Options->{IsActivated} = '1';
        return checkForCustomerGroups($Options);
    },
    ActivePaymentMethodCustomerGroups     => sub {
        my ($Options) = @_;
        $Options->{Attribute} = 'PaymentMethods';
        $Options->{IsActivated} = '1';
        return checkForCustomerGroups($Options);
    },
    InActiveDeliveryMethodCustomerGroups  => sub {
        my ($Options) = @_;
        $Options->{Attribute} = 'ShippingMethods';
        $Options->{IsActivated} = '0';
        return checkForCustomerGroups($Options);
    },
    InActivePaymentMethodCustomerGroups   => sub {
        my ($Options) = @_;
        $Options->{Attribute} = 'PaymentMethods';
        $Options->{IsActivated} = '0';
        return checkForCustomerGroups($Options);
    },
    ProductLowOnStockAction               => sub {
        my ($Options) = @_;
        $Options->{Attribute} = 'ProductLowOnStockAction';
        return hasShopAttributeValue($Options);
    },
    CurrencyMatchesLocale                 => \&checkCurrencyMatchesLocale,
    MinOrderSubTotalAttributeName         => sub {
        my ($Options) = @_;
        $Options->{Attribute} = 'MinOrderSubTotalAttributeName';
        return hasShopAttributeValue($Options);
    },
    ShopAliasEqualWebServerScriptNamePart => \&checkShopAliasEqualWebServerScriptNamePart,
    OneSaas                               => \&checkOneSaasUser,
    ERP                                   => sub {
        my ($Options) = @_;
        $Options->{GroupAlias} = 'ERPUsers';
        return hasShopGroupWithUsers($Options);
    },
    SageOne                               => sub {
        my ($Options) = @_;
        $Options->{Attribute} = 'IsSageOneConnected';
        return hasShopBoolAttributeWithValue($Options);
    },
    UniversalPortalSites                  => sub {
        my ($Options) = @_;
        $Options->{ChildAlias} = 'UniversalPortalSites';
        return hasShopChildWithChildren($Options);
    },
    ELogistics                            => \&checkELogistics,
    CouponCampaigns                       => \&checkCouponCampaigns,
};
## critic

RunScript('Sub' => \&Main);

sub Main {

    local $| = 1;

    my ($Password, $StoreName, $Help, $PrintList, $ShopType, $CSVFileName, $SetFlag, $ShopAlias);
    $MaximumFeatureValuesForShopMigration = getMaximumFeatureValuesForShopMigration();
    my %Options = (
        'help'        => \$Help,
        'list'        => \$PrintList,
        'shoptype=s'  => \$ShopType,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shop=s'      => \$ShopAlias,
        'output!'     => \$Output,
        'csv=s'       => \$CSVFileName,
        'setflag'     => \$SetFlag,
        'detail'      => \$ShowDetail,
    );

    usage("Unkown parameter specified.\n") unless GetOptions(%Options);
    usage() if $Help;

    unless (defined $StoreName) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub {
            use constant LINESIZE => 80;
            usage("Cannot use -setflag if shop attribute 'MigrationStatus' does not exist.\n") if ($SetFlag && !existsStatusFlag());
            usage("Cannot use -setflag if shop attribute 'MigrationFailReasonsString' does not exist.\n") if ($SetFlag && !existsReasonString());
            my $UnityMigratableShops = [];
            my $AllNonUnityShopIDs;
            if (defined $ShopAlias) {
                usage(sprintf("\nNo such shop: %s!\n\n", $ShopAlias)) unless LoadRootObject()->child('Shops')->existsChild($ShopAlias);
                my $Shop = LoadRootObject()->child('Shops')->child($ShopAlias);
                usage(sprintf("\nThis is a unity shop: %s!\n\n", $ShopAlias)) if $Shop->get('IsUnityRegistered');
                $AllNonUnityShopIDs = [ $Shop->id ];
            } else {
                $AllNonUnityShopIDs = getAllNonUnityShopIDs($ShopType);
            }
            my @Limitations = ();
            push @Limitations, "of shoptype $ShopType" if defined $ShopType;
            push @Limitations, 'ClosedByProvider=0' if defined $MaximumFeatureValuesForShopMigration->{ClosedByProvider};
            push @Limitations, 'ClosedByMerchant=0' if defined $MaximumFeatureValuesForShopMigration->{ClosedByMerchant};
            push @Limitations, 'IsInternalTestShop=0' if defined $MaximumFeatureValuesForShopMigration->{IsInternalTestShop};
            push @Limitations, 'IsTrialShop=0' if defined $MaximumFeatureValuesForShopMigration->{IsTrialShop};
            printf "%s\nTotal Non-Unity Shops%s: %s\n",
                '-' x LINESIZE, scalar @Limitations ? ' (' . join(', ', @Limitations) . ')' : '', scalar @$AllNonUnityShopIDs;
            my ($Counter, @WaitChar) = qw(0 - \ | /);
            for (@$AllNonUnityShopIDs) {
                print $WaitChar[$Counter++ % scalar @WaitChar], chr(8) if $Output;
                my $Shop = LoadObject($_);
                my $Result = runAllChecks($Shop);
                push @$UnityMigratableShops, @$Result;
                setShopFlags($Shop, $Result) if $SetFlag;
            }
            my $UnityMigratableShopsCombinedReasons = {};
            my @Fields = qw(ShopAlias IsMigratable MigrationStatus ShopType IsInternalTestShop DomainName);
            for (@$UnityMigratableShops) {
                my $Name = $_->[0];
                unless (exists $UnityMigratableShopsCombinedReasons->{$Name}) {
                    @{$UnityMigratableShopsCombinedReasons->{$Name}}{@Fields} = @$_[0, 1, 2, 3, 4, 5];
                    $UnityMigratableShopsCombinedReasons->{$Name}{SoftFailReasons} = [];
                    $UnityMigratableShopsCombinedReasons->{$Name}{HardFailReasons} = [];
                }
                my $ReasonType = sprintf('%sFailReasons', $_->[8]);
                push @{$UnityMigratableShopsCombinedReasons->{$Name}{$ReasonType}}, $_->[6];
            }
            my $CountMigratable = scalar uniq map {$_->[0]} grep {$_->[1] == 1;} @$UnityMigratableShops;
            my $CountAllNonUnityShopIDs = scalar @$AllNonUnityShopIDs;
            ## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
            printf "Count of Unity-migratable shops: %s\nThis is about %.2f %%\n",
                $CountMigratable, $CountAllNonUnityShopIDs ? (100 * $CountMigratable / $CountAllNonUnityShopIDs) : 0, '-' x LINESIZE;
            ## critic
            printList($UnityMigratableShopsCombinedReasons) if ($PrintList);
            writeCSV($CSVFileName, $UnityMigratableShops) if ($CSVFileName);
        }
    );
    eval {SynchronizeCache();};
    Warning(GetError()) if ExistsError();

    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1 if $Output;
}

sub getMaximumFeatureValuesForShopMigration {
    my $MaximumFeatureValues = {};
    my $MigConfig = ShopMigrationConfig();
    @$MaximumFeatureValues{qw(Soft Hard)} = (
        $MigConfig->section('ShopMigrationCheck.Soft'),
        $MigConfig->section('ShopMigrationCheck.Hard')
    );
    foreach my $Type (qw(Soft Hard)) {
        foreach my $Name (keys %{$MaximumFeatureValues->{$Type}}) {
            @{$MaximumFeatureValues->{$Name}}{ qw(Name Type Allowed) } = (
                $Name, $Type, $MaximumFeatureValues->{$Type}{$Name},
            );
        }
    }
    delete $MaximumFeatureValues->{Soft};
    delete $MaximumFeatureValues->{Hard};
    return $MaximumFeatureValues;
}

sub existsStatusFlag {
    state $ExistsFlag = LoadClassByAlias('Shop')->existsAttribute('MigrationStatus');
    return $ExistsFlag;
}

sub existsReasonString {
    state $ExistsString = LoadClassByAlias('Shop')->existsAttribute('MigrationFailReasonsString');
    return $ExistsString;
}

sub setShopFlags {
    my ($Shop, $Result) = @_;
    my @HardFails = grep {$_->[8] eq 'Hard';} @$Result;
    my $HasHardFails = scalar @HardFails ? 1 : 0;
    if (!$HasHardFails) {
        my $CurrentValue = $Shop->get('MigrationStatus');
        if ($CurrentValue > SHOPMIGRATION_INITIAL_ALLOW) {
            printf "Shop %s has already started migration (Status %s) refusing to change state\n",
                $Shop->alias, $CurrentValue;
        } else {
            $Shop->set({ IsMigrationShop => 1, MigrationStatus => SHOPMIGRATION_INITIAL_ALLOW })
        }
    }
    my $CombinedString = join(',', map {join(':', $_->[6], $_->[7], $_->[8]);} @$Result);
    $Shop->set({ MigrationFailReasonsString => $CombinedString });
}

sub printList {
    my ($UnityMigratableShopsCombinedReasons) = @_;

    my @DisplayOrder = qw(ShopAlias ShopType HardReasons SoftReasons DomainName);
    foreach my $IsMigratable (qw(0 1)) {
        printf "\n%s\n%sMigratable Shops\n%s\n\n", '-' x LINESIZE, ($IsMigratable ? '' : 'Non-'), '-' x LINESIZE;
        my @SubDisplayOrder = $IsMigratable ? grep {$_ ne 'HardReasons'} @DisplayOrder : @DisplayOrder;
        printf "%s\n", join(',', @SubDisplayOrder);
        foreach my $ShopAlias (keys %$UnityMigratableShopsCombinedReasons) {
            foreach my $Entry ($UnityMigratableShopsCombinedReasons->{$ShopAlias}) {
                next unless $Entry->{IsMigratable} == $IsMigratable;
                $Entry->{ShopAlias} = $ShopAlias;
                foreach my $ReasonType (qw(HardReasons SoftReasons)) {
                    my $ArrayName = $ReasonType;
                    $ArrayName =~ s/Reasons/FailReasons/;
                    if (scalar @{$Entry->{$ArrayName}}) {
                        $Entry->{$ReasonType} = sprintf('%s', join('|', @{$Entry->{$ArrayName}}));
                    } else {
                        $Entry->{$ReasonType} = 'n/a';
                    }
                }
                printf("%s\n", join ',', @$Entry{@SubDisplayOrder});
            }
        }
    }
}

sub writeCSV {
    my ($CSVFileName, $MigratableShops) = @_;

    $CSVFileName .= '.csv' unless ($CSVFileName =~ /[.]csv$/);
    my $Exporter = DE_EPAGES::Core::API::CSV::BaseExportDriver->new();
    $Exporter->exportCSV($CSVFileName, undef, [ qw(ShopAlias IsMigratable MigrationStatus ShopType IsInternalTestShop DomainName FailReason Allowed FailReasonType) ], $MigratableShops);
    printf("CSV file written to '%s'\n", $CSVFileName);
}

sub getAllNonUnityShopIDs {
    my ($ShopType) = @_;

    # construct SQL statement to get all non-Unity shops
    my (@WhereClauses, @Parameter);
    my $FeatureID = LoadFeatureByAlias('Unity')->id;
    push @WhereClauses, 'sf.featureid = ?';
    push @Parameter, $FeatureID;
    push @WhereClauses, 'sf.maxval = 0';
    if (defined $MaximumFeatureValuesForShopMigration->{ClosedByProvider}) {
        push @WhereClauses, 's.closedbyprovider = 0';
    }
    if (defined $MaximumFeatureValuesForShopMigration->{ClosedByMerchant}) {
        push @WhereClauses, 's.closedbymerchant = 0';
    }
    if (defined $MaximumFeatureValuesForShopMigration->{IsInternalTestShop}) {
        push @WhereClauses, 's.isinternaltestshop = 0';
    }
    if (defined $MaximumFeatureValuesForShopMigration->{IsTrialShop}) {
        push @WhereClauses, 's.istrialshop = 0';
    }
    push @WhereClauses, 'o.alias not like \'PreCreated%\'';

    # optionally restrict SQL statement to a given shoptype
    my $JoinClause = '';
    if (defined $ShopType) {
        my $ShopTypeRefID = LoadObjectByPath('/ShopTypeRefs')->child($ShopType)->id;
        push @WhereClauses, 's.shoptyperefid = ?';
        push @Parameter, $ShopTypeRefID;
    }
    $JoinClause = 'INNER JOIN shop s ON sf.shopid = s.shopid ';
    $JoinClause .= 'INNER JOIN object o ON sf.shopid = o.objectid ';

    # join SQL statement
    ##no critic (ProhibitVariablesInSQLStatements)
    my $sql = 'SELECT sf.shopid FROM shopfeature sf ' .
        $JoinClause .
        'WHERE ' . join(' AND ', @WhereClauses);

    # execute SQL
    my $dbi = GetCurrentDBHandle();
    my $aaRows = $dbi->execute($sql, \@Parameter, [ 'Integer' ]);
    my @ShopIDs = map {$_->[0]} @$aaRows;
    return \@ShopIDs;
}

sub runAllChecks {
    my ($Shop) = @_;
    my @AllResults = ();

    printf "Working on %s\n", $Shop->alias if $Output;
    my $Options = {
        Shop                => $Shop,
        FeatureCurrentValue => $Shop->get('FeatureCurrentValue'),
    };
    my @MoreFields = ($Shop->get('ShopTypeRef')->alias, $Shop->get('IsInternalTestShop'), $Shop->get('WebUrl'));
    state $ExistsFlag = LoadClassByAlias('Shop')->existsAttribute('MigrationStatus');
    unshift(@MoreFields, $ExistsFlag ? $Shop->get('MigrationStatus') : 'n/a');
    for my $FeatureOrCheck (keys %$checkFeatureMethod) {
        my $Start = time();
        $Options->{FailReason} = undef;
        my $Skip = 0;
        if (!exists $MaximumFeatureValuesForShopMigration->{$FeatureOrCheck} ||
            !keys %{$MaximumFeatureValuesForShopMigration->{$FeatureOrCheck}}) {
            $Skip = 1;
        }
        my $Sub = $checkFeatureMethod->{$FeatureOrCheck};
        $Options->{FeatureOrCheck} = $FeatureOrCheck;
        $Options->{Type} = $MaximumFeatureValuesForShopMigration->{$FeatureOrCheck}{Type};
        $Options->{Allowed} = $MaximumFeatureValuesForShopMigration->{$FeatureOrCheck}{Allowed};
        $Sub->($Options) unless $Skip;
        if ($Output && ($ShowDetail || defined $Options->{FailReason})) {
            printf "- check: %s%s%s (took %.4f s)\n",
                $FeatureOrCheck,
                $Skip ? ' (skipped/missing in shopmigration.conf)' : '',
                defined $Options->{FailReason} ? sprintf(' FailReason: %s', $Options->{FailReason}) : '',
                time() - $Start;
        }
        if (defined $Options->{FailReason}) {
            push @AllResults, [ $Shop->alias, 0, @MoreFields, $FeatureOrCheck, $Options->{Allowed}, $Options->{Type} ];
        }
    }
    # update all IsMigratable flags if only soft fails are reported
    my @HardFails = grep {$_->[8] eq 'Hard';} @AllResults;
    unless (scalar @HardFails) {
        for (@AllResults) {$_->[1] = 1;}
    }
    if (not scalar @AllResults) {
        push @AllResults, [ $Shop->alias, 1, @MoreFields, 'n/a', 'n/a', 'n/a' ];
    }
    return \@AllResults;
}

sub checkShopFeature {
    my ($Options) = @_;

    my $Name = $Options->{FeatureOrCheck};
    my $Allowed = $Options->{Allowed};
    my $FeatureCurrentValue = $Options->{FeatureCurrentValue}{$Name};
    if ($FeatureCurrentValue > $Allowed) {
        $Options->{FailReason} = sprintf('Feature %s CurrentValue: %s > Allowed: %s', $Name, $FeatureCurrentValue, $Allowed);
    }
}

sub checkBundleProducts {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    if (not defined $BundleAttributeID) {
        $BundleAttributeID = LoadClassByAlias('Product')->attribute('IsBundleProduct')->id;
    }
    my $sql = <<SQL;
select count(1)
from booleanattribute ba
         inner join object o on o.objectid=ba.objectid and o.siteid=? and ba.attributeid=?;
SQL
    my @Parameter = ($Shop->id, $BundleAttributeID);
    my $dbi = GetCurrentDBHandle();
    my $aaRows = $dbi->execute($sql, \@Parameter, [ 'Integer' ]);
    if ($aaRows->[0][0] != 0) {
        my $sql_detail = <<SQL;
select o.alias
from product p join object o on o.objectid=p.productid
inner join  booleanattribute ba on o.objectid=ba.objectid
where o.siteid=? and ba.attributeid=?
SQL
        my $aaDetails = $dbi->execute($sql_detail, \@Parameter, [ 'Integer' ]);
        my $DetailInfo = sprintf(' (Products: %s)', join(', ', map {$_->[0]} @$aaDetails));
        $Options->{FailReason} = sprintf('Feature %s failed, shop has bundle products%s', $Name, $DetailInfo);
    }
}

sub checkAppointments {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $sql = <<SQL;
select count(1)
from product
where shopid=? and isappointment=1;
SQL
    my @Parameter = ($Shop->id);
    my $dbi = GetCurrentDBHandle();
    my $aaRows = $dbi->execute($sql, \@Parameter, [ 'Integer' ]);
    if ($aaRows->[0][0] != 0) {
        my $sql_detail = <<SQL;
select o.alias
from product p join object o on o.objectid=p.productid
where shopid=? and isappointment=1;
SQL
        my $aaDetails = $dbi->execute($sql_detail, \@Parameter, [ 'Integer' ]);
        my $DetailInfo = sprintf(' (Products: %s)', join(', ', map {$_->[0]} @$aaDetails));
        $Options->{FailReason} = sprintf('Feature %s failed, shop has appointment products%s', $Name, $DetailInfo);
    }
}

sub checkDownloadProducts {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $sql = <<SQL;
select count(1)
from product
where shopid=? and isdownloadproduct=1;
SQL
    my @Parameter = ($Shop->id);
    my $dbi = GetCurrentDBHandle();
    my $aaRows = $dbi->execute($sql, \@Parameter, [ 'Integer' ]);
    if ($aaRows->[0][0] != 0) {
        my $sql_detail = <<SQL;
select o.alias
from product p join object o on o.objectid=p.productid
where shopid=? and isdownloadproduct=1;
SQL
        my $aaDetails = $dbi->execute($sql_detail, \@Parameter, [ 'Integer' ]);
        my $DetailInfo = sprintf(' (Products: %s)', join(', ', map {$_->[0]} @$aaDetails));
        $Options->{FailReason} = sprintf('Feature %s failed, shop has download products%s', $Name, $DetailInfo);
    }
}

sub checkProductCustomization {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $sql = <<SQL;
select count(1)
from product
where shopid=?
  and iscustomizable=1;
SQL
    my @Parameter = ($Shop->id);
    my $dbi = GetCurrentDBHandle();
    my $aaRows = $dbi->execute($sql, \@Parameter, [ 'Integer' ]);
    if ($aaRows->[0][0] != 0) {
        my $sql_detail = <<SQL;
select o.alias
from product p join object o on p.productid=o.objectid
where p.shopid=?
  and p.iscustomizable=1;
SQL
        my $aaDetails = $dbi->execute($sql_detail, \@Parameter, [ 'Integer' ]);
        my $DetailInfo = sprintf(' (Products: %s)', join(', ', map {$_->[0]} @$aaDetails));
        $Options->{FailReason} = sprintf('Feature %s failed, shop has customizable products%s', $Name, $DetailInfo);
    }
}

sub checkVariations {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $Allowed = $Options->{Allowed};
    my $FeatureCurrentValue = $Options->{FeatureCurrentValue}{$Name};
    # only run the detail query if we have a shop with more then allowed Variations
    # but that does not mean it has them all selected/used for variation products in which case is *should* be fine
    if ($FeatureCurrentValue > $Allowed) {
        my @Parameter = ($Shop->id);
        my $sql_masterproducts = <<SQL;
select p.productid
from product p
where p.shopid=?
  and p.superproductid IS NULL
  AND EXISTS ( SELECT sp.productid FROM product sp WHERE sp.superproductid=p.productid)
SQL
        my $dbi = GetCurrentDBHandle();
        my $aaMasterProducts = $dbi->execute($sql_masterproducts, \@Parameter, [ 'Integer' ]);
        my $DetailInfo = '';
        my @BadProducts = ();
        foreach my $aRow (@$aaMasterProducts) {
            my $Product = LoadObject($aRow->[0]);
            my $Count = scalar @{$Product->get('VariationTypeValues')};
            if ($Count > $Allowed) {
                push @BadProducts, { Count => $Count, Alias => $Product->alias };
            }
            $DetailInfo = sprintf(' (Products: %s)', join(', ',
                map {sprintf('%s (UsedVarationAttributes=%s)', $_->{Alias}, $_->{Count})} @BadProducts
            ));
            if (scalar @BadProducts) {
                $Options->{FailReason} = sprintf('Feature %s CurrentValue: %s > Allowed: %s%s',
                    $Name, $FeatureCurrentValue, $Allowed, $DetailInfo);
            }
        }
    }
}

sub hasPaymentsWithBoolAttributeTrue {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $Attribute = $Options->{Attribute};
    my $CurrentCount = 0;
    foreach my $Payment (@{$Shop->get('PaymentMethods')}) {
        if ($Payment->get($Attribute)) {
            $CurrentCount++;
            last;
        }
    }
    if ($CurrentCount) {
        $Options->{FailReason} = sprintf('Check \'%s\' failed, a paymentmethod has boolean Attribute %s with true value', $Name, $Attribute);
    }
}

sub hasShopAttributeValue {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $Allowed = $Options->{Allowed};
    my $Attribute = $Options->{Attribute};
    if ($Shop->class->existsAttribute($Attribute) && $Shop->get($Attribute) ne $Allowed) {
        $Options->{FailReason} = sprintf('Check \'%s\' failed, shop has Attribute %s with not allowed value %s (should be %s)', $Name, $Attribute, $Shop->get($Attribute), $Allowed);
    }
}

sub hasShopBoolAttributeWithValue {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $Attribute = $Options->{Attribute};
    if ($Shop->class->existsAttribute($Attribute) && $Shop->get($Attribute)) {
        $Options->{FailReason} = sprintf('Check \'%s\'  failed, shop has boolean Attribute %s with true value', $Name, $Attribute);
    }
}

sub hasShopAttributeDefined {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $Attribute = $Options->{Attribute};
    if ($Shop->class->existsAttribute($Attribute) && defined $Shop->get($Attribute)) {
        $Options->{FailReason} = sprintf('Check \'%s\' failed, shop has Attribute %s defined', $Name, $Attribute);
    }
}

sub hasShopFolderWithChildren {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $Attribute = $Options->{Attribute};
    if (scalar @{$Shop->get($Attribute)}) {
        $Options->{FailReason} = sprintf('Check \'%s\' failed, shop has non-empty folder %s', $Name, $Attribute);
    }
}

sub hasShopChildWithChildren {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $ChildAlias = $Options->{ChildAlias};
    if ($Shop->existsChild($ChildAlias) && scalar @{$Shop->child($ChildAlias)->children}) {
        $Options->{FailReason} = sprintf('Check \'%s\' failed, shop has child object %s with children', $Name, $ChildAlias);
    }
}

sub hasShopGroupWithUsers {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $Groups = $Shop->child('Groups');
    my $GroupAlias = $Options->{GroupAlias};
    if (defined $Groups && $Groups->existsChild($GroupAlias) && scalar @{$Groups->child($GroupAlias)->directMembers()}) {
        $Options->{FailReason} = sprintf('Check \'%s\' failed, shop has an group %s with children', $Name, $GroupAlias);
    }
}

sub existsMinOrderAmount {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $ExistsMinOrderAmount = 0;
    my $MinOrderAmounts = $Shop->get('MinOrderAmounts');
    foreach my $MinOrderAmount (@$MinOrderAmounts) {
        if (defined $MinOrderAmount->{'Value'}) {
            $ExistsMinOrderAmount = 1;
            last;
        }
    }
    if ($ExistsMinOrderAmount) {
        $Options->{FailReason} = sprintf('Check \'%s\' failed, shop has MinOrderAmounts', $Name);
    }
}

sub hasLastOrderWithinDays {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $Allowed = $Options->{Allowed};
    my $OrderWithinTheLast30Days = 0;
    my $LatestOrderCreationDate = GetLatestOrderCreationDate($Shop);
    if (defined $Allowed && $Allowed > -1) {
        if (defined $LatestOrderCreationDate) {
            my $CompareDate = AddDuration($LatestOrderCreationDate, 'days', $Allowed);
            my $Now = GetCurrentDateTime();
            # Workaround for "Invalid local time for date in time zone: Europe/Berlin"
            $CompareDate->set(hour => 0, minute => 0, second => 1);
            $Now->set(hour => 0, minute => 0, second => 1);
            if ($CompareDate > $Now) {
                $OrderWithinTheLast30Days = 1;
            }
        }
    } else {
        $OrderWithinTheLast30Days = 1;
    }
    if (!$OrderWithinTheLast30Days) {
        $Options->{FailReason} = sprintf('Check \'%s\' failed, shop has no order within last %s days', $Name, $Allowed);
    }
}

sub checkMaxCategoryDepth {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $Allowed = $Options->{Allowed};
    if (not defined $MaxDepthAttributeID) {
        $MaxDepthAttributeID = LoadClassByAlias('Category')->attribute('CategoryDepth')->id;
    }
    my $sql = <<SQL;
select max(value)
from integerattribute ia
         inner join object o on o.objectid=ia.objectid and o.siteid=?
where attributeid=?;
SQL
    my @Parameter = ($Shop->id, $MaxDepthAttributeID);
    my $dbi = GetCurrentDBHandle();
    my $aaRows = $dbi->execute($sql, \@Parameter, [ 'Integer' ]);
    my $MaxDepth = $aaRows->[0][0];
    if ($MaxDepth > $Allowed) {
        $Options->{FailReason} = sprintf('Check \'%s\' failed, shop has max category path depth of %s', $Name, $MaxDepth);
    }
}

sub checkLastMerchantLogin {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $Allowed = $Options->{Allowed};
    my $LoginOkay = 0;
    my $LastMerchantLogin = $Shop->get('LastMerchantLogin');
    if ($Allowed > -1) {
        if (defined $LastMerchantLogin) {
            my $CompareDate = AddDuration($LastMerchantLogin, 'days', $Allowed);
            my $Now = GetCurrentDateTime();
            if ($CompareDate > $Now) {
                $LoginOkay = 1;
            }
        }
    } else {
        $LoginOkay = 1;
    }
    if (!$LoginOkay) {
        $Options->{FailReason} = sprintf('Check \'%s\' failed shop has no login within the last %s days', $Name, $Allowed);
    }
}

sub checkForCustomerGroups {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $Attribute = $Options->{Attribute};
    my $IsActivated = $Options->{IsActivated};
    my $CurrentCount = 0;
    my $FailElementName = '';
    foreach my $Element (@{$Shop->get($Attribute)}) {
        next unless ($Element->get('IsActivated') eq $IsActivated);
        if (scalar(@{$Element->get('CustomerGroups')})) {
            $FailElementName = $Element->alias;
            $CurrentCount++;
            last;
        }
    }
    if ($CurrentCount) {
        $Options->{FailReason} = sprintf('Check \'%s\' failed, %s \'%s\' has CustomerGroups', $Name, $Attribute, $FailElementName);
    }
}

sub checkCurrencyMatchesLocale {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};

    my $CurrencyID = $Shop->get('CurrencyID');
    my $LocaleID = $Shop->get('LocaleID');
    my $hLocale = GetInfoLocale($LocaleID);
    if ($hLocale->{CurrencyID} ne $CurrencyID) {
        $Options->{FailReason} = sprintf('Check \'%s\' failed, Locale \'%s\' and Currency \'%s\' do not match', $Name, $LocaleID, $CurrencyID);
    }
}

sub checkShopAliasEqualWebServerScriptNamePart {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my @Keys = qw(Alias WebServerScriptNamePart);
    my ($Alias, $WSSNP) = @{$Shop->get(\@Keys)}{@Keys};
    if ($Alias ne $WSSNP) {
        $Options->{FailReason} = sprintf('Check \'%s\' failed, ShopAlias \'%s\' and WebServerScriptNamePart \'%s\' do not match', $Name, $Alias, $WSSNP);
    }
}

sub checkOneSaasUser {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    if (defined $Shop->findPath('Users/OneSaasConnector')) {
        $Options->{FailReason} = sprintf('Check \'%s\' failed, shop has a OneSaasConnector user', $Name);
    }
}

sub checkELogistics {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $aShippingMethods = $Shop->get('ShippingMethods');
    foreach (grep {defined $_->get('SelectedShippingFulfillmentMethod')} @{$aShippingMethods}) {
        if ($_->get('SelectedShippingFulfillmentMethod')->get('ShippingFulfillmentType')->alias eq 'ELogistics') {
            $Options->{FailReason} = sprintf('Check \'%s\' failed, shop uses ShippingFulfillmentTypes ELogistics', $Name);
            last;
        }
    }
}

sub checkCouponCampaigns {
    my ($Options) = @_;

    my $Shop = $Options->{Shop};
    my $Name = $Options->{FeatureOrCheck};
    my $Allowed = $Options->{Allowed};
    my $sql = <<SQL;
SELECT COUNT(*)
FROM couponcampaign cca
  INNER JOIN shop s ON (cca.shopid=s.shopid AND s.closedbymerchant=0 AND s.closedbyprovider=0 AND s.isinternaltestshop=0 AND s.istrialshop=0)
WHERE EXISTS (SELECT 1 FROM couponcode cco WHERE cco.couponcampaignid=cca.couponcampaignid and isinvalidated=0)
  AND (cca.timeto IS NULL OR cca.timeto > NOW())
  AND cca.shopid=?;
SQL
    my @Parameter = ($Shop->id);
    my $dbi = GetCurrentDBHandle();
    my $aaRows = $dbi->execute($sql, \@Parameter, [ 'Integer' ]);
    my $Count = $aaRows->[0][0];
    if ($Count > $Allowed) {
        $Options->{FailReason} = sprintf('Check \'%s\' failed, shop has %s coupon campaigns', $Name, $Count);
    }
}

sub usage {
    my ($ErrorNote) = @_;

    print $ErrorNote if $ErrorNote;
    print <<END_USAGE;
Description:
    Count the number of shops that are eligible for the migration to
    Unity according to the given feature usage.

Usage:
    \$PERL $0 [options] [flags] datafiles

options:
    -shoptype   alias of a specific shoptype to filter for (optional)
    -passwd     database user password                     ()
    -storename  name of store                              ()
    -csv        export content as csv to named file        (optional)
    -shop       only check this shop                       (optional)

flags:
    -list       print a list of shop aliases additional to the count
    -output     detailed debug output for each check run that fails
    -detail     detailed debug output for each check run that passes or fails
    -help       show the command line options
    -setflag    also set the shop attributes 'MigrationStatus' accordingly

Example:
    \$PERL $0 -storename Store
    \$PERL $0 -storename Store -output
    \$PERL $0 -storename Store -list
    \$PERL $0 -storename Store -csv shoplist.csv
END_USAGE
    exit 2;
}

