#========================================================================================
# package      DE_EPAGES::ShopMigration::t::CrossSellingsExportImport
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::ShopMigration::API::JSONLines::CrossSellingsExportImport
#========================================================================================
package DE_EPAGES::ShopMigration::t::CrossSellingsExportImport;
use base DE_EPAGES::Test::API::StoreUnitTest;

use strict;
use utf8;
use warnings;
use File::Spec::Functions;
use DE_EPAGES::Core::API::File qw(GetFileContent GetFileContentLines ExistsFile DeleteDirectory DeleteFile Text2Unix GetFileNames);
use DE_EPAGES::CrossSelling::API::CrossSelling qw(FindCrossSelling); ##no critic (RequireCorrectImports)
use DE_EPAGES::Object::API::Factory qw(LoadRootObject LoadClassByAlias LoadObjectByPath);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw(ImportObjects);
use DE_EPAGES::ShopMigration::API::JSON qw(DecodeJSONArray);
use DE_EPAGES::ShopMigration::API::JSONLines::CrossSellingsExportImport qw(ExportCrossSellingsJSON ImportCrossSellingsJSON);
use DE_EPAGES::ShopMigration::API::CrossSellingsCommon qw(GetCrossSellingIDs);
use DE_EPAGES::ShopMigration::API::MigrationShop qw(CreatePrepShop ExistsPrepShop SetShopFeatureValues);
use File::Temp qw(tempdir);
use Test::More;
use Test::Differences qw(context_diff eq_or_diff);

my $TmpDir = tempdir();

if ($ENV{SKIP_IN_PR_TESTS}) {
    plan skip_all => 'Skipping it because SKIP_IN_PR_TESTS == 1';
}

sub test {
    context_diff();
    testExportWithCrossSellingsExportImportBase2Prep();
    testImportWithCrossSellingsExportImportBase2Prep();
    done_testing();
}

my %TestData = (
    TestShop      => undef,
    TestShopAlias => 'TestShop',
    ImportPath    => '/Test/Shops/TestShop',
);
use constant FILENAME => 'crosssellings.json';
use constant FILE_DIR => [ qw(ShopData.d export TestShop) ];
use constant FILE_LINES => 4;
use constant IMPORT_CROSSSELLINGS_PREP => 4;

sub testExportWithCrossSellingsExportImportBase2Prep {
    my $Shop = $TestData{TestShop};

    note('test CrossSellingsExportDriver (Base2Prep)');
    isnt($Shop, undef, 'Testshop exists');
    is(scalar @{$Shop->get('Languages')}, 1, 'Testshop has now one Language');
    is(scalar @{$Shop->get('SiteLocales')}, 1, 'Testshop has one SiteLocale');
    ok(!$Shop->get('IsMigrationPrepShop'), 'Testshop has IsMigrationPrepShop=0');

    my $Exporter = ExportCrossSellingsJSON($Shop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);

    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, FILE_LINES, 'file lines count');
    ok($Exporter->fileName =~ /crosssellings.json/, 'export file name matches crosssellings.json');

    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(catfile(@{FILE_DIR()}, 'base2prep_' . FILENAME), { encoding => 'utf8' });
    # Order of the crossselling items in the json file can arbitrarily differ depending on how database returns it
    my $ahExportedJSON =
        [ sort {$a->{Attributes}{Product} cmp $b->{Attributes}{Product}} @{DecodeJSONArray($$sFileContent)} ];
    my $ahExpectedJSON =
        [ sort {$a->{Attributes}{Product} cmp $b->{Attributes}{Product}} @{DecodeJSONArray($$sExpected)} ];
    eq_or_diff($ahExportedJSON, $ahExpectedJSON, 'file content matches');
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub testImportWithCrossSellingsExportImportBase2Prep {

    my $aExpectedValues = [
        {
            'Product'       => 'Products/cg_0100504001',
            'TargetProduct' => 'Products/lt_0401107001',
            'Position'      => '10',
        },
        {
            'Product'       => 'Products/cg_0100504001',
            'TargetProduct' => 'Products/hm_0100401001',
            'Position'      => '20',
        },
        {
            'Product'       => 'Products/be_40401',
            'TargetProduct' => 'Products/be_40402',
            'Position'      => '10',
        },
        {
            'Product'       => 'Products/be_40402',
            'TargetProduct' => 'Products/be_40401',
            'Position'      => '10',
        },
    ];

    my $PrepShop = $TestData{PrepShop};
    my $DirName = catdir('ShopData.d', 'import');
    note('test CrossSellingsImportHandler (Base2Prep)');
    isnt($PrepShop, undef, 'Prepshop exists');

    my $CurrentCrossSellingIDs = GetCrossSellingIDs($PrepShop);
    is(scalar @$CurrentCrossSellingIDs, 0, 'no cross sellings on PrepShop');

    my $Importer = ImportCrossSellingsJSON($PrepShop, $DirName);
    $CurrentCrossSellingIDs = GetCrossSellingIDs($PrepShop);
    is(scalar @$CurrentCrossSellingIDs, IMPORT_CROSSSELLINGS_PREP, 'imported cross sellings count');
    ok($Importer->fileName() =~ /base2prep_${\FILENAME}/, 'ImportFileName was constructed properly');

    foreach my $hEntry (@$aExpectedValues) {
        my $Product = LoadObjectByPath($hEntry->{'Product'}, $PrepShop->id);
        my $TargetProduct = LoadObjectByPath($hEntry->{'TargetProduct'}, $PrepShop->id);
        # exists crossselling?
        my $CrossSelling = FindCrossSelling($Product, $TargetProduct, 'CrossSelling');
        is(defined $CrossSelling, 1, sprintf('CrossSelling %s - %s exists', $hEntry->{'Product'}, $hEntry->{'TargetProduct'}));
        my $hInfoCrossSelling = $CrossSelling->get([ qw(Position) ]);
        foreach my $key (keys %$hInfoCrossSelling) {
            is($hInfoCrossSelling->{$key}, $hEntry->{$key}, "attribute $key matches.");
        }
    }
}

# We need to create a fake ShopTypeRef UnityDE + ECom5000 in case they don't exists
# otherwise the Mapper won't work
sub init {
    my $self = shift;

    $self->SUPER::init;
    my $ShopAlias = $TestData{TestShopAlias};
    my $System = LoadRootObject();
    my $Shop;
    my $TestFolder = $System->folder('Test')->folder('Shops');
    my $ShopTypeRefFolder = $System->folder('ShopTypeRefs');
    my $Class = LoadClassByAlias('ShopTypeRef');
    $Class->insertObject({ Alias => 'UnityDE', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('UnityDE'));
    $Class->insertObject({ Alias => 'ECom5000', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('ECom5000'));
    if ($TestFolder->existsChild($ShopAlias)) {
        $Shop = $TestFolder->child($ShopAlias);
    } else {
        note(sprintf('Creating %s', $ShopAlias));
        $Shop = $TestFolder->insertChild('Shop', {
            Alias                   => $ShopAlias,
            ShopTypeRef             => $ShopTypeRefFolder->child('ECom5000'),
            WebServerScriptNamePart => $ShopAlias,
        });
        SetShopFeatureValues($Shop, [
            { Alias => 'Regions', MaxValue => '2' },
            { Alias => 'Languages', MaxValue => '15' },
            { Alias => 'Pages', MaxValue => '100' },
            { Alias => 'Subcategories', MaxValue => '100' },
            { Alias => 'Products', MaxValue => '100' },
            { Alias => 'ProductMaxSubProducts', MaxValue => '1000' },
            { Alias => 'ProductTypes', MaxValue => '100' },
            { Alias => 'ProductTypeAttributes', MaxValue => '100' },
            { Alias => 'Variations', MaxValue => '100' },
            { Alias => 'VariationAttributeValues', MaxValue => '100' },
            { Alias => 'ManualCrossSelling', MaxValue => '1' },
        ]);
    }
    note('Creating PrepShop') if (!ExistsPrepShop($Shop));
    $TestData{TestShop} = $Shop;
    $TestData{PrepShop} = ExistsPrepShop($Shop) ? $Shop->get('MigrationPrepShop') : CreatePrepShop($Shop);
    foreach my $JSONFile (@{GetFileNames('ShopData.d', { 'recursive' => 1 })}) {
        Text2Unix($JSONFile) if ($^O eq 'linux');
    }
    ImportObjects('ImportFiles/CrossSellingsBase.xml', $TestData{TestShop}->pathString());
    ImportObjects('ImportFiles/CrossSellingsPrep.xml', $TestData{PrepShop}->pathString());
    return;
}

sub cleanup {
    my $self = shift;

    my $ShopAlias = $TestData{TestShopAlias};
    my $TestFolder = LoadRootObject()->folder('Test')->folder('Shops');
    if ($TestFolder->existsChild($ShopAlias)) {
        my $Shop = $TestFolder->child($ShopAlias);
        my $PrepShop = $Shop->get('MigrationPrepShop');
        if (defined $PrepShop) {
            note('Cleaning up PrepShop');
            $PrepShop->delete()
        }
        note(sprintf('Cleaning up %s', $ShopAlias));
        $Shop->delete();
        LoadRootObject()->folder('Test')->delete();
    }
    DeleteDirectory($TmpDir, { 'recursive' => 1 });
    return;
}

__PACKAGE__->runScript();

1;
