#========================================================================================
# package      DE_EPAGES::ShopMigration::t::HTMLSanitizer
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::ShopMigration::API::HTMLSanitizer
#========================================================================================
package DE_EPAGES::ShopMigration::t::HTMLSanitizer;
use base DE_EPAGES::Test::API::UnitTest;

use strict;
use utf8;
use warnings;
use DE_EPAGES::ShopMigration::API::HTMLSanitizer qw(SanitizeHTMLForScribe);
use Test::More;

SKIP: {
    eval {require HTML::Restrict};
    if ($@) {
        plan skip_all => 'HTML::Restrict is not installed';
    }
    else {
        plan tests => 36;
    }
}

## no critic( Tics::ProhibitLongLines )
sub test {
    my $Sanitizer = DE_EPAGES::ShopMigration::API::HTMLSanitizer->new();

    my @ExpectedTestResults = (
        {
            Name   => 'Keep plain text as it is',
            Input  => 'Text Input',
            Result => 'Text Input',
        },
        {
            Name   => 'Remove unwanted img tags',
            Input  => '<img src="pic.jpg" alt="me" id="test"/></img>',
            Result => undef,
        },
        {
            Name   => 'Remove unwanted table tags',
            Input  => <<HTML,
<table align="center" cellpadding="1" class="cke-table editor_table_with_border">
    <tbody>
    <tr>
        <td class="editor_td_with_border" style="border-width: 1px; border-style: solid; border-color: rgb(0, 0, 0);">
            Content</td>
    </tbody>
</table>
HTML
            Result => 'Content',
        },
        {
            Name   => 'Remove unwanted attributes from p tags',
            Input  => '<p style="text-align: right;" style="margin-left: 40px;" class="ep-gadget"; id="123">Content</p>',
            Result => '<p style="text-align: right;">Content</p>',
        },
        {
            Name   => 'Remove unwanted attributes from h1 tags',
            Input  => '<h1 style="text-align: right;" style="margin-left: 40px;" class="ep-gadget"; id="124">Header1</h1>',
            Result => '<h1 style="text-align: right;">Header1</h1>',
        },
        {
            Name   => 'Remove unwanted attributes from h2 tags',
            Input  => '<h2 style="text-align: right;" style="margin-left: 40px;" class="ep-gadget"; id="125">Header2</h2>',
            Result => '<h2 style="text-align: right;">Header2</h2>',
        },
        {
            Name   => 'Remove unwanted attributes from h3 tags',
            Input  => '<h3 style="text-align: right;" style="margin-left: 40px;" class="ep-gadget" id="126" align="center">Header3</h3>',
            Result => '<h3 style="text-align: right;" align="center">Header3</h3>',
        },
        {
            Name   => 'Remove unwanted attributes from ul/li tags',
            Input  => '<ul style="text-align: right;" id="my_unordered_list"><li style="text-align: left;" class="ep-something">ListItem</li></ul>',
            Result => '<ul style="text-align: right;"><li style="text-align: left;">ListItem</li></ul>'
        },
        {
            Name   => 'Remove unwanted attributes from ol/li tags',
            Input  => '<ol style="text-align: right;" id="my_unordered_list"><li style="text-align: center;" class="ep-something">ListItem</li></ol>',
            Result => '<ol style="text-align: right;"><li style="text-align: center;">ListItem</li></ol>'
        },
        {
            Name   => 'Remove unwanted attributes from blockquote tags',
            Input  => '<blockquote cite="http://developer.mozilla.org" align="center" id="999"><p>Cite</p></blockquote>',
            Result => '<blockquote align="center"><p>Cite</p></blockquote>',
        },
        {
            Name   => 'Remove unwanted attributes from span tags',
            Input  => '<span style="color:#c1217b;" style="text-align: right;" class="ep-gadget" id="127" align="center">Span</span>',
            Result => '<span style="color:#c1217b;">Span</span>',
        },
        {
            Name   => 'Remove unwanted attributes from strong tags',
            Input  => '<strong style="color:#c1217b;" style="text-align: right;" class="ep-gadget" id="128" align="center">Strong</span>',
            Result => '<strong>Strong</span>',
        },
        {
            Name   => 'Remove unwanted attributes from b tags',
            Input  => '<b style="color:#c1217b;" style="text-align: right;" class="ep-gadget" id="129" align="center">Bold</b>',
            Result => '<b>Bold</b>',
        },
        {
            Name   => 'Remove unwanted attributes from em tags',
            Input  => '<em style="color:#c1217b;" style="text-align: right;" class="ep-gadget" id="127" align="center">Emphasized</em>',
            Result => '<em>Emphasized</em>',
        },
        {
            Name   => 'Remove unwanted attributes from i tags',
            Input  => '<i style="color:#c1217b;" style="text-align: right;" class="ep-gadget" id="127" align="center">Italic</i>',
            Result => '<i>Italic</i>',
        },
        {
            Name   => 'Remove unwanted attributes from u tags',
            Input  => '<u style="color:#c1217b;" style="text-align: right;" class="ep-gadget" id="127" align="center">Underlined</u>',
            Result => '<u>Underlined</u>',
        },
        {
            Name   => 'Remove unwanted attributes from a tags',
            Input  => '<a data-link-type="tree" data-type="linkpicker" target="_blank" href="?ObjectPath=/Shops/DemoShop">DemoShop</a>',
            Result => '<a href="?ObjectPath=/Shops/DemoShop">DemoShop</a>',
        },
        {
            Name   => 'Keep <br /> in html fields',
            Input  => '<br />Test<br />',
            Result => '<br />Test<br />',
        },

    );
    ## critic
    note('test with OO interface');
    foreach my $Test (@ExpectedTestResults) {
        is($Sanitizer->sanitze($Test->{Input}), $Test->{Result}, $Test->{Name});
    }
    note('test with non-OO interface');
    foreach my $Test (@ExpectedTestResults) {
        is(SanitizeHTMLForScribe($Test->{Input}), $Test->{Result}, $Test->{Name});
    }
    done_testing();
}

sub init {
    my $self = shift;
    $self->SUPER::init;
    return;
}

__PACKAGE__->runScript();

1;






