#========================================================================================
# §package      DE_EPAGES::ShopMigration::t::JSON
# §base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# §description  unit test for DE_EPAGES::ShopMigration::API::JSON
#========================================================================================
package DE_EPAGES::ShopMigration::t::JSON;
use base DE_EPAGES::Test::API::UnitTest;

use strict;
use utf8;
use warnings;
use File::Spec::Functions;
use File::Temp qw(tempdir);
use Test::More;
use DE_EPAGES::ShopMigration::API::JSON qw(EncodeJSON DecodeJSON DecodeJSONArray EncodeJSONArray);
use DE_EPAGES::ShopMigration::API::JSONLines::BaseExportDriver;
use DE_EPAGES::ShopMigration::API::JSONLines::BaseImportHandler;
use DE_EPAGES::Core::API::Error qw(ExistsError GetError);
use DE_EPAGES::Core::API::File qw(GetFileContent WriteFile DeleteDirectory ExistsFile DeleteFile);

plan tests => 58;
my $TmpDir = tempdir();
my $TmpFile = 'custom.json';
my $FileWithPath = catfile($TmpDir, $TmpFile);

sub test {
    testEncodingDecodingExamples();
    testGetLineFunction();
    testArrayFunctions();
    testBaseExportDriver();
    testBaseImportHandler();
    done_testing();
}

## no critic (Tics::ProhibitLongLines)
sub testEncodingDecodingExamples {
    my $Processor = DE_EPAGES::ShopMigration::API::JSON->new({ canonical => 1 });
    my $ProcessorWithPretty = DE_EPAGES::ShopMigration::API::JSON->new({ canonical => 1, pretty => 1 });
    my @TestData = (
        {
            Hash       => {},
            JSON       => '{}',
            NameEncode => 'JSON encode empty hash',
            NameDecode => 'JSON decode empty hash',
        },
        {
            Hash       => {},
            JSON       => '{}',
            NameEncode => 'JSON encode empty hash',
            NameDecode => 'JSON decode empty hash',
        },
        {
            Hash       => { key1 => [], key2 => [ 'test', 'test2' ] },
            JSON       => '{"key1":[],"key2":["test","test2"]}',
            NameEncode => 'JSON encode hash with empty arrays',
            NameDecode => 'JSON decode hash with empty arrays',
        },
        {
            Hash       => { key => 'value' },
            JSON       => '{"key":"value"}',
            NameEncode => 'JSON encode simple key/value pair',
            NameDecode => 'JSON decode simple key/value pair',
        },
        {
            Hash       => { key => 'value', key2 => { i_key1 => 'string', i_key2 => [ 'a', 'b', 'c' ] } },
            JSON       => '{"key":"value","key2":{"i_key1":"string","i_key2":["a","b","c"]}}',
            NameEncode => 'JSON encode complex hash',
            NameDecode => 'JSON decode complex hash',
        },
        {
            Hash       => { key => 'äüö', key2 => { i_key1 => 'string', i_key2 => [ { Index => 'Σ' }, { Index => 'Ω' } ] } },
            JSON       => '{"key":"äüö","key2":{"i_key1":"string","i_key2":[{"Index":"Σ"},{"Index":"Ω"}]}}',
            NameEncode => 'JSON encode complex hash with utf-8 and umlauts',
            NameDecode => 'JSON decode complex hash with utf-8 and umlauts',
        },
    );
    my @TestDataWithPretty = (
        {
            Hash       => { key => 'äüö', key2 => { i_key1 => 'string', i_key2 => [ { Index => 'Σ' }, { Index => 'Ω' } ] } },
            JSON       => <<TXT,
{
   "key" : "äüö",
   "key2" : {
      "i_key1" : "string",
      "i_key2" : [
         {
            "Index" : "Σ"
         },
         {
            "Index" : "Ω"
         }
      ]
   }
}
TXT
            NameEncode => 'JSON encode complex hash with utf-8 and umlauts (with pretty)',
            NameDecode => 'JSON decode complex hash with utf-8 and umlauts (from pretty)',
        },
        {
            Hash       => { key1 => [], key2 => [ 'test', 'test2' ] },
            JSON       => <<TXT,
{
   "key1" : [],
   "key2" : [
      "test",
      "test2"
   ]
}
TXT
            NameEncode => 'JSON encode hash with empty arrays (with pretty)',
            NameDecode => 'JSON decode hash with empty arrays (from pretty)',
            Options    => { pretty => 1 },
        },
    );
## critic
    note('test encoding/decoding via oo interface');
    foreach my $Test (@TestData) {
        is($Processor->encode($Test->{Hash}), $Test->{JSON}, $Test->{NameEncode});
        is_deeply($Processor->decode($Test->{JSON}), $Test->{Hash}, $Test->{NameDecode});
    }
    foreach my $Test (@TestDataWithPretty) {
        is($ProcessorWithPretty->encode($Test->{Hash}), $Test->{JSON}, $Test->{NameEncode});
        is_deeply($ProcessorWithPretty->decode($Test->{JSON}), $Test->{Hash}, $Test->{NameDecode});
    }
    note('test encoding/decoding via functional interface');
    foreach my $Test (@TestData) {
        is(EncodeJSON($Test->{Hash}, { canonical => 1 }), $Test->{JSON}, $Test->{NameEncode});
        is_deeply(DecodeJSON($Test->{JSON}), $Test->{Hash}, $Test->{NameDecode});
    }
    foreach my $Test (@TestDataWithPretty) {
        is(EncodeJSON($Test->{Hash}, { canonical => 1, pretty => 1 }), $Test->{JSON}, $Test->{NameEncode});
        is_deeply(DecodeJSON($Test->{JSON}), $Test->{Hash}, $Test->{NameDecode});
    }
}

sub testGetLineFunction {
    my $TestData = {
        Data => [ { zkey1 => 'ÄÖÜüöäß', akey2 => 'other1' }, { zkey1 => 'Σ', akey2 => 'other2' }, { zkey1 => 'Ω', akey2 => 'other3' }, ],
        JSON => <<TXT,
   {
      "akey2":"other1",
      "zkey1":"ÄÖÜüöäß"}
   {
      "akey2":"other2",
      "zkey1":"Σ"}
   {
      "akey2":"other3",
      "zkey1":"Ω"}
TXT
    };
    note('test basic API functions');
    my $Exporter = DE_EPAGES::ShopMigration::API::JSONLines::BaseExportDriver->new();
    $Exporter->exportJSONLines($TmpDir, $TmpFile, undef, { canonical => 1 }, $TestData->{Data});

    my $JsonAPI = DE_EPAGES::ShopMigration::API::JSON->new;
    my $fh = FileHandle->new;
    $fh->open($FileWithPath, '<:encoding(utf-8)') or die "Can't read $FileWithPath: $!";
    my $Line = 0;
    while (my $ahFields = $JsonAPI->getline($fh)) {
        is_deeply($ahFields, $TestData->{Data}[$Line], "getLine() via FileHandle in line $Line returns correct Hash");
        $Line++;
    }
    is($JsonAPI->getline($fh), undef, 'repeated getLine() via FileHandle on EOF returns undef')
}

sub testArrayFunctions {
    my $TestData = {
        Data => [ { zkey1 => 'ÄÖÜüöäß', akey2 => 'other1' }, { zkey1 => 'Σ', akey2 => 'other2' }, { zkey1 => 'Ω', akey2 => 'other3' }, ],
        JSON => "{\"akey2\":\"other1\",\"zkey1\":\"ÄÖÜüöäß\"}\n{\"akey2\":\"other2\",\"zkey1\":\"Σ\"}\n{\"akey2\":\"other3\",\"zkey1\":\"Ω\"}"
    };
    note('test Array API functions');

    is(EncodeJSONArray($TestData->{Data}), $TestData->{JSON}, 'EncodeJSONArray');
    is_deeply(DecodeJSONArray($TestData->{JSON}), $TestData->{Data}, 'DecodeJSONArray');
}

sub testBaseExportDriver {
    my $Exporter = DE_EPAGES::ShopMigration::API::JSONLines::BaseExportDriver->new();
    my @TestData = (
        {
            Data       => [ { zkey1 => 'ÄÖÜüöäß', akey2 => 'other1' } ],
            JSON       => <<TXT,
{"akey2":"other1","zkey1":"ÄÖÜüöäß"}
TXT
            NameEncode => 'exportJSONLines : array of single hash',
        },
        {
            Data       => [ { zkey1 => 'ÄÖÜüöäß', akey2 => 'other1' }, { zkey1 => 'Σ', akey2 => 'other2' }, { zkey1 => 'Ω', akey2 => 'other3' }, ],
            JSON       => <<TXT,
{"akey2":"other1","zkey1":"ÄÖÜüöäß"}
{"akey2":"other2","zkey1":"Σ"}
{"akey2":"other3","zkey1":"Ω"}
TXT
            NameEncode => 'exportJSONLines : array of hashes',
        },
        {
            Data       => [ {}, {}, {} ],
            JSON       => <<TXT,
{}
{}
{}
TXT
            NameEncode => 'exportJSONLines : empty array of hashes',
        },
        {
            Data       => [],
            JSON       => <<TXT,
TXT
            NameEncode => 'exportJSONLines : empty array',
        },
    );

    note('test BaseExportDriver');
    $Exporter->exportJSONLines($TmpDir, $TmpFile, undef, undef, $TestData[0]{Data});
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName());
    ok($Exporter->fileName() =~ /${TmpFile}$/, "custom filename matches $TmpFile");
    DeleteFile($Exporter->fileName());
    $Exporter->exportJSONLines($TmpDir, '', undef, undef, $TestData[0]{Data});
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName());
    ok($Exporter->fileName() =~ /export.json$/, 'default filename matches export.json');
    foreach my $Test (@TestData) {
        $Exporter->exportJSONLines($TmpDir, $TmpFile, undef, { canonical => 1 }, $Test->{Data});
        my $sFileContent = GetFileContent($FileWithPath, { encoding => 'utf8' });
        is($$sFileContent, $Test->{JSON}, $Test->{NameEncode});
    }
}

sub testBaseImportHandler {
    my @TestData = (
        {
            Data  => [ { zkey1 => 'ÄÖÜüöäß', akey2 => 'other1' }, { zkey1 => 'Σ', akey2 => 'other2' }, { zkey1 => 'Ω', akey2 => 'other3' }, ],
            Count => 3,
            Name  => 'array of hashes'
        },
        {
            Data  => [ {}, {}, {} ],
            Count => 3,
            Name  => 'empty array of hashes'
        },
        {
            Data  => [],
            Count => 0,
            Name  => 'empty array'
        },
    );
    my $Exporter = DE_EPAGES::ShopMigration::API::JSONLines::BaseExportDriver->new();
    note('test BaseImportHandler');
    foreach my $Test (@TestData) {
        my $Importer;
        $Exporter->exportJSONLines($TmpDir, $TmpFile, undef, { canonical => 1 }, $Test->{Data});
        eval {
            $Importer = MyImportHandler->importJSONLines($TmpDir, $TmpFile);
        };
        if (ExistsError()) {
            my $Error = GetError();
            print $Error->message;
        }
        is($Importer->lineNumber(), $Test->{Count}, 'importJSONLines: number of lines');
        is_deeply($Importer->{'Items'}, $Test->{Data}, sprintf('importJSONLines: %s', $Test->{Name}));
    }
    my $BadFileContent = <<EOB;
            { "key" : "value" }
        some invalid json content
EOB
    WriteFile($FileWithPath, \$BadFileContent, undef);
    eval {
        MyImportHandler->importJSONLines($TmpDir, $TmpFile);
        fail('must throw an exception');
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'JSON_READ_LINE_FAILED', 'importJSONLines: invalid json throws exception') or Test::More::diag($Error->longMessage);
        is($Error->vars->{Line}, 2, 'importJSONLines: exception lists correct line number') or Test::More::diag($Error->longMessage);
        ok($Error->message() =~ /malformed JSON/, 'importJSONLines: "malformed JSON" error detected') or Test::More::diag($Error->longMessage);
    }
    $BadFileContent = <<EOB;
            { "key" : "value" } garbage garbage
EOB
    WriteFile($FileWithPath, \$BadFileContent, undef);
    eval {
        MyImportHandler->importJSONLines($TmpDir, $TmpFile);
        fail('must throw an exception');
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'JSON_READ_LINE_FAILED', 'importJSONLines: invalid json throws exception') or Test::More::diag($Error->longMessage);
        is($Error->vars->{Line}, 1, 'importJSONLines: exception lists correct line number') or Test::More::diag($Error->longMessage);
        ok($Error->message() =~ /garbage after JSON object/, 'importJSONLines: "garbage after JSON object" error detected') or Test::More::diag($Error->longMessage);
    }
}

sub init {
    my $self = shift;
    return;
}

sub cleanup {
    my $self = shift;
    DeleteDirectory($TmpDir, { 'recursive' => 1 });
    return;
}

__PACKAGE__->runScript();

# to test the Importer we have to add a meaningful importLine() function to
# DE_EPAGES::ShopMigration::API::JSONLines::BaseImportHandler

package MyImportHandler; ## no critic (Modules::ProhibitMultiplePackages)
use base qw(DE_EPAGES::ShopMigration::API::JSONLines::BaseImportHandler);
use strict;

sub new {
    my $class = shift;
    my (@Params) = @_;
    my $self = $class->SUPER::new(@Params);

    $self->{'Items'} = [];
    return $self;
}

sub importLine {
    my $self = shift;

    push @{$self->{'Items'}}, $self->topID('Item');
    return;
}

1;
