#========================================================================================
# §package      DE_EPAGES::ShopMigration::t::LegalPagesExportImport
# §base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# §description  unit test for DE_EPAGES::ShopMigration::API::JSONLines::LegalPagesExportImport
#========================================================================================
package DE_EPAGES::ShopMigration::t::LegalPagesExportImport;
use base DE_EPAGES::Test::API::StoreUnitTest;
## no critic (Tics::ProhibitLongLines)
use strict;
use utf8;
use warnings;
use File::Spec::Functions;
use DE_EPAGES::Core::API::File qw(GetFileContent GetFileContentLines ExistsFile DeleteDirectory DeleteFile Text2Unix GetFileNames);
use DE_EPAGES::Object::API::Factory qw(LoadRootObject LoadClassByAlias);
use DE_EPAGES::Object::API::Language qw(GetPKeyLanguageByCode);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw(ImportObjects);
use DE_EPAGES::ShopMigration::API::JSON qw(DecodeJSON);
use DE_EPAGES::ShopMigration::API::JSONLines::LegalPagesExportImport qw(ExportLegalPagesJSON ImportLegalPagesJSON);
use DE_EPAGES::ShopMigration::API::MigrationShop qw(CreatePrepShop ExistsPrepShop SetShopFeatureValues TransferEpagesUI);
use File::Temp qw(tempdir);
use Test::More;

my $TmpDir = tempdir();

if ($ENV{SKIP_IN_PR_TESTS}) {
    plan skip_all => 'Skipping it because SKIP_IN_PR_TESTS == 1';
} else {
    plan tests => 58;
}

sub test {
    testExportWithLegalPagesExportImportBase2Prep();
    testExportWithLegalPagesExportImportPrep2Base();
    testImportWithLegalPagesExportImportBase2Prep();
    testImportWithLegalPagesExportImportPrep2Base();
    done_testing();
}

my %TestData = (
    TestShop      => undef,
    TestShopAlias => 'TestShop',
    ImportPath    => '/Test/Shops/TestShop',
);

# This is a bit hacky, the json exporter *can* export a numeric value either as 1 or as "1" (string) depending on how it was used before
# I got random results, so we just check both (as it just concerns the "IsActive" key)
sub testExportWithLegalPagesExportImportBase2Prep {
    my $Shop = $TestData{TestShop};

    note('test LegalPagesExportDriver (Base2Prep)');
    isnt($Shop, undef, 'Testshop exists');
    is(scalar @{$Shop->get('Languages')}, 1, 'Testshop has now one Language');
    is(scalar @{$Shop->get('SiteLocales')}, 1, 'Testshop has one SiteLocale');
    my $Exporter = ExportLegalPagesJSON($Shop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, 5, 'Export file has 5 lines');
    ok($Exporter->fileName =~ /legalpages.json/, 'export file name matches legalpages.json');
    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(
        catfile('ShopData.d', 'export', 'TestShop', 'base2prep_legalpages.json'), { encoding => 'utf8' });
    is($$sFileContent, $$sExpected, 'file content matches');
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');

    foreach my $Lang (qw(de en ru)) {
        my $LanguageID = GetPKeyLanguageByCode($Lang);
        $Shop->addLanguage($LanguageID) unless $Shop->existsLanguage($LanguageID);
    }
    foreach my $Locale (qw(ru_RU de_DE en_GB)) {
        $Shop->addLocale($Locale) unless $Shop->existsLocale($Locale);
    }
    is(scalar @{$Shop->get('Languages')}, 3, 'Testshop has now three Languages');
    is(scalar @{$Shop->get('SiteLocales')}, 3, 'Testshop has now three SiteLocales');
    $Exporter = ExportLegalPagesJSON($Shop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, 5, 'Export file has 5 lines');
    my $aContentLines = GetFileContentLines($Exporter->fileName, { encoding => 'utf8' });
    my $hValues = DecodeJSON($aContentLines->[0]);
    ok(exists $hValues->{LocalizedAttributes}{ru}, 'Export file has localized attributes for language ru');
    ok(exists $hValues->{LocalizedAttributes}{de}, 'Export file has localized attributes for language de');
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub testExportWithLegalPagesExportImportPrep2Base {
    my $PrepShop = $TestData{PrepShop};

    note('test LegalPagesExportDriver (Prep2Base)');
    isnt($PrepShop, undef, 'PrepShop exists');
    is(scalar @{$PrepShop->get('Languages')}, 1, 'Testshop has now one Language');
    is(scalar @{$PrepShop->get('SiteLocales')}, 1, 'Testshop has one SiteLocale');
    ok($PrepShop->get('IsMigrationPrepShop'), 'PrepShop has IsMigrationPrepShop=1');
    my $Exporter = ExportLegalPagesJSON($PrepShop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists ' . $Exporter->fileName);
    use constant FILE_LINES => 6;
    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, FILE_LINES, 'Export file lines');
    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(
        catfile('ShopData.d', 'export', 'TestShop', 'prep2base_legalpages.json'), { encoding => 'utf8' });
    is($$sFileContent, $$sExpected, 'file content matches');
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');

    foreach my $Lang (qw(de en ru)) {
        my $LanguageID = GetPKeyLanguageByCode($Lang);
        $PrepShop->addLanguage($LanguageID) unless $PrepShop->existsLanguage($LanguageID);
    }
    foreach my $Locale (qw(ru_RU de_DE en_GB)) {
        $PrepShop->addLocale($Locale) unless $PrepShop->existsLocale($Locale);
    }
    is(scalar @{$PrepShop->get('Languages')}, 3, 'PrepShop has now three Languages');
    is(scalar @{$PrepShop->get('SiteLocales')}, 3, 'PrepShop has now three SiteLocales');
    $Exporter = ExportLegalPagesJSON($PrepShop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, FILE_LINES, 'Export file lines');
    my $aContentLines = GetFileContentLines($Exporter->fileName, { encoding => 'utf8' });
    my $hValues = DecodeJSON($aContentLines->[0]);
    ok(exists $hValues->{LocalizedAttributes}{ru}, 'Export file has localized attributes for language ru');
    ok(exists $hValues->{LocalizedAttributes}{de}, 'Export file has localized attributes for language de');
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub testImportWithLegalPagesExportImportBase2Prep {

    my $ExpectedValues = {
        'Imprint'                       => {
            'Attributes'          => { 'IsVisible' => 1 },
            'LocalizedAttributes' => { 'en' => { 'Text' => 'The legal notice is intended for information about the shop operator, such as address and further contact data. This information is mandatory in many countries. Amend the content of this page according to the legal requirements. Further details are provided in our Helpcenter under the heading of "Legal pages". To open the Helpcenter, navigate back to the main menu, and then choose "Help" > <a href="https://help.epages.com/now/en/legalpages" target="_blank"> Open Help Center</a>.' } },
        },
        'TermsAndConditions'            => {
            'Attributes'          => { 'IsVisible' => 1 },
            'LocalizedAttributes' => { 'en' => { 'Text' => 'In the Terms & conditions you define the legal context for the business deals with your customers. This information is mandatory in many countries. Amend the content of this page according to the legal requirements. Further details are provided in our Helpcenter under the heading of "Legal pages". To open the Helpcenter, navigate back to the main menu, and then choose "Help" > <a href="https://help.epages.com/now/en/legalpages" target="_blank"> Open Help Center</a>.' } },
        },
        'CustomerInformation'           => {
            'Attributes'          => { 'IsVisible' => 0 },
            'LocalizedAttributes' => { 'en' => { 'Text' => 'Provide your customers with information about the right of withdrawal, amongst others the conditions, revocation period as well as a withdrawal form. This information is mandatory in many countries. Amend the content of this page according to the legal requirements. Further details are provided in our Helpcenter under the heading of "Legal pages". To open the Helpcenter, navigate back to the main menu, and then choose "Help" > <a href="https://help.epages.com/now/en/legalpages" target="_blank"> Open Help Center</a>.' } },
        },
        'PrivacyPolicy'                 => {
            'Attributes'          => { 'IsVisible' => 1 },
            'LocalizedAttributes' => { 'en' => { 'Text' => 'Provide your customers with general privacy policy information. This includes type, range and purpose of each data collection as well as the use of personal data. This information is mandatory in many countries. Amend the content of this page according to the legal requirements. Further details are provided in our Helpcenter under the heading of "Legal pages". To open the Helpcenter, navigate back to the main menu, and then choose "Help" > <a href="https://help.epages.com/now/en/legalpages" target="_blank"> Open Help Center</a>.' } },
        },
        'ShippingAndPaymentInformation' => {
            'Attributes'          => { 'IsVisible' => 1 },
            'LocalizedAttributes' => { 'en' => { 'Text' => 'Inform your customers about available delivery methods, delivery providers as well as incurring costs. This information is mandatory in many countries. Amend the content of this page according to the legal requirements. Further details are provided in our Helpcenter under the heading of "Legal pages". To open the Helpcenter, navigate back to the main menu, and then choose "Help" > <a href="https://help.epages.com/now/en/legalpages" target="_blank"> Open Help Center</a>.' } },
        },
        'CookiePolicy'                  => {
            'Attributes'          => { 'IsVisible' => 1 },
            'LocalizedAttributes' => { 'en' => { 'Text' => 'Provide your customers with information about the cookie policy and make them aware of the GDPR compliant cookie usage in your shop, e.g. “In the following you can change your cookie settings.”' } },
        },
    };

    my $PrepShop = $TestData{PrepShop};
    my $DirName = catdir('ShopData.d', 'import');
    note('test LegalPagesImportHandler (Base2Prep)');
    isnt($PrepShop, undef, 'Prepshop exists');
    my $Importer = ImportLegalPagesJSON($PrepShop, $DirName);
    ok($Importer->fileName() =~ /TestShop.base2prep_legalpages[.]json/, 'ImportFileName was constructed properly');
    foreach my $LegalPageAlias (keys %$ExpectedValues) {
        my $LegalPage = $PrepShop->get($LegalPageAlias);
        foreach my $Attribute (keys %{$ExpectedValues->{$LegalPageAlias}{Attributes}}) {
            is($LegalPage->get($Attribute), $ExpectedValues->{$LegalPageAlias}{Attributes}{$Attribute}, "LegalPage attribute $Attribute matches")
        }
        foreach my $LanguageCode (keys %{$ExpectedValues->{$LegalPageAlias}{LocalizedAttributes}}) {
            my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
            foreach my $Attribute (keys %{$ExpectedValues->{$LegalPageAlias}{LocalizedAttributes}{$LanguageCode}}) {
                is($LegalPage->get($Attribute, $LanguageID),
                    $ExpectedValues->{$LegalPageAlias}{LocalizedAttributes}{$LanguageCode}{$Attribute},
                    "LegalPage $LegalPageAlias localized attribute $Attribute for $LanguageCode matches ");
            }
        }
    }
}

sub testImportWithLegalPagesExportImportPrep2Base {
    my $Shop = $TestData{TestShop};
    TransferEpagesUI($Shop);
    my $ExpectedValues = {
        'Imprint'                       => {
            'Attributes'          => { 'IsVisible' => 1 },
            'LocalizedAttributes' => { 'en' => { 'Text' => 'TestCase: The legal notice is intended for information about the shop operator, such as address and further contact data. This information is mandatory in many countries. Amend the content of this page according to the legal requirements. Further details are provided in our Helpcenter under the heading of "Legal pages". To open the Helpcenter, navigate back to the main menu, and then choose "Help" > <a href="https://help.epages.com/now/en/legalpages" target="_blank"> Open Help Center</a>.' } },
        },
        'TermsAndConditions'            => {
            'Attributes'          => { 'IsVisible' => 1 },
            'LocalizedAttributes' => { 'en' => { 'Text' => 'TestCase: In the Terms & conditions you define the legal context for the business deals with your customers. This information is mandatory in many countries. Amend the content of this page according to the legal requirements. Further details are provided in our Helpcenter under the heading of "Legal pages". To open the Helpcenter, navigate back to the main menu, and then choose "Help" > <a href="https://help.epages.com/now/en/legalpages" target="_blank"> Open Help Center</a>.' } },
        },
        'CustomerInformation'           => {
            'Attributes'          => { 'IsVisible' => 0 },
            'LocalizedAttributes' => { 'en' => { 'Text' => 'TestCase: Provide your customers with information about the right of withdrawal, amongst others the conditions, revocation period as well as a withdrawal form. This information is mandatory in many countries. Amend the content of this page according to the legal requirements. Further details are provided in our Helpcenter under the heading of "Legal pages". To open the Helpcenter, navigate back to the main menu, and then choose "Help" > <a href="https://help.epages.com/now/en/legalpages" target="_blank"> Open Help Center</a>.' } },
        },
        'PrivacyPolicy'                 => {
            'Attributes'          => { 'IsVisible' => 1 },
            'LocalizedAttributes' => { 'en' => { 'Text' => 'TestCase: Provide your customers with general privacy policy information. This includes type, range and purpose of each data collection as well as the use of personal data. This information is mandatory in many countries. Amend the content of this page according to the legal requirements. Further details are provided in our Helpcenter under the heading of "Legal pages". To open the Helpcenter, navigate back to the main menu, and then choose "Help" > <a href="https://help.epages.com/now/en/legalpages" target="_blank"> Open Help Center</a>.' } },
        },
        'ShippingAndPaymentInformation' => {
            'Attributes'          => { 'IsVisible' => 1 },
            'LocalizedAttributes' => { 'en' => { 'Text' => 'TestCase: Inform your customers about available delivery methods, delivery providers as well as incurring costs. This information is mandatory in many countries. Amend the content of this page according to the legal requirements. Further details are provided in our Helpcenter under the heading of "Legal pages". To open the Helpcenter, navigate back to the main menu, and then choose "Help" > <a href="https://help.epages.com/now/en/legalpages" target="_blank"> Open Help Center</a>.' } },
        },
        'CookiePolicy'                  => {
            'Attributes'          => { 'IsVisible' => 1 },
            'LocalizedAttributes' => { 'en' => { 'Text' => 'TestCase: Provide your customers with information about the cookie policy and make them aware of the GDPR compliant cookie usage in your shop, e.g. “In the following you can change your cookie settings.”' } },
        },
    };

    my $DirName = catdir('ShopData.d', 'import');
    note('test LegalPagesImportHandler (Prep2Base)');
    isnt($Shop, undef, 'Testshop exists');
    foreach my $LegalPageAlias (keys %$ExpectedValues) {
        foreach my $LanguageCode (keys %{$ExpectedValues->{$LegalPageAlias}{LocalizedAttributes}}) {
            my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
            my $LegalPage = $Shop->get($LegalPageAlias);
            $LegalPage->set({ 'Text' => 'dummy' }, $LanguageID);
        }
    }
    my $Importer = ImportLegalPagesJSON($Shop, $DirName);
    ok($Importer->fileName() =~ /TestShop.prep2base_legalpages[.]json/, 'ImportFileName was constructed properly');
    foreach my $LegalPageAlias (keys %$ExpectedValues) {
        my $LegalPage = $Shop->get($LegalPageAlias);
        foreach my $Attribute (keys %{$ExpectedValues->{$LegalPageAlias}{Attributes}}) {
            is($LegalPage->get($Attribute), $ExpectedValues->{$LegalPageAlias}{Attributes}{$Attribute},
                "LegalPage attribute $Attribute matches")
        }

        foreach my $LanguageCode (keys %{$ExpectedValues->{$LegalPageAlias}{LocalizedAttributes}}) {
            my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
            foreach my $Attribute (keys %{$ExpectedValues->{$LegalPageAlias}{LocalizedAttributes}{$LanguageCode}}) {
                is($LegalPage->get($Attribute, $LanguageID),
                    $ExpectedValues->{$LegalPageAlias}{LocalizedAttributes}{$LanguageCode}{$Attribute},
                    "LegalPage $LegalPageAlias localized attribute $Attribute for $LanguageCode matches ")
            }
        }
    }
}


# We need to create a fake ShopTypeRef UnityDE + ECom5000 in case they don't exists
# otherwise the Mapper won't work
sub init {
    my $self = shift;

    $self->SUPER::init;
    my $ShopAlias = $TestData{TestShopAlias};
    my $System = LoadRootObject();
    my $Shop;
    my $TestFolder = $System->folder('Test')->folder('Shops');
    my $ShopTypeRefFolder = $System->folder('ShopTypeRefs');
    my $Class = LoadClassByAlias('ShopTypeRef');
    $Class->insertObject({ Alias => 'UnityDE', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('UnityDE'));
    $Class->insertObject({ Alias => 'ECom5000', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('ECom5000'));
    if ($TestFolder->existsChild($ShopAlias)) {
        $Shop = $TestFolder->child($ShopAlias);
    } else {
        $Shop = $TestFolder->insertChild('Shop', {
            Alias                   => $ShopAlias,
            ShopTypeRef             => $ShopTypeRefFolder->child('ECom5000'),
            WebServerScriptNamePart => $ShopAlias,
        });
        SetShopFeatureValues($Shop,
            [ { Alias => 'Regions', MaxValue => '2' }, { Alias => 'Languages', MaxValue => '15' } ]);
    }
    $TestData{TestShop} = $Shop;
    $TestData{PrepShop} = ExistsPrepShop($Shop) ? $Shop->get('MigrationPrepShop') : CreatePrepShop($Shop);
    SetShopFeatureValues($TestData{PrepShop}, [ { Alias => 'Languages', MaxValue => '15' } ]);
    foreach my $JSONFile (@{GetFileNames('ShopData.d', { 'recursive' => 1 })}) {
        Text2Unix($JSONFile) if ($^O eq 'linux');
    }
    ImportObjects('ImportFiles/LegalPagesBase.xml', $TestData{TestShop}->pathString());
    ImportObjects('ImportFiles/LegalPagesNow.xml', $TestData{PrepShop}->pathString());
    return;
}

sub cleanup {
    my $self = shift;

    my $ShopAlias = $TestData{TestShopAlias};
    my $TestFolder = LoadRootObject()->folder('Test')->folder('Shops');
    if ($TestFolder->existsChild($ShopAlias)) {
        my $Shop = $TestFolder->child($ShopAlias);
        my $PrepShop = $Shop->get('MigrationPrepShop');
        $PrepShop->delete() if (defined $PrepShop);
        $Shop->delete();
        LoadRootObject()->folder('Test')->delete();
    }
    DeleteDirectory($TmpDir, { 'recursive' => 1 });
    return;
}


__PACKAGE__->runScript();

1;
