#========================================================================================
# package      DE_EPAGES::ShopMigration::t::MailTypesExportImport
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::ShopMigration::API::JSONLines::MailTypesExportImport
#========================================================================================
package DE_EPAGES::ShopMigration::t::MailTypesExportImport;
use base DE_EPAGES::Test::API::StoreUnitTest;
## no critic (Tics::ProhibitLongLines)
use strict;
use utf8;
use warnings;
use File::Spec::Functions;
use DE_EPAGES::Core::API::File qw(GetFileContent GetFileContentLines ExistsFile DeleteDirectory DeleteFile Text2Unix GetFileNames);
use DE_EPAGES::Core::API::PerlTools qw(ElementOf);
use DE_EPAGES::Object::API::Factory qw(LoadRootObject LoadClassByAlias);
use DE_EPAGES::Object::API::Language qw(GetPKeyLanguageByCode);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw(ImportObjects);
use DE_EPAGES::ShopMailType::API::ShopMailType qw(GetShopMailType); ##no critic (RequireCorrectImports)
use DE_EPAGES::ShopMigration::API::JSON qw(DecodeJSON DecodeJSONArray);
use DE_EPAGES::ShopMigration::API::Constants qw(MAILTYPE_LOCALIZEDATTRIBUTES_FOR_RESET MAILTYPE_BOOLEAN_ATTRIBUTES_FOR_RESET);
use DE_EPAGES::ShopMigration::API::JSONLines::MailTypesExportImport qw(ExportMailTypesJSON ImportMailTypesJSON);
use DE_EPAGES::ShopMigration::API::MigrationShop qw(CreatePrepShop ExistsPrepShop SetShopFeatureValues TransferEpagesUI);
use File::Temp qw(tempdir);
use Test::More;
use Test::Differences qw(context_diff eq_or_diff) ;

my $TmpDir = tempdir();

if ($ENV{SKIP_IN_PR_TESTS}) {
    plan skip_all => 'Skipping it because SKIP_IN_PR_TESTS == 1';
}

sub test {
    context_diff();
    testExportWithMailTypesExportImportBase2Prep();
    testExportWithMailTypesExportImportPrep2Base();
    testImportWithMailTypesExportImportBase2Prep();
    testImportWithMailTypesExportImportPrep2Base();
    done_testing();
}

my %TestData = (
    TestShop      => undef,
    TestShopAlias => 'TestShop',
    ImportPath    => '/Test/Shops/TestShop',
);
use constant FILENAME => 'mailtypes.json';
use constant FILE_DIR => [ qw(ShopData.d export TestShop) ];
use constant FILE_LINES => 5;
use constant FILE_LINES_PREP => 19;

# This is a bit hacky, the json exporter *can* export a numeric value either as 1 or as "1" (string) depending on how it was used before
# I got random results, so we just check both (as it just concerns the "IsActive" key)
sub testExportWithMailTypesExportImportBase2Prep {
    my $Shop = $TestData{TestShop};

    note('test MailTypesExportDriver (Base2Prep)');
    isnt($Shop, undef, 'Testshop exists');
    is(scalar @{$Shop->get('Languages')}, 1, 'Testshop has now one Language');
    is(scalar @{$Shop->get('SiteLocales')}, 1, 'Testshop has one SiteLocale');
    my $Exporter = ExportMailTypesJSON($Shop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    my $LinesCount = scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })};
    is($LinesCount, FILE_LINES, "export file lines count is ${\FILE_LINES}");
    ok($Exporter->fileName =~ /${\FILENAME}/, "export file name matches ${\FILENAME}");
    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(catfile(@{FILE_DIR()}, 'base2prep_' . FILENAME), { encoding => 'utf8' });
    compareMailTypeHashes($sFileContent, $sExpected);
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');

    foreach my $Lang (qw(de en ru)) {
        my $LanguageID = GetPKeyLanguageByCode($Lang);
        $Shop->addLanguage($LanguageID) unless $Shop->existsLanguage($LanguageID);
    }
    foreach my $Locale (qw(ru_RU de_DE en_GB)) {
        $Shop->addLocale($Locale) unless $Shop->existsLocale($Locale);
    }
    is(scalar @{$Shop->get('Languages')}, 3, 'Testshop has now three Languages');
    is(scalar @{$Shop->get('SiteLocales')}, 3, 'Testshop has now three SiteLocales');
    $Exporter = ExportMailTypesJSON($Shop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, 5, 'Export file has 5 lines');
    my $aContentLines = GetFileContentLines($Exporter->fileName, { encoding => 'utf8' });
    my $hValues = DecodeJSON($aContentLines->[0]);
    ok(exists $hValues->{LocalizedAttributes}{ru}, 'export file has localized attributes for language ru');
    ok(exists $hValues->{LocalizedAttributes}{de}, 'export file has localized attributes for language de');
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub testExportWithMailTypesExportImportPrep2Base {
    my $PrepShop = $TestData{PrepShop};

    note('test MailTypesExportDriver (Prep2Base)');
    isnt($PrepShop, undef, 'PrepShop exists');
    is(scalar @{$PrepShop->get('Languages')}, 1, 'Testshop has now one Language');
    is(scalar @{$PrepShop->get('SiteLocales')}, 1, 'Testshop has one SiteLocale');
    ok($PrepShop->get('IsMigrationPrepShop'), 'PrepShop has IsMigrationPrepShop=1');
    my $ShopMailType = GetShopMailType($PrepShop, 'CustomerOrderConfirmation');
    my @LegalDocs = grep {$_->alias ne 'Imprint'} @{$PrepShop->child('LegalPagesUnity')->children};
    $ShopMailType->set({ 'UnityAttachLegalDocumentsPDF' => \@LegalDocs });
    my $Exporter = ExportMailTypesJSON($PrepShop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists ' . $Exporter->fileName);
    my $LinesCount = scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })};
    is($LinesCount, FILE_LINES_PREP, "export file lines count is ${\FILE_LINES_PREP}");
    ok($Exporter->fileName =~ /${\FILENAME}/, "export file name matches ${\FILENAME}");
    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(catfile(@{FILE_DIR()}, 'prep2base_' . FILENAME), { encoding => 'utf8' });
    compareMailTypeHashes($sFileContent, $sExpected);
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');

    foreach my $Lang (qw(de en ru)) {
        my $LanguageID = GetPKeyLanguageByCode($Lang);
        $PrepShop->addLanguage($LanguageID) unless $PrepShop->existsLanguage($LanguageID);
    }
    foreach my $Locale (qw(ru_RU de_DE en_GB)) {
        $PrepShop->addLocale($Locale) unless $PrepShop->existsLocale($Locale);
    }
    is(scalar @{$PrepShop->get('Languages')}, 3, 'PrepShop has now three Languages');
    is(scalar @{$PrepShop->get('SiteLocales')}, 3, 'PrepShop has now three SiteLocales');
    $Exporter = ExportMailTypesJSON($PrepShop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, FILE_LINES_PREP, 'Export file lines');
    my $aContentLines = GetFileContentLines($Exporter->fileName, { encoding => 'utf8' });
    my $hValues = DecodeJSON($aContentLines->[0]);
    ok(exists $hValues->{LocalizedAttributes}{ru}, 'export file has localized attributes for language ru');
    ok(exists $hValues->{LocalizedAttributes}{de}, 'export file has localized attributes for language de');
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub compareMailTypeHashes {
    my ($CurrentContent, $ExpectedContent) = @_;

    # Removes all unknown (not installed language elements) from $Expected otherwise
    # we get a diff, as it contains all enabled languages after installing all langpacks
    my $aCurrent = DecodeJSONArray($$CurrentContent);
    my $aExpected = DecodeJSONArray($$ExpectedContent);
    my @KeepLanguages = ();
    foreach my $Code2 (map {$_->{'Code2'}} @{LoadRootObject()->get('Languages')}) {
        push @KeepLanguages, $Code2;
    }
    foreach my $Line ( @$aExpected ) {
        foreach my $Lang (keys %{$Line->{'LocalizedAttributes'}}) {
            next if ElementOf($Lang, @KeepLanguages);
            delete $Line->{'LocalizedAttributes'}{$Lang};
        }
    }
    eq_or_diff($aCurrent, $aExpected, 'file content matches');
}

sub testImportWithMailTypesExportImportBase2Prep {

    my $ExpectedValues = {
        'CustomerOrderConfirmation' => {
            'Attributes'          => { 'FROM' => 'from@somewhere1', 'CC' => 'to@somewhere1', 'BCC' => 'to@somewhere1' },
            'LocalizedAttributes' => { 'en' => { 'Subject' => 'subject1', 'Header' => 'header1' } },
        },
        'CustomerWelcome'           => {
            'Attributes'          => { 'FROM' => 'from@somewhere2', 'CC' => 'to@somewhere2', 'BCC' => 'to@somewhere2' },
            'LocalizedAttributes' => { 'en' => { 'Subject' => 'subject2', 'Header' => 'header2' } }
        },
        'UnityNewPassword'          => {
            'Attributes'          => { 'FROM' => 'from@somewhere3', 'CC' => 'to@somewhere3', 'BCC' => 'to@somewhere3' },
            'LocalizedAttributes' => { 'en' => { 'Subject' => 'subject3', 'Header' => 'header3' } }
        },
        'ProductOutOfStock'         => {
            'Attributes'          => { 'FROM' => 'from@somewhere4', 'CC' => 'to@somewhere4', 'BCC' => 'to@somewhere4' },
            'LocalizedAttributes' => { 'en' => { 'Subject' => 'subject4', 'Header' => 'header4' } }
        },
        'ProductStockWarning'       => {
            'Attributes'          => { 'FROM' => 'from@somewhere5', 'CC' => 'to@somewhere5', 'BCC' => 'to@somewhere5' },
            'LocalizedAttributes' => { 'en' => { 'Subject' => 'subject5', 'Header' => 'header5' } }
        }
    };

    my $PrepShop = $TestData{PrepShop};
    my $DirName = catdir('ShopData.d', 'import');
    note('test MailTypesImportHandler (Base2Prep)');
    isnt($PrepShop, undef, 'Prepshop exists');
    $PrepShop->folder('MailTypes')->deleteChildren;
    is(scalar @{$PrepShop->folder('MailTypes')->findChildren}, 0, 'no MailTypes on PrepShop');
    my $Importer = ImportMailTypesJSON($PrepShop, $DirName);
    is(scalar @{$PrepShop->folder('MailTypes')->findChildren}, 5, 'imported 5 mailtypes');
    ok($Importer->fileName() =~ /TestShop.base2prep_mailtypes[.]json/, 'ImportFileName was constructed properly');
    foreach my $MailTypeAlias (keys %$ExpectedValues) {
        note("mailtype '$MailTypeAlias'");
        my $ShopMailType = GetShopMailType($PrepShop, $MailTypeAlias);
        foreach my $Attribute (keys %{$ExpectedValues->{$MailTypeAlias}{Attributes}}) {
            is($ShopMailType->get($Attribute), $ExpectedValues->{$MailTypeAlias}{Attributes}{$Attribute}, "attribute $Attribute matches")
        }
        foreach my $Attribute (@{MAILTYPE_BOOLEAN_ATTRIBUTES_FOR_RESET()}) {
            is($ShopMailType->get($Attribute), 0, "attribute $Attribute matches (reset)")
        }
        foreach my $LanguageCode (keys %{$ExpectedValues->{$MailTypeAlias}{LocalizedAttributes}}) {
            my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
            foreach my $Attribute (keys %{$ExpectedValues->{$MailTypeAlias}{LocalizedAttributes}{$LanguageCode}}) {
                is($ShopMailType->get($Attribute, $LanguageID),
                    $ExpectedValues->{$MailTypeAlias}{LocalizedAttributes}{$LanguageCode}{$Attribute},
                    "localized attribute $Attribute for $LanguageCode matches ");
            }
        }
        foreach my $LanguageCode (map {$_->{'Code2'}} @{LoadRootObject()->get('Languages')}) {
            my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
            foreach my $Attribute (@{MAILTYPE_LOCALIZEDATTRIBUTES_FOR_RESET()}) {
                is($ShopMailType->get($Attribute, $LanguageID), undef,
                    "localized attribute $Attribute for $LanguageCode matches (reset)");
            }
        }
    }
    ## critic
}

sub testImportWithMailTypesExportImportPrep2Base {
    my $Shop = $TestData{TestShop};
    TransferEpagesUI($Shop);
    my $AllLegalDocs = [ grep {$_->alias ne 'Imprint'} @{$Shop->child('LegalPagesUnity')->children} ];
    my $TandC = [ $Shop->child('LegalPagesUnity')->child('TermsAndConditions') ];
    my $ExpectedValues = {
        'CustomerOrderConfirmation' => {
            'Attributes'          => {
                'FROM'                       => 'from@somewhere1', 'CC' => 'to@somewhere1', 'BCC' => 'to@somewhere1',
                UnityAttachLegalDocumentsPDF => $AllLegalDocs,
            },
            'LocalizedAttributes' => { 'en' => { 'Subject' => 'subject1', 'Header' => 'header1', 'AdditionalText' => undef } },
        },
        'CustomerWelcome'           => {
            'Attributes'          => { 'FROM' => 'from@somewhere2', 'CC' => 'to@somewhere2', 'BCC' => 'to@somewhere2',
                UnityAttachLegalDocumentsPDF  => $TandC,
            },
            'LocalizedAttributes' => { 'en' => { 'Subject' => 'subject2', 'Header' => 'header2', 'AdditionalText' => undef } }
        },
        'UnityNewPassword'          => {
            'Attributes'          => { 'FROM' => 'from@somewhere3', 'CC' => 'to@somewhere3', 'BCC' => 'to@somewhere3',
                UnityAttachLegalDocumentsPDF  => [],
            },
            'LocalizedAttributes' => { 'en' => { 'Subject' => 'subject3', 'Header' => 'header3', 'AdditionalText' => undef } }
        },
        'ProductOutOfStock'         => {
            'Attributes'          => { 'FROM' => 'from@somewhere4', 'CC' => 'to@somewhere4', 'BCC' => 'to@somewhere4',
                UnityAttachLegalDocumentsPDF  => [],
            },
            'LocalizedAttributes' => { 'en' => { 'Subject' => 'subject4', 'Header' => 'header4', 'AdditionalText' => undef } }
        },
        'ProductStockWarning'       => {
            'Attributes'          => { 'FROM' => 'from@somewhere5', 'CC' => 'to@somewhere5', 'BCC' => 'to@somewhere5',
                UnityAttachLegalDocumentsPDF  => [], },
            'LocalizedAttributes' => { 'en' => { 'Subject' => 'subject5', 'Header' => 'header5', 'AdditionalText' => undef } }
        }
    };

    my $DirName = catdir('ShopData.d', 'import');
    note('test MailTypesImportHandler (Prep2Base)');
    isnt($Shop, undef, 'Testshop exists');
    foreach my $MailTypeAlias (keys %$ExpectedValues) {
        foreach my $LanguageCode (keys %{$ExpectedValues->{$MailTypeAlias}{LocalizedAttributes}}) {
            my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
            my $ShopMailType = GetShopMailType($Shop, $MailTypeAlias);
            $ShopMailType->set({ AdditionalText => 'dummy' }, $LanguageID);
        }
    }
    my $Importer = ImportMailTypesJSON($Shop, $DirName);
    ok($Importer->fileName() =~ /TestShop.prep2base_mailtypes[.]json/, 'ImportFileName was constructed properly');
    foreach my $MailTypeAlias (keys %$ExpectedValues) {
        note("mailtype '$MailTypeAlias'");
        my $ShopMailType = GetShopMailType($Shop, $MailTypeAlias);
        foreach my $Attribute (keys %{$ExpectedValues->{$MailTypeAlias}{Attributes}}) {
            if ($Attribute eq 'UnityAttachLegalDocumentsPDF') {
                my @Wanted = sort map {$_->alias} @{$ExpectedValues->{$MailTypeAlias}{Attributes}{$Attribute}};
                my @Have = sort map {$_->alias} @{$ShopMailType->get($Attribute)};
                is_deeply(\@Have, \@Wanted, sprintf('Elements for %s match, cardinality %s', $Attribute, scalar @Wanted))
            } else {
                is($ShopMailType->get($Attribute), $ExpectedValues->{$MailTypeAlias}{Attributes}{$Attribute},
                    "attribute $Attribute matches")
            }
        }
        foreach my $Attribute (@{MAILTYPE_BOOLEAN_ATTRIBUTES_FOR_RESET()}) {
            is($ShopMailType->get($Attribute), 0, "attribute $Attribute matches (reset)")
        }
        foreach my $LanguageCode (keys %{$ExpectedValues->{$MailTypeAlias}{LocalizedAttributes}}) {
            my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
            foreach my $Attribute (keys %{$ExpectedValues->{$MailTypeAlias}{LocalizedAttributes}{$LanguageCode}}) {
                is($ShopMailType->get($Attribute, $LanguageID),
                    $ExpectedValues->{$MailTypeAlias}{LocalizedAttributes}{$LanguageCode}{$Attribute},
                    "localized attribute $Attribute for $LanguageCode matches")
            }
        }
        foreach my $LanguageCode (map {$_->{'Code2'}} @{LoadRootObject()->get('Languages')}) {
            my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
            foreach my $Attribute (@{MAILTYPE_LOCALIZEDATTRIBUTES_FOR_RESET()}) {
                is($ShopMailType->get($Attribute, $LanguageID), undef,
                    "localized attribute $Attribute for $LanguageCode matches (reset)");
            }
        }
    }
}


# We need to create a fake ShopTypeRef UnityDE + ECom5000 in case they don't exists
# otherwise the Mapper won't work
sub init {
    my $self = shift;

    $self->SUPER::init;
    my $ShopAlias = $TestData{TestShopAlias};
    my $System = LoadRootObject();
    my $Shop;
    my $TestFolder = $System->folder('Test')->folder('Shops');
    my $ShopTypeRefFolder = $System->folder('ShopTypeRefs');
    my $Class = LoadClassByAlias('ShopTypeRef');
    $Class->insertObject({ Alias => 'UnityDE', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('UnityDE'));
    $Class->insertObject({ Alias => 'ECom5000', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('ECom5000'));
    if ($TestFolder->existsChild($ShopAlias)) {
        $Shop = $TestFolder->child($ShopAlias);
    } else {
        $Shop = $TestFolder->insertChild('Shop', {
            Alias                   => $ShopAlias,
            ShopTypeRef             => $ShopTypeRefFolder->child('ECom5000'),
            WebServerScriptNamePart => $ShopAlias,
        });
        SetShopFeatureValues($Shop,
            [ { Alias => 'Regions', MaxValue => '2' }, { Alias => 'Languages', MaxValue => '15' } ]);
    }
    $TestData{TestShop} = $Shop;
    $TestData{PrepShop} = ExistsPrepShop($Shop) ? $Shop->get('MigrationPrepShop') : CreatePrepShop($Shop);
    SetShopFeatureValues($TestData{PrepShop}, [ { Alias => 'Languages', MaxValue => '15' } ]);
    foreach my $JSONFile (@{GetFileNames('ShopData.d', { 'recursive' => 1 })}) {
        Text2Unix($JSONFile) if ($^O eq 'linux');
    }
    ImportObjects('ImportFiles/MailTypesBase.xml', $TestData{TestShop}->pathString());
    ImportObjects('ImportFiles/MailTypesNow.xml', $TestData{PrepShop}->pathString());
    return;
}

sub cleanup {
    my $self = shift;

    my $ShopAlias = $TestData{TestShopAlias};
    my $TestFolder = LoadRootObject()->folder('Test')->folder('Shops');
    if ($TestFolder->existsChild($ShopAlias)) {
        my $Shop = $TestFolder->child($ShopAlias);
        my $PrepShop = $Shop->get('MigrationPrepShop');
        $PrepShop->delete() if (defined $PrepShop);
        $Shop->delete();
        LoadRootObject()->folder('Test')->delete();
    }
    DeleteDirectory($TmpDir, { 'recursive' => 1 });
    return;
}

__PACKAGE__->runScript();

1;
