#========================================================================================
# §package      DE_EPAGES::ShopMigration::t::ProductTypesExportImport
# §base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# §description  unit test for DE_EPAGES::ShopMigration::API::JSONLines::ProductTypesExportImport
#========================================================================================
package DE_EPAGES::ShopMigration::t::ProductTypesExportImport;
use base DE_EPAGES::Test::API::StoreUnitTest;
## no critic (Tics::ProhibitLongLines)
use strict;
use utf8;
use warnings;
use File::Spec::Functions;
use DE_EPAGES::Core::API::File qw(GetFileContent GetFileContentLines ExistsFile DeleteDirectory DeleteFile Text2Unix GetFileNames);
use DE_EPAGES::Object::API::Factory qw(LoadRootObject LoadClassByAlias ResetFactoryCaches);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw(ImportObjects);
use DE_EPAGES::ShopMigration::API::JSONLines::BaseExportDriver;
use DE_EPAGES::ShopMigration::API::JSONLines::ProductTypesExportImport qw(ExportProductTypesJSON ImportProductTypesJSON);
use DE_EPAGES::ShopMigration::API::MigrationShop qw(CreatePrepShop ExistsPrepShop SetShopFeatureValues);
use DE_EPAGES::ShopMigration::API::ResetShopHandler;
use DE_EPAGES::ShopMigration::API::ProductTypesCommon qw(GetProductTypeData);
use File::Temp qw(tempdir);
use Test::Differences qw(eq_or_diff);
use Test::More;

my $TmpDir = tempdir();

if ($ENV{SKIP_IN_PR_TESTS}) {
    plan skip_all => 'Skipping it because SKIP_IN_PR_TESTS == 1';
} else {
    plan tests => 38;
}

# hack BaseExportdDriver to use canonical for easier diff, not enabled per default for performance reasons
$DE_EPAGES::ShopMigration::API::JSONLines::BaseExportDriver::JSON_OPTS{canonical} = 1;

sub test {
    testExportWithProductTypesExportImportBase2Prep();
    testExportWithProductTypesExportImportPrep2Base();
    testImportWithProductTypesExportImportBase2Prep();
    testImportWithProductTypesExportImportPrep2Base();
    done_testing();
}

my %TestData = (
    TestShop      => undef,
    TestShopAlias => 'TestShop',
    ImportPath    => '/Test/Shops/TestShop',
    OldValues     => {
        'ProductClass'  => {
            'Attributes'            => {
                'Alias'    => '',
                'Position' => '140',
            },
            'LocalizedAttributes'   => {
                'en' => {
                    'Name' => 'Standard',
                }
            },
            'ProductTypeAttributes' => [
                {
                    Attributes            => {
                        'Alias'          => 'MyString',
                        'IsHTML'         => '0',
                        'IsObject'       => '0',
                        'IsSearchFilter' => '0',
                        'IsUserDefined'  => '1',
                        'IsVisible'      => '1',
                        'Position'       => '10',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'LocalizedString'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyString'
                        }
                    },
                    'PreDefAttributes'    => [],
                }
            ],
        },
        'MyProductType' => {
            'Attributes'            => {
                'Alias'    => 'MyProductType',
                'Position' => '10'
            },
            'LocalizedAttributes'   => {
                'en' => {
                    'Name' => 'MyProductType'
                }
            },
            'ProductTypeAttributes' => [
                {
                    'PreDefAttributes'    => [],
                    'Attributes'          => {
                        'Alias'          => 'MyDate',
                        'IsSearchFilter' => '0',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'LocalizedString',
                        'IsVisible'      => '1',
                        'Position'       => '60',
                        'IsObject'       => '0',
                        'IsHTML'         => '0',
                        'IsUserDefined'  => '1'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyDate'
                        }
                    }
                },
                {
                    'PreDefAttributes'    => [],
                    'Attributes'          => {
                        'Alias'          => 'MyDateTime',
                        'IsSearchFilter' => '0',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'LocalizedString',
                        'IsVisible'      => '1',
                        'Position'       => '50',
                        'IsObject'       => '0',
                        'IsHTML'         => '0',
                        'IsUserDefined'  => '1'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyDateTime'
                        }
                    }
                },
                {
                    'PreDefAttributes'    => [],
                    'Attributes'          => {
                        'Alias'          => 'MyFloat',
                        'IsSearchFilter' => '0',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'LocalizedString',
                        'IsVisible'      => '1',
                        'Position'       => '30',
                        'IsObject'       => '0',
                        'IsHTML'         => '0',
                        'IsUserDefined'  => '1'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyFloat'
                        }
                    }
                },
                {
                    'PreDefAttributes'    => [],
                    'Attributes'          => {
                        'Alias'          => 'MyInteger',
                        'IsSearchFilter' => '0',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'LocalizedString',
                        'IsVisible'      => '1',
                        'Position'       => '20',
                        'IsObject'       => '0',
                        'IsHTML'         => '0',
                        'IsUserDefined'  => '1'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyInteger'
                        }
                    }
                },
                {
                    'PreDefAttributes'    => [],
                    'Attributes'          => {
                        'Alias'          => 'MyMoney',
                        'IsSearchFilter' => '0',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'LocalizedString',
                        'IsVisible'      => '1',
                        'Position'       => '40',
                        'IsObject'       => '0',
                        'IsHTML'         => '0',
                        'IsUserDefined'  => '1'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyMoney'
                        }
                    }
                },
                {
                    'PreDefAttributes'    => [
                        {
                            'Attributes'          => {
                                'Alias'     => 'blau',
                                'Position'  => '10',
                                'Attribute' => 'MyPreDefLocalizedString'
                            },
                            'LocalizedAttributes' => {
                                'en' => {
                                    'ValueLocString' => 'blue'
                                }
                            }
                        },
                        {
                            'Attributes'          => {
                                'Alias'     => 'green',
                                'Position'  => '20',
                                'Attribute' => 'MyPreDefLocalizedString'
                            },
                            'LocalizedAttributes' => {
                                'en' => {
                                    'ValueLocString' => 'green'
                                }
                            }
                        },
                        {
                            'Attributes'          => {
                                'Alias'     => 'white',
                                'Position'  => '30',
                                'Attribute' => 'MyPreDefLocalizedString'
                            },
                            'LocalizedAttributes' => {
                                'en' => {
                                    'ValueLocString' => 'white'
                                }
                            }
                        }
                    ],
                    'Attributes'          => {
                        'Alias'          => 'MyPreDefLocalizedString',
                        'IsSearchFilter' => '0',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'PreDefLocalizedString',
                        'IsVisible'      => '1',
                        'Position'       => '10',
                        'IsObject'       => '1',
                        'IsHTML'         => '0',
                        'IsUserDefined'  => '1'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyPreDefLocalizedString'
                        }
                    }
                },
                {
                    'PreDefAttributes'    => [],
                    'Attributes'          => {
                        'Alias'          => 'MyTime',
                        'IsSearchFilter' => '0',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'LocalizedString',
                        'IsVisible'      => '1',
                        'Position'       => '70',
                        'IsObject'       => '0',
                        'IsHTML'         => '0',
                        'IsUserDefined'  => '1'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyTime'
                        }
                    }
                }
            ]
        }
    },
    NewValues     => {
        'ProductClass'  => {
            'Attributes'            => {
                'Alias'    => '',
                'Position' => '111',
            },
            'LocalizedAttributes'   => {
                'en' => {
                    'Name' => 'Standard',
                }
            },
            'ProductTypeAttributes' => [
                {
                    Attributes            => {
                        'Alias'          => 'MyString',
                        'IsHTML'         => '0',
                        'IsObject'       => '0',
                        'IsSearchFilter' => '0',
                        'IsUserDefined'  => '1',
                        'IsVisible'      => '1',
                        'Position'       => '112',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'LocalizedString'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyString'
                        }
                    },
                    'PreDefAttributes'    => [],
                },
                {
                    Attributes            => {
                        'Alias'          => 'MyString2',
                        'IsHTML'         => '0',
                        'IsObject'       => '0',
                        'IsSearchFilter' => '0',
                        'IsUserDefined'  => '1',
                        'IsVisible'      => '1',
                        'Position'       => '113',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'LocalizedString'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyString2'
                        }
                    },
                    'PreDefAttributes'    => [],
                }
            ]
        },
        'MyProductType' => {
            'Attributes'            => {
                'Alias'    => 'MyProductType',
                'Position' => '222'
            },
            'LocalizedAttributes'   => {
                'en' => {
                    'Name' => 'MyProductType'
                }
            },
            'ProductTypeAttributes' => [
                {
                    'PreDefAttributes'    => [],
                    'Attributes'          => {
                        'Alias'          => 'MyDate',
                        'IsSearchFilter' => '0',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'LocalizedString',
                        'IsVisible'      => '1',
                        'Position'       => '223',
                        'IsObject'       => '0',
                        'IsHTML'         => '0',
                        'IsUserDefined'  => '1'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyDate'
                        }
                    }
                },
                {
                    'PreDefAttributes'    => [],
                    'Attributes'          => {
                        'Alias'          => 'MyDateTime',
                        'IsSearchFilter' => '0',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'LocalizedString',
                        'IsVisible'      => '1',
                        'Position'       => '224',
                        'IsObject'       => '0',
                        'IsHTML'         => '0',
                        'IsUserDefined'  => '1'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyDateTime'
                        }
                    }
                },
                {
                    'PreDefAttributes'    => [],
                    'Attributes'          => {
                        'Alias'          => 'MyFloat',
                        'IsSearchFilter' => '0',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'LocalizedString',
                        'IsVisible'      => '1',
                        'Position'       => '30',
                        'IsObject'       => '0',
                        'IsHTML'         => '0',
                        'IsUserDefined'  => '1'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyFloat'
                        }
                    }
                },
                {
                    'PreDefAttributes'    => [],
                    'Attributes'          => {
                        'Alias'          => 'MyInteger',
                        'IsSearchFilter' => '0',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'LocalizedString',
                        'IsVisible'      => '1',
                        'Position'       => '20',
                        'IsObject'       => '0',
                        'IsHTML'         => '0',
                        'IsUserDefined'  => '1'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyInteger'
                        }
                    }
                },
                {
                    'PreDefAttributes'    => [],
                    'Attributes'          => {
                        'Alias'          => 'MyMoney',
                        'IsSearchFilter' => '0',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'LocalizedString',
                        'IsVisible'      => '1',
                        'Position'       => '40',
                        'IsObject'       => '0',
                        'IsHTML'         => '0',
                        'IsUserDefined'  => '1'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyMoney'
                        }
                    }
                },
                {
                    'PreDefAttributes'    => [
                        {
                            'Attributes'          => {
                                'Alias'     => 'blau',
                                'Position'  => '10',
                                'Attribute' => 'MyPreDefLocalizedString'
                            },
                            'LocalizedAttributes' => {
                                'en' => {
                                    'ValueLocString' => 'blue'
                                }
                            }
                        },
                        {
                            'Attributes'          => {
                                'Alias'     => 'green',
                                'Position'  => '20',
                                'Attribute' => 'MyPreDefLocalizedString'
                            },
                            'LocalizedAttributes' => {
                                'en' => {
                                    'ValueLocString' => 'green'
                                }
                            }
                        },
                        {
                            'Attributes'          => {
                                'Alias'     => 'white',
                                'Position'  => '30',
                                'Attribute' => 'MyPreDefLocalizedString'
                            },
                            'LocalizedAttributes' => {
                                'en' => {
                                    'ValueLocString' => 'white'
                                }
                            }
                        }
                    ],
                    'Attributes'          => {
                        'Alias'          => 'MyPreDefLocalizedString',
                        'IsSearchFilter' => '0',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'PreDefLocalizedString',
                        'IsVisible'      => '1',
                        'Position'       => '10',
                        'IsObject'       => '1',
                        'IsHTML'         => '0',
                        'IsUserDefined'  => '1'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyPreDefLocalizedString'
                        }
                    }
                },
                {
                    'PreDefAttributes'    => [
                        {
                            'Attributes'          => {
                                'Alias'     => 'value1',
                                'Position'  => '10',
                                'Attribute' => 'MyPreDefLocalizedString2'
                            },
                            'LocalizedAttributes' => {
                                'en' => {
                                    'ValueLocString' => 'value1'
                                }
                            }
                        },
                        {
                            'Attributes'          => {
                                'Alias'     => 'value2',
                                'Position'  => '20',
                                'Attribute' => 'MyPreDefLocalizedString2'
                            },
                            'LocalizedAttributes' => {
                                'en' => {
                                    'ValueLocString' => 'value2'
                                }
                            }
                        }
                    ],
                    'Attributes'          => {
                        'Alias'          => 'MyPreDefLocalizedString2',
                        'IsSearchFilter' => '0',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'PreDefLocalizedString',
                        'IsVisible'      => '1',
                        'Position'       => '10',
                        'IsObject'       => '1',
                        'IsHTML'         => '0',
                        'IsUserDefined'  => '1'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyPreDefLocalizedString2'
                        }
                    }
                },
                {
                    'PreDefAttributes'    => [],
                    'Attributes'          => {
                        'Alias'          => 'MyTime',
                        'IsSearchFilter' => '0',
                        'ShowInOwnTab'   => '0',
                        'Type'           => 'LocalizedString',
                        'IsVisible'      => '1',
                        'Position'       => '225',
                        'IsObject'       => '0',
                        'IsHTML'         => '0',
                        'IsUserDefined'  => '1'
                    },
                    'LocalizedAttributes' => {
                        'en' => {
                            'Description' => undef,
                            'Name'        => 'MyTime'
                        }
                    }
                }
            ]
        }

    },
);

use constant FILENAME => 'producttypes.json';
use constant FILE_DIR => [ qw(ShopData.d export TestShop) ];
use constant FILE_LINES => 2;

sub testExportWithProductTypesExportImportBase2Prep {
    my $Shop = $TestData{TestShop};

    note('test ProductTypesExportDriver (Base2Prep)');
    isnt($Shop, undef, 'Testshop exists');
    is(scalar @{$Shop->get('Languages')}, 1, 'Testshop has now one Language');
    is(scalar @{$Shop->get('SiteLocales')}, 1, 'Testshop has one SiteLocale');
    my $Exporter = ExportProductTypesJSON($Shop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, FILE_LINES, 'Export file lines');
    ok($Exporter->fileName =~ /${\FILENAME}/, "export file name matches ${\FILENAME}");
    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(catfile(@{FILE_DIR()}, 'base2prep_' . FILENAME), { encoding => 'utf8' });
    is($$sFileContent, $$sExpected, 'file content matches');

    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub testExportWithProductTypesExportImportPrep2Base {
    my $PrepShop = $TestData{PrepShop};

    note('test ProductTypesExportDriver (Prep2Base)');
    isnt($PrepShop, undef, 'PrepShop exists');
    is(scalar @{$PrepShop->get('Languages')}, 1, 'Testshop has now one Language');
    is(scalar @{$PrepShop->get('SiteLocales')}, 1, 'Testshop has one SiteLocale');
    ok($PrepShop->get('IsMigrationPrepShop'), 'PrepShop has IsMigrationPrepShop=1');
    my $Exporter = ExportProductTypesJSON($PrepShop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists ' . $Exporter->fileName);
    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, FILE_LINES, 'Export file lines');
    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(catfile(@{FILE_DIR()}, 'prep2base_' . FILENAME), { encoding => 'utf8' });
    is($$sFileContent, $$sExpected, 'file content matches');

    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub testImportWithProductTypesExportImportBase2Prep {

    my $Shop = $TestData{PrepShop};
    my $DirName = catdir('ShopData.d', 'import');
    note('test ProductTypesImportHandler (Base2Prep)');
    isnt($Shop, undef, 'Prepshop exists');

    my $ProductTypeFolder = $Shop->get('ProductTypes');
    #_compareProductTypes($Shop, $TestData{OldValues}, 'Old');
    my $Importer = ImportProductTypesJSON($Shop, $DirName);
    ok($Importer->fileName() =~ /TestShop.base2prep_producttypes[.]json/, 'ImportFileName was constructed properly');
    isnt($Shop->get('ProductClass'), undef, 'Shop->ProductClass exists');
    _compareProductTypes($Shop, $TestData{NewValues}, 'Imported');

    my $Alias = 'MyProductType';
    my $ProductType = $ProductTypeFolder->child($Alias);
    ok($ProductTypeFolder->existsChild($Alias), "ProductType $Alias exists");
    $ProductType->delete;
    ok(!$ProductTypeFolder->existsChild($Alias), "ProductType $Alias deleted");

    note('Running Importer again');
    $Importer = ImportProductTypesJSON($Shop, $DirName);
    ok($ProductTypeFolder->existsChild($Alias), "ProductType $Alias reimported");
    _compareProductTypes($Shop, $TestData{NewValues}, 'Reimported');
}

sub _compareProductTypes {
    my ($Shop, $TargetValues, $Note) = @_;

    my $ProductTypeFolder = $Shop->get('ProductTypes');
    foreach my $ProductTypeAlias (keys %$TargetValues) {
        my $ProductType = $ProductTypeAlias eq 'ProductClass' ?
            $Shop->get('ProductClass') :
            $ProductTypeFolder->child($ProductTypeAlias);
        my $Values = GetProductTypeData($Shop, $ProductType);
        eq_or_diff($TargetValues->{$ProductTypeAlias}, $Values, "$Note product type values for '$ProductTypeAlias' match");
    }
}

sub testImportWithProductTypesExportImportPrep2Base {

    my $Shop = $TestData{TestShop};
    my $DirName = catdir('ShopData.d', 'import');
    note('test ProductTypesImportHandler (Prep2Base)');
    isnt($Shop, undef, 'Shop exists');

    # ResetProduct converts every Attribute to Type LocalizedStringAttribute
    note('Running ResetProduct()');
    DE_EPAGES::ShopMigration::API::ResetShopHandler::ResetProduct($Shop);
    # necessary because we do direct SQL Updates
    ResetFactoryCaches();

    my $ProductTypeFolder = $Shop->get('ProductTypes');
    _compareProductTypes($Shop, $TestData{OldValues}, 'Old');

    my $Importer = ImportProductTypesJSON($Shop, $DirName);
    note('Running Importer');
    ok($Importer->fileName() =~ /TestShop.prep2base_producttypes[.]json/, 'ImportFileName was constructed properly');
    isnt($Shop->get('ProductClass'), undef, 'Shop->ProductClass exists');
    _compareProductTypes($Shop, $TestData{NewValues}, 'Imported');

    my $Alias = 'MyProductType';
    my $ProductType = $ProductTypeFolder->child($Alias);
    ok($ProductTypeFolder->existsChild($Alias), "ProductType $Alias exists");
    $ProductType->delete;
    ok(!$ProductTypeFolder->existsChild($Alias), "ProductType $Alias deleted");

    note('Running Importer again');
    $Importer = ImportProductTypesJSON($Shop, $DirName);
    ok($ProductTypeFolder->existsChild($Alias), "ProductType $Alias reimported");
    _compareProductTypes($Shop, $TestData{NewValues}, 'Reimported');
}


# We need to create a fake ShopTypeRef UnityDE + ECom5000 in case they don't exists
# otherwise the Mapper won't work
sub init {
    my $self = shift;

    $self->SUPER::init;
    my $ShopAlias = $TestData{TestShopAlias};
    my $System = LoadRootObject();
    my $Shop;
    my $TestFolder = $System->folder('Test')->folder('Shops');
    my $ShopTypeRefFolder = $System->folder('ShopTypeRefs');
    my $Class = LoadClassByAlias('ShopTypeRef');
    $Class->insertObject({ Alias => 'UnityDE', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('UnityDE'));
    $Class->insertObject({ Alias => 'ECom5000', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('ECom5000'));
    if ($TestFolder->existsChild($ShopAlias)) {
        $Shop = $TestFolder->child($ShopAlias);
    } else {
        $Shop = $TestFolder->insertChild('Shop', {
            Alias                   => $ShopAlias,
            ShopTypeRef             => $ShopTypeRefFolder->child('ECom5000'),
            WebServerScriptNamePart => $ShopAlias,
        });
        SetShopFeatureValues($Shop,
            [
                { Alias => 'ProductTypes', MaxValue => '100' }, { Alias => 'ProductTypeAttributes', MaxValue => '15' },
                { Alias => 'Products', MaxValue => '100' }, { Alias => 'ProductMaxSubProducts', MaxValue => '50' },
                { Alias => 'Variations', MaxValue => '100' },
            ]);
    }
    $TestData{TestShop} = $Shop;
    $TestData{PrepShop} = ExistsPrepShop($Shop) ? $Shop->get('MigrationPrepShop') : CreatePrepShop($Shop);
    foreach my $JSONFile (@{GetFileNames('ShopData.d', { 'recursive' => 1 })}) {
        Text2Unix($JSONFile) if ($^O eq 'linux');
    }
    ImportObjects('ImportFiles/ProductTypesBase.xml', $TestData{TestShop}->pathString());
    ImportObjects('ImportFiles/ProductTypesNow.xml', $TestData{PrepShop}->pathString());
    return;
}

sub cleanup {
    my $self = shift;

    my $ShopAlias = $TestData{TestShopAlias};
    my $TestFolder = LoadRootObject()->folder('Test')->folder('Shops');
    if ($TestFolder->existsChild($ShopAlias)) {
        my $Shop = $TestFolder->child($ShopAlias);
        my $PrepShop = $Shop->get('MigrationPrepShop');
        $PrepShop->delete() if (defined $PrepShop);
        $Shop->delete();
        LoadRootObject()->folder('Test')->delete();
    }
    DeleteDirectory($TmpDir, { 'recursive' => 1 });
    return;
}


__PACKAGE__->runScript();

1;
