#========================================================================================
# package      DE_EPAGES::ShopMigration::t::ResetShopHandler
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::ShopMigration::API::ResetShopHandler
#========================================================================================
package DE_EPAGES::ShopMigration::t::ResetShopHandler;
use base DE_EPAGES::Test::API::StoreUnitTest;

use strict;

use utf8;
use warnings;

## no critic (RequireCorrectImports)
use DE_EPAGES::Catalog::API::Shop qw(SetPageSize);
use DE_EPAGES::Core::API::PerlTools qw(GetCurrentDateTime);
use DE_EPAGES::Core::API::File qw(WriteFile ExistsFile);
use DE_EPAGES::CrossSelling::API::Table::AutoCrossSelling qw(GetAllInfoAutoCrossSellingsByShopID);
use DE_EPAGES::Database::API::Connection qw(GetCurrentDBHandle);
use DE_EPAGES::GoogleAdWords::API::Table::GoogleAdWordsCode qw(InsertGoogleAdWordsCode GetPKeyGoogleAdWordsCodesByShopID);
use DE_EPAGES::Object::API::Factory qw(LoadRootObject LoadClassByAlias ExistsObject LoadObject LoadObjectByPath);
use DE_EPAGES::Object::API::Language qw(GetPKeyLanguageByCode);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw(ImportObjects);
use DE_EPAGES::Object::API::Table::BooleanAttribute qw(GetPKeyBooleanAttributesByAttributeID);
use DE_EPAGES::Object::API::Table::DateTimeAttribute qw(GetPKeyDateTimeAttributesByAttributeID);
use DE_EPAGES::Object::API::Table::FloatAttribute qw(GetPKeyFloatAttributesByAttributeID);
use DE_EPAGES::Object::API::Table::IntegerAttribute qw(GetPKeyIntegerAttributesByAttributeID);
use DE_EPAGES::Object::API::Table::MoneyAttribute qw(GetPKeyMoneyAttributesByAttributeID);
use DE_EPAGES::Object::API::Table::StringAttribute qw(GetPKeyStringAttributesByAttributeID);
use DE_EPAGES::ProductRecommendation::API::Table::ProductRecommendationUser qw(
    InsertProductRecommendationUser GetPKeyProductRecommendationUsersByShopID);
use DE_EPAGES::ShoppingList::API::Constants qw(UNITY_SHOPPINGLIST_ALIAS);
use DE_EPAGES::ShopMigration::API::MigrationShop qw(SetShopFeatureValues);
use DE_EPAGES::ShopMigration::API::ResetShopHandler qw(ResetBaseShop);
use File::Spec::Functions;
use Test::More;
use Test::Differences qw(context_diff eq_or_diff);
## use critic

if ($ENV{SKIP_IN_PR_TESTS}) {
    plan skip_all => 'Skipping it because SKIP_IN_PR_TESTS == 1';
} else {
    plan tests => 263;
}

sub test {
    context_diff();
    testResetShopHandler();
    done_testing();
}

my %TestData = (
    TestShop      => undef,
    TestShopAlias => 'TestResetBaseShop',
);

sub testResetShopHandler {
    note('test ResetShopHandler');
    my $Shop = $TestData{TestShop};
    isnt($Shop, undef, 'epages6 Testshop created');

    # importResetBaseShopCartridgeImportFile (all Cartridges)
    my $Expected = getTestData()->{Init};
    my $Current = getCurrentValues($Shop);
    is_deeply($Current, $Expected, 'Shop has expected settings with non-empty/non-zero values');
    # ResetShop
    my @Merchants = grep {$_->get('IsSuperMerchant') == 0} @{$Shop->child('Groups')->child('Merchants')->directMembers};
    is(scalar @Merchants, 2, 'Shop has some non-supermerchant users');
    # ResetShopMailType
    is($Shop->folder('MailTypes')->existsChild('NewPassword'), 1, 'Shop has mailtype NewPassword');
    # ResetCrossSelling
    is(existsShopObjectsbyClassAlias($Shop, 'ProductComparison'), 1, 'Shop has ProductComparison objects');
    is(existsShopObjectsbyClassAlias($Shop, 'Accessory'), 1, 'Shop has Accessory objects');
    is(GetAllInfoAutoCrossSellingsByShopID($Shop->id)->count(), 1, 'Shop has one AutoCrossSelling entry');
    # ResetDesign
    is(scalar @{$Shop->get('Styles')}, 3, 'Shop has multiple styles');
    is($Shop->get('Logo'), 'Logo.gif', 'Shop has Logo set');
    is($Shop->get('FavIcon'), 'favicon.ico', 'Shop has FavIcon set');
    # ResetProduct
    is(existsBookingHours($Shop), 1, 'Shop has BusinessHours/Calendar');
    is($Shop->folder('MailTypes')->existsChild('AppointmentAccepted'), 1, 'Shop has mailtype AppointmentAccepted');
    is($Shop->folder('MailTypes')->existsChild('AppointmentCancelled'), 1, 'Shop has mailtype AppointmentCancelled');
    is($Shop->folder('MailTypes')->existsChild('AppointmentCreated'), 1, 'Shop has mailtype AppointmentCreated');
    is($Shop->folder('MailTypes')->existsChild('DownloadActivated'), 1, 'Shop has mailtype DownloadActivated');
    is($Shop->folder('MailTypes')->existsChild('ProductAvailibityNotification'), 1, 'Shop has mailtype ProductAvailibityNotification');
    is($Shop->folder('MailTypes')->existsChild('ProductInquiry'), 1, 'Shop has mailtype ProductInquiry');
    checkProductType($Shop, 'PreReset');
    # ResetCatalog
    is(existsShopObjectsbyClassAlias($Shop, 'CategorySpecialOffer'), 1, 'Shop has CategorySpecialOffer objects');
    is(startPagesAreDefault($Shop), 0, 'Startpages have user-defined settings (Intro)');
    # ResetCustomer
    is(existsShopObjectsbyClassAlias($Shop, 'CustomerGroup'), 1, 'Shop has CustomerGroup objects');
    is(existsUnAssignedAddress($Shop), 1, 'Shop has unassigned address objects');
    is($Shop->folder('MailTypes')->existsChild('UserAddressChanged'), 1, 'Shop has mailtype UserAddressChanged');
    is($Shop->folder('MailTypes')->existsChild('UserByMerchantWelcome'), 1, 'Shop has mailtype UserByMerchantWelcome');
    is($Shop->folder('MailTypes')->existsChild('UserEmailChanged'), 1, 'Shop has mailtype UserEmailChanged');
    # ResetProductRating
    is(existsShopObjectsbyClassAlias($Shop, 'ProductRating'), 1, 'Shop has ProductRating objects');
    is($Shop->folder('MailTypes')->existsChild('NewProductRating'), 1, 'Shop has mailtype NewProductRating');
    is($Shop->folder('MailTypes')->existsChild('ProductRatingInvitation'), 1, 'Shop has mailtype ProductRatingInvitation');
    # ResetProductRecommendation
    is(existsProductRecommendations($Shop), 1, 'Shop has ProductRecommendations');
    is($Shop->folder('MailTypes')->existsChild('ProductRecommendation'), 1, 'Shop has mailtype ProductRecommendation');
    # ResetGoogleAdWords
    is(existsGoogleAdWords($Shop), 1, 'Shop has GoogleAdwords');
    # ResetSafeBuy
    is($Shop->get('SafeBuyShield'), 'SafeBuyShield', 'Shop has SafeBuyShield');
    is(ExistsFile(catfile($Shop->get('PublicPath'), $Shop->get('SafeBuyShield'))), 1, 'Shop has SafeBuyShieldFile');
    # ResetEkomi
    is($Shop->folder('MailTypes')->existsChild('ProductRecommendation'), 1, 'Shop has mailtype EkomiRating');
    # ResetShoppingList
    my $ShoppingLists = $Shop->folder('Users')->child('user1')->folder('ShoppingLists');
    is(scalar @{$ShoppingLists->findChildren()}, 2, 'User user1 has two shoppinglists');
    # ResetShippingOption
    is(existsShopObjectsbyClassAlias($Shop, 'ShippingOption'), 1, 'Shop has ShippingOption objects');
    # ResetPriceList
    is(existsShopObjectsbyClassAlias($Shop, 'PriceList'), 1, 'Shop has PriceList objects');
    # ResetCoupon
    is(existsShopObjectsbyClassAlias($Shop, 'CouponCampaign'), 1, 'Shop has CouponCampaign objects');
    # ResetShopNewsletter
    is(existsShopObjectsbyClassAlias($Shop, 'NewsletterCampaign'), 1, 'Shop has NewsletterCampaign objects');
    is($Shop->folder('MailTypes')->existsChild('SubscribeConfirmation'), 1, 'Shop has mailtype SubscribeConfirmation');
    is($Shop->folder('MailTypes')->existsChild('SubscribeConfirmationRequest'), 1, 'Shop has mailtype SubscribeConfirmationRequest');
    is($Shop->folder('MailTypes')->existsChild('UnsubscribeConfirmation'), 1, 'Shop has mailtype UnsubscribeConfirmation');
    is($Shop->folder('MailTypes')->existsChild('UnsubscribeConfirmationRequest'), 1, 'Shop has mailtype UnsubscribeConfirmationRequest');
    # ResetCreditCheck
    is(scalar @{$Shop->folder('CreditCheckProviderConfigs')->findChildren()}, 1, 'Shop has CreditCheckProviderConfigs');
    is(scalar @{$Shop->folder('CreditCheckAgencyConfigs')->findChildren()},, 1, 'Shop has CreditCheckAgencyConfigs');
    # ResetSearch
    is(existsShopObjectsbyClassAlias($Shop, 'SearchTerm'), 1, 'Shop has SearchTerm objects');
    # ResetMediaGallery
    is($Shop->child('MediaGallery')->existsDirectory('TestDir'), 1, 'Shop has MediaGallery with files');
    # ResetContent
    is(existsShopObjectsbyClassAlias($Shop, 'Link'), 1, 'Shop has Link objects');
    is(existsShopObjectsbyClassAlias($Shop, 'Article'), 1, 'Shop has Article objects');
    is(existsShopObjectsbyClassAlias($Shop, 'SiteMap'), 1, 'Shop has SiteMap objects');
    # ResetImageGallery
    is(existsShopObjectsbyClassAlias($Shop, 'ImageGallery'), 1, 'Shop has ImageGallery objects');
    # ResetForum
    is(existsShopObjectsbyClassAlias($Shop, 'Forum'), 1, 'Shop has Forum objects');
    is(existsShopObjectsbyClassAlias($Shop, 'GuestBook'), 1, 'Shop has GuestBook objects');
    is(existsShopObjectsbyClassAlias($Shop, 'Blog'), 1, 'Shop has Blog objects');
    is($Shop->folder('MailTypes')->existsChild('NewPost'), 1, 'Shop has mailtype NewPost');
    # ResetCustomForm
    is(existsShopObjectsbyClassAlias($Shop, 'CustomForm'), 1, 'Shop has CustomForm objects');
    # ResetEtracker
    is(PChasEtrackerURLExtension($Shop), 1, 'Any PortalConfig has EtrackerURLExtension set');
    # ResetPayment
    is(existsShopObjectsbyClassAlias($Shop, 'InterestRateMatrix'), 1, 'Shop has InterestRateMatrix objects');
    is(hasPaymentWithHirePurchase($Shop), 1, 'Shop has Payment with IsHirePurchaseEnabled=1');
    # ResetPaymentTypes
    is(hasPaymentOfType($Shop, 'FixPrice'), 1, 'Shop has payment FixPrice');
    is(hasPaymentOfType($Shop, 'FreeLimit'), 1, 'Shop has payment FreeLimit');
    is(hasPaymentOfType($Shop, 'DirectDebitViaFax'), 1, 'Shop has payment DirectDebitViaFax');
    is(hasPaymentOfType($Shop, 'DirectCreditCard'), 1, 'Shop has payment DirectCreditCard');
    # ResetAmazonPayments
    is($Shop->folder('MailTypes')->existsChild('AmazonPaymentsPaymentDeclined'), 1, 'Shop has mailtype AmazonPaymentsPaymentDeclined');
    # ResetMoneybookers
    is(hasPaymentOfType($Shop, 'MoneybookersCreditCard'), 1, 'Shop has payment MoneybookersCreditCard');
    # ResetOgone
    is(hasPaymentOfType($Shop, 'Acceptance'), 1, 'Shop has payment Acceptance');
    # ResetPayPal
    is(hasPaymentOfType($Shop, 'PayPal'), 1, 'Shop has payment PayPal');
    is(hasPaymentOfType($Shop, 'PayPalIntegralEvolution'), 1, 'Shop has payment PayPalIntegralEvolution');
    # ResetPayPalPro
    is(hasPaymentOfType($Shop, 'PayPalPro'), 1, 'Shop has payment PayPalPro');
    # ResetWorldPay
    is(hasPaymentOfType($Shop, 'WorldPay'), 1, 'Shop has payment WorldPay');
    # ResetSipsAtos
    is(hasPaymentOfType($Shop, 'SipsAtos'), 1, 'Shop has payment SipsAtos');
    # ResetSPSBillingOnline [not installed in RnD Standard]
    # is(hasPaymentOfType($Shop, 'SPSBillingOnline'), 1, 'Shop has payment SPSBillingOnline');
    # ResetDHL
    is(hasShippingOfType($Shop, 'DHL'), 1, 'Shop has shipping with filfillmenttype DHL');
    # ResetSDA
    is(hasShippingOfType($Shop, 'SDA'), 1, 'Shop has shipping with filfillmenttype SDA');
    # ResetSEUR
    is(hasShippingOfType($Shop, 'SEUR'), 1, 'Shop has shipping with filfillmenttype SEUR');
    # ResetUPS
    is(hasShippingOfType($Shop, 'UPS'), 1, 'Shop has shipping with filfillmenttype UPS');
    # ResetEnvialia ne [not installed in RnD Standard]
    # is(hasShippingOfType($Shop, 'Envialia'), 1, 'Shop has shipping with filfillmenttype Envialia');
    # ResetKelkoo
    is(hasPortalConfigOfType($Shop, 'KelkooConfig'), 1, 'Shop has portalconfig with type KelkooConfig');
    # ResetPangora
    is(hasPortalConfigOfType($Shop, 'PangoraConfig'), 1, 'Shop has portalconfig with type PangoraConfig');
    # ResetPreissuchmaschine
    is(hasPortalConfigOfType($Shop, 'PreissuchmaschineConfig'), 1, 'Shop has portalconfig with type PreissuchmaschineConfig');
    # ResetPriceMinister
    is(hasPortalConfigOfType($Shop, 'PriceMinisterConfig'), 1, 'Shop has portalconfig with type PriceMinisterConfig');
    # ResetShopzilla
    is(hasPortalConfigOfType($Shop, 'ShopzillaConfig'), 1, 'Shop has portalconfig with type ShopzillaConfig');
    # ResetToppreise
    is(hasPortalConfigOfType($Shop, 'ToppreiseConfig'), 1, 'Shop has portalconfig with type ToppreiseConfig');
    # ResetTwenga
    is(hasPortalConfigOfType($Shop, 'TwengaConfig'), 1, 'Shop has portalconfig with type TwengaConfig');
    # ResetEbay
    is(scalar @{$Shop->get('EbayUserTemplates')}, 2, 'Shop has some EbayUserTemplates');
    checkDefaultAuction($Shop, 'PreReset');

    #------------------------------------------------------------------------------------------------------------------#
    note('running ResetBaseShop');
    ResetBaseShop($Shop);
    #------------------------------------------------------------------------------------------------------------------#

    $Expected = getTestData()->{Reset};
    $Current = getCurrentValues($Shop);

    is_deeply($Current, $Expected, 'All Shop settings successfully reset');
    # ResetShop
    @Merchants = grep {$_->get('IsSuperMerchant') == 0} @{$Shop->child('Groups')->child('Merchants')->directMembers};
    is(scalar @Merchants, 0, 'Shop only has the supermerchant user');
    # ResetShopMailType
    is($Shop->folder('MailTypes')->existsChild('NewPassword'), '', 'Shop has no mailtype NewPassword');
    # ResetCrossSelling
    is(existsShopObjectsbyClassAlias($Shop, 'ProductComparison'), 0, 'Shop has no ProductComparison objects');
    is(existsShopObjectsbyClassAlias($Shop, 'Accessory'), 0, 'Shop has no Accessory objects');
    is(GetAllInfoAutoCrossSellingsByShopID($Shop->id)->count(), 0, 'Shop has no AutoCrossSelling entries');
    # ResetDesign
    is($Shop->get('Logo'), undef, 'Shop has Logo reset');
    is($Shop->get('FavIcon'), undef, 'Shop has FavIcon reset');
    is(scalar @{$Shop->get('Styles')}, 2, 'Shop has two styles');
    is($Shop->get('DefaultStyle')->alias,'MyDesign', 'Shop has DefaultStyle MyDesign');
    # ResetProduct
    is(existsBookingHours($Shop), 0, 'Shop has no BusinessHours/Calendar');
    is($Shop->folder('MailTypes')->existsChild('AppointmentAccepted'), '', 'Shop has no mailtype AppointmentAccepted');
    is($Shop->folder('MailTypes')->existsChild('AppointmentCancelled'), '', 'Shop has no mailtype AppointmentCancelled');
    is($Shop->folder('MailTypes')->existsChild('AppointmentCreated'), '', 'Shop has no mailtype AppointmentCreated');
    is($Shop->folder('MailTypes')->existsChild('DownloadActivated'), '', 'Shop has no mailtype DownloadActivated');
    is($Shop->folder('MailTypes')->existsChild('ProductAvailibityNotification'), '', 'Shop has no mailtype ProductAvailibityNotification');
    is($Shop->folder('MailTypes')->existsChild('ProductInquiry'), '', 'Shop has no mailtype ProductInquiry');
    checkProductType($Shop, 'PostReset');
    # ResetCatalog
    is(existsShopObjectsbyClassAlias($Shop, 'CategorySpecialOffer'), 0, 'Shop has no CategorySpecialOffer objects');
    is(startPagesAreDefault($Shop), 1, 'Startpages have default values');
    # ResetCustomer
    is(existsShopObjectsbyClassAlias($Shop, 'CustomerGroup'), 0, 'Shop has no CustomerGroup objects');
    is($Shop->folder('MailTypes')->existsChild('UserAddressChanged'), '', 'Shop has no mailtype UserAddressChanged');
    is($Shop->folder('MailTypes')->existsChild('UserByMerchantWelcome'), '', 'Shop has no mailtype UserByMerchantWelcome');
    is($Shop->folder('MailTypes')->existsChild('UserEmailChanged'), '', 'Shop has no mailtype UserEmailChanged');
    is(existsUnAssignedAddress($Shop), 0, 'Shop has no unassigned address objects');
    # ResetProductRating
    is(existsShopObjectsbyClassAlias($Shop, 'ProductRating'), 0, 'Shop has no ProductRating objects');
    is($Shop->folder('MailTypes')->existsChild('NewProductRating'), '', 'Shop has no mailtype NewProductRating');
    is($Shop->folder('MailTypes')->existsChild('ProductRatingInvitation'), '', 'Shop has no mailtype ProductRatingInvitation');
    # ResetProductRecommendation
    is(existsProductRecommendations($Shop), 0, 'Shop has no ProductRecommendations');
    is($Shop->folder('MailTypes')->existsChild('ProductRecommendation'), '', 'Shop has mailtype ProductRecommendation');
    # ResetGoogleAdWords
    is(existsGoogleAdWords($Shop), 0, 'Shop has no GoogleAdWords');
    # ResetSafeBuy
    is($Shop->get('SafeBuyShield'), undef, 'Shop has SafeBuyShield');
    is(ExistsFile(catfile($Shop->get('PublicPath'), $Shop->get('SafeBuyShield'))), 0, 'Shop has no SafeBuyShieldFile');
    # ResetEkomi
    is($Shop->folder('MailTypes')->existsChild('ProductRecommendation'), '', 'Shop has no mailtype EkomiRating');
    # ResetShoppingList
    $ShoppingLists = $Shop->folder('Users')->child('user1')->folder('ShoppingLists');
    is(scalar @{$ShoppingLists->findChildren()}, 1, 'User user1 has one shoppinglist');
    is($ShoppingLists->existsChild(UNITY_SHOPPINGLIST_ALIAS), 1, 'Shoppinglist has name UNITY_SHOPPINGLIST_ALIAS');
    # ResetShippingOption
    is(existsShopObjectsbyClassAlias($Shop, 'ShippingOption'), 0, 'Shop has no ShippingOption objects');
    # ResetPriceList
    is(existsShopObjectsbyClassAlias($Shop, 'PriceList'), 0, 'Shop has no PriceList objects');
    # ResetCoupon
    is(existsShopObjectsbyClassAlias($Shop, 'CouponCampaign'), 0, 'Shop has no CouponCampaign objects');
    # ResetShopNewsletter
    is(existsShopObjectsbyClassAlias($Shop, 'NewsletterCampaign'), 0, 'Shop has no NewsletterCampaign objects');
    is($Shop->folder('MailTypes')->existsChild('SubscribeConfirmation'), '', 'Shop has no mailtype SubscribeConfirmation');
    is($Shop->folder('MailTypes')->existsChild('SubscribeConfirmationRequest'), '', 'Shop has no mailtype SubscribeConfirmationRequest');
    is($Shop->folder('MailTypes')->existsChild('UnsubscribeConfirmation'), '', 'Shop has no mailtype UnsubscribeConfirmation');
    is($Shop->folder('MailTypes')->existsChild('UnsubscribeConfirmationRequest'), '', 'Shop has no mailtype UnsubscribeConfirmationRequest');
    # ResetCreditCheck
    is(scalar @{$Shop->folder('CreditCheckProviderConfigs')->findChildren()}, 0, 'Shop has no CreditCheckProviderConfigs');
    is(scalar @{$Shop->folder('CreditCheckAgencyConfigs')->findChildren()},, 0, 'Shop has no CreditCheckAgencyConfigs');
    # ResetSearch
    is(existsShopObjectsbyClassAlias($Shop, 'SearchTerm'), 0, 'Shop has no SearchTerm objects');
    # ResetMediaGallery
    is($Shop->child('MediaGallery')->existsDirectory('TestDir'), 0, 'Shop has empty MediaGallery');
    # ResetContent
    is(existsShopObjectsbyClassAlias($Shop, 'Link'), 0, 'Shop has no Link objects');
    is(existsShopObjectsbyClassAlias($Shop, 'Article'), 0, 'Shop has no Article objects');
    is(existsShopObjectsbyClassAlias($Shop, 'SiteMap'), 0, 'Shop has no SiteMap objects');
    # ResetImageGallery
    is(existsShopObjectsbyClassAlias($Shop, 'ImageGallery'), 0, 'Shop has no ImageGallery objects');
    # ResetForum
    is(existsShopObjectsbyClassAlias($Shop, 'Forum'), 0, 'Shop has no Forum objects');
    is(existsShopObjectsbyClassAlias($Shop, 'GuestBook'), 0, 'Shop has no GuestBook objects');
    is(existsShopObjectsbyClassAlias($Shop, 'Blog'), 0, 'Shop has no Blog objects');
    is($Shop->folder('MailTypes')->existsChild('NewPost'), '', 'Shop has no mailtype NewPost');
    # ResetCustomForm
    is(existsShopObjectsbyClassAlias($Shop, 'CustomForm'), 0, 'Shop has no CustomForm objects');
    # ResetEtracker
    is(PChasEtrackerURLExtension($Shop), 0, 'No PortalConfig has EtrackerURLExtension set');
    # ResetPayment
    is(existsShopObjectsbyClassAlias($Shop, 'InterestRateMatrix'), 0, 'Shop has no InterestRateMatrix objects');
    is(hasPaymentWithHirePurchase($Shop), 0, 'Shop has no Payment with IsHirePurchaseEnabled=1');
    # ResetPaymentTypes
    is(hasPaymentOfType($Shop, 'FixPrice'), 0, 'Shop has no payment FixPrice');
    is(hasPaymentOfType($Shop, 'FreeLimit'), 0, 'Shop has no payment FreeLimit');
    is(hasPaymentOfType($Shop, 'DirectDebitViaFax'), 0, 'Shop has no payment DirectDebitViaFax');
    is(hasPaymentOfType($Shop, 'DirectCreditCard'), 0, 'Shop has no payment DirectCreditCard');
    # ResetAmazonPayments
    is($Shop->folder('MailTypes')->existsChild('AmazonPaymentsPaymentDeclined'), '', 'Shop has no mailtype AmazonPaymentsPaymentDeclined');
    # ResetMoneybookers
    is(hasPaymentOfType($Shop, 'MoneybookersCreditCard'), 0, 'Shop has no payment MoneybookersCreditCard');
    # ResetOgone
    is(hasPaymentOfType($Shop, 'Acceptance'), 0, 'Shop has no payment Acceptance');
    # ResetPayPal
    is(hasPaymentOfType($Shop, 'PayPal'), 0, 'Shop has no payment PayPal');
    is(hasPaymentOfType($Shop, 'PayPalIntegralEvolution'), 0, 'Shop has no payment PayPalIntegralEvolution');
    # ResetPayPalPro
    is(hasPaymentOfType($Shop, 'PayPalPro'), 0, 'Shop has no payment PayPalPro');
    # ResetWorldPay
    is(hasPaymentOfType($Shop, 'WorldPay'), 0, 'Shop has no payment WorldPay');
    # ResetSipsAtos
    is(hasPaymentOfType($Shop, 'SipsAtos'), 0, 'Shop has no payment SipsAtos');
    # ResetSPSBillingOnline [not installed in RnD Standard]
    # is(hasPaymentOfType($Shop, 'SPSBillingOnline'), 1, 'Shop has no payment SPSBillingOnline');
    # ResetDHL
    is(hasShippingOfType($Shop, 'DHL'), 0, 'Shop has no shipping with filfillmenttype DHL');
    # ResetSDA
    is(hasShippingOfType($Shop, 'SDA'), 0, 'Shop has no shipping with filfillmenttype SDA');
    # ResetSEUR
    is(hasShippingOfType($Shop, 'SEUR'), 0, 'Shop has no shipping with filfillmenttype SEUR');
    # ResetUPS
    is(hasShippingOfType($Shop, 'UPS'), 0, 'Shop has no shipping with filfillmenttype UPS');
    # ResetEnvialia ne [not installed in RnD Standard]
    # is(hasShippingOfType($Shop, 'Envialia'), 0, 'Shop has no shipping with filfillmenttype Envialia');
    # ResetKelkoo
    is(hasPortalConfigOfType($Shop, 'KelkooConfig'), 0, 'Shop has no portalconfig with type KelkooConfig');
    # ResetPangora
    is(hasPortalConfigOfType($Shop, 'PangoraConfig'), 0, 'Shop has no portalconfig with type PangoraConfig');
    # ResetPreissuchmaschine
    is(hasPortalConfigOfType($Shop, 'PreissuchmaschineConfig'), 0, 'Shop has no portalconfig with type PreissuchmaschineConfig');
    # ResetPriceMinister
    is(hasPortalConfigOfType($Shop, 'PriceMinisterConfig'), 0, 'Shop has no portalconfig with type PriceMinisterConfig');
    # ResetShopzilla
    is(hasPortalConfigOfType($Shop, 'ShopzillaConfig'), 0, 'Shop has no portalconfig with type ShopzillaConfig');
    # ResetToppreise
    is(hasPortalConfigOfType($Shop, 'ToppreiseConfig'), 0, 'Shop has no portalconfig with type ToppreiseConfig');
    # ResetTwenga
    is(hasPortalConfigOfType($Shop, 'TwengaConfig'), 0, 'Shop has no portalconfig with type TwengaConfig');
    # ResetEbay
    is(scalar @{$Shop->get('EbayUserTemplates')}, 0, 'Shop has no EbayUserTemplates');
    checkDefaultAuction($Shop, 'PostReset');
}

sub checkDefaultAuction {
    my ($Shop, $ShopType) = @_;

    my $Expected = {
        PreReset  => {
            'IsBoldTitle'        => 1,
            'IsHighlight'        => 1,
            'IsBordered'         => 1,
            'HitCounter'         => 'BasicStyle',
            'IsFeatured'         => 1,
            'IsHomePageFeatured' => 1,
            'GalleryUrl'         => 'http://url.com',
            'PictureUrl'         => 'http://url.com',
            'Gallery'            => 'Featured'
        },
        PostReset => {
            'IsBoldTitle'        => 0,
            'IsHighlight'        => 0,
            'IsBordered'         => 0,
            'HitCounter'         => 'HiddenStyle',
            'IsFeatured'         => 0,
            'IsHomePageFeatured' => 0,
            'GalleryUrl'         => undef,
            'PictureUrl'         => undef,
            'Gallery'            => 'None'
        }
    };
    my $DefaultAuction = $Shop->get('EbayDefaultAuction');
    my $Values = $DefaultAuction->get([ keys %{$Expected->{$ShopType}} ]);
    eq_or_diff($Values, $Expected->{$ShopType}, 'EbayDefaultAuction settings are as expected');
}

sub checkProductType {
    my ($Shop, $ShopType) = @_;

    my $Expected = {
        PreReset  => {
            MyString                     => {
                Type => 'String', Package => 'DE_EPAGES::Object::API::Attributes::DefaultAttribute' },
            MyPreDefString               => {
                Type => 'PreDefString', Package => 'DE_EPAGES::Object::API::Attributes::PreDefined' },
            MyLocalizedString            => {
                Type => 'LocalizedString', Package => 'DE_EPAGES::Object::API::Attributes::LocalizedAttribute' },
            MyPreDefLocalizedString      => {
                Type => 'PreDefLocalizedString', Package => 'DE_EPAGES::Object::API::Attributes::PreDefined' },
            MyPreDefMultiString          => {
                Type => 'PreDefMultiString', Package => 'DE_EPAGES::Object::API::Attributes::PreDefMulti' },
            MyPreDefMultiLocalizedString => {
                Type => 'PreDefMultiLocalizedString', Package => 'DE_EPAGES::Object::API::Attributes::PreDefMulti' },
            MyPreDefCustomerString       => {
                Type => 'PreDefCustomerString', Package => 'DE_EPAGES::Object::API::Attributes::PreDefMulti' },
            MyInteger                    => {
                Type => 'Integer', Package => 'DE_EPAGES::Object::API::Attributes::DefaultAttribute' },
            MyFloat                      => {
                Type => 'Float', Package => 'DE_EPAGES::Object::API::Attributes::DefaultAttribute' },
            MyBoolean                    => {
                Type => 'Boolean', Package => 'DE_EPAGES::Object::API::Attributes::DefaultAttribute' },
            MyMoney                      => {
                Type => 'Money', Package => 'DE_EPAGES::Object::API::Attributes::DefaultAttribute' },
            MyDateTime                   => {
                Type => 'DateTime', Package => 'DE_EPAGES::Object::API::Attributes::DefaultAttribute' },
            MyDate                       => {
                Type => 'Date', Package => 'DE_EPAGES::Object::API::Attributes::DefaultAttribute' },
            MyTime                       => {
                Type => 'Time', Package => 'DE_EPAGES::Object::API::Attributes::DefaultAttribute' },
            MyFile                       => {
                Type => 'File', Package => 'DE_EPAGES::Object::API::Attributes::DefaultAttribute' },
            MyLocalizedFile              => {
                Type => 'LocalizedFile', Package => 'DE_EPAGES::Object::API::Attributes::LocalizedAttribute' },
        },
        PostReset => {
            MyString                     => {
                Type => 'LocalizedString', Package => 'DE_EPAGES::Object::API::Attributes::LocalizedAttribute' },
            MyPreDefString               => {
                Type => 'PreDefLocalizedString', Package => 'DE_EPAGES::Object::API::Attributes::PreDefined' },
            MyLocalizedString            => {
                Type => 'LocalizedString', Package => 'DE_EPAGES::Object::API::Attributes::LocalizedAttribute' },
            MyPreDefLocalizedString      => {
                Type => 'PreDefLocalizedString', Package => 'DE_EPAGES::Object::API::Attributes::PreDefined' },
            MyPreDefMultiString          => { IsDeleted => 1 },
            MyPreDefMultiLocalizedString => { IsDeleted => 1 },,
            MyPreDefCustomerString       => { IsDeleted => 1 },
            MyInteger                    => {
                Type => 'LocalizedString', Package => 'DE_EPAGES::Object::API::Attributes::LocalizedAttribute' },
            MyFloat                      => {
                Type => 'LocalizedString', Package => 'DE_EPAGES::Object::API::Attributes::LocalizedAttribute' },
            MyBoolean                    => {
                Type => 'LocalizedString', Package => 'DE_EPAGES::Object::API::Attributes::LocalizedAttribute' },
            MyMoney                      => {
                Type => 'LocalizedString', Package => 'DE_EPAGES::Object::API::Attributes::LocalizedAttribute' },
            MyDateTime                   => {
                Type => 'LocalizedString', Package => 'DE_EPAGES::Object::API::Attributes::LocalizedAttribute' },
            MyDate                       => {
                Type => 'LocalizedString', Package => 'DE_EPAGES::Object::API::Attributes::LocalizedAttribute' },
            MyTime                       => {
                Type => 'LocalizedString', Package => 'DE_EPAGES::Object::API::Attributes::LocalizedAttribute' },
            MyFile                       => { IsDeleted => 1 },
            MyLocalizedFile              => { IsDeleted => 1 },
        }
    };

    my $Alias = 'MyProductType';
    ok($Shop->get('ProductTypes')->existsChild($Alias), "ProductType $Alias exists");
    my $MyProductType = $Shop->get('ProductTypes')->child($Alias);
    my $aAttributes = $MyProductType->get('UserDefinedAttributes');
    my @AttributeAliases = map {$_->alias} @$aAttributes;
    # check for existence of all wanted attributes
    note("Tests for ProductType $Alias ($ShopType)");
    my %ConvertedAttributes = (
        'String'   => \&GetPKeyStringAttributesByAttributeID,
        'Integer'  => \&GetPKeyIntegerAttributesByAttributeID,
        'Float'    => \&GetPKeyFloatAttributesByAttributeID,
        'Boolean'  => \&GetPKeyBooleanAttributesByAttributeID,
        'Money'    => \&GetPKeyMoneyAttributesByAttributeID,
        'DateTime' => \&GetPKeyDateTimeAttributesByAttributeID,
        'Date'     => \&GetPKeyDateTimeAttributesByAttributeID,
        'Time'     => \&GetPKeyDateTimeAttributesByAttributeID,

    );

    foreach my $Name (keys %{$Expected->{$ShopType}}) {
        if (exists $Expected->{$ShopType}{$Name}{IsDeleted}) {
            isnt($Name ~~ @AttributeAliases, 1, "Attribute '$Name' is deleted in ProductType '$Alias'");
        } else {
            is($Name ~~ @AttributeAliases, 1, "Attribute '$Name' exists in ProductType '$Alias'");
        }
    }
    # check package and type of existing attributes
    foreach my $Attribute (@$aAttributes) {
        next if $Attribute->get('AttributeClass')->id != $MyProductType->id;
        my @Fields = qw(Alias Type Package);
        my ($Name, $Type, $Package) = @{$Attribute->get([ @Fields ])}{@Fields};
        next if $Expected->{$ShopType}{$Name}{IsDeleted};
        is($Type, $Expected->{$ShopType}{$Name}{Type}, "Attribute' $Name' has Type '$Type'");
        is($Package, $Expected->{$ShopType}{$Name}{Package}, "Attribute '$Name' has Package '$Package'");
        #use Data::Dumper; print Dumper(keys %ConvertedAttributes); die;
        if ($Type ~~ [ keys %ConvertedAttributes ]) {
            my $func = $ConvertedAttributes{$Type};
            if ($ShopType eq 'PreReset') {
                isnt(scalar @{$func->($Attribute->id)}, 0, "Attribute '$Name' has entries in the $Type attribute table")
            } else {
                is(scalar @{$func->($Attribute->id)}, 0, "Attribute '$Name' has NO entries in the $Type attribute table")
            }
        }
    }
}

sub hasPortalConfigOfType {
    my ($Shop, $PortalAlias) = @_;

    my $HasPortalConfig = 0;
    my $aShopPortals = $Shop->get('PortalConfigs');
    foreach my $PortalConfig (@$aShopPortals) {
        $HasPortalConfig = 1 if $PortalConfig->instanceOf($PortalAlias);
    }
    return $HasPortalConfig;
}

sub hasShippingOfType {
    my ($Shop, $ShippingFullfillmentTypeAlias) = @_;
    my $HasShipping = 0;

    my $aShippingMethods = $Shop->get('ShippingMethods');
    foreach my $Shipping (grep {defined $_->get('SelectedShippingFulfillmentMethod')} @$aShippingMethods) {
        if ($Shipping->get('SelectedShippingFulfillmentMethod')->get('ShippingFulfillmentType')->alias eq $ShippingFullfillmentTypeAlias) {
            $HasShipping = 1;
            last;
        }
    }
    return $HasShipping;
}

sub hasPaymentOfType {
    my ($Shop, $PaymentAlias) = @_;

    my $HasPayment = 0;
    my $aShopPaymentMethods = $Shop->get('PaymentMethods');
    foreach my $Payment (@$aShopPaymentMethods) {
        if ($Payment->get('PaymentType')->alias eq $PaymentAlias) {
            $HasPayment = 1;
            last;
        }
    }
    return $HasPayment;
}

sub hasPaymentWithHirePurchase {
    my ($Shop) = @_;
    my $HasHRPayment = 0;
    foreach my $PaymentMethod (@{$Shop->get('PaymentMethods')}) {
        if (defined $PaymentMethod->get('IsHirePurchaseEnabled') && $PaymentMethod->get('IsHirePurchaseEnabled')) {
            $HasHRPayment = 1;
            last;
        }
    }
    return $HasHRPayment;
}

sub PChasEtrackerURLExtension {
    my ($Shop) = @_;
    my $HasExtensionSet = 0;
    foreach my $PortalConfig (@{$Shop->get('PortalConfigs')}) {
        if (defined $PortalConfig->get('EtrackerURLExtension')) {
            $HasExtensionSet = 1;
            last;
        }
    }
    return $HasExtensionSet;
}

sub existsGoogleAdWords {
    my ($Shop) = @_;
    my $aObjects = GetPKeyGoogleAdWordsCodesByShopID($Shop->id);
    return scalar @$aObjects ? 1 : 0;
}

sub existsProductRecommendations {
    my ($Shop) = @_;
    my $aObjects = GetPKeyProductRecommendationUsersByShopID($Shop->id);
    return scalar @$aObjects ? 1 : 0;
}

sub startPagesAreDefault {
    my ($Shop) = @_;

    my $IsDefault = 1;
    my $StartPageClass = LoadClassByAlias('StartPage');
    my $aStartPageIDs = $StartPageClass->findObjects('BySiteIDClassID', [ $Shop->id, $StartPageClass->id ]);
    foreach my $StartPageID (@$aStartPageIDs) {
        next unless ExistsObject($StartPageID);
        my $StartPage = LoadObject($StartPageID);
        my $Values = $StartPage->get([ 'IntroIsVisible', 'IntroContent' ]);
        if ($Values->{IntroIsVisible} != 0 || defined $Values->{IntroContent}) {
            $IsDefault = 0;
        }
    }
    return $IsDefault;
}

sub existsUnAssignedAddress {
    my ($Shop) = @_;

    my $dbi = GetCurrentDBHandle();
    my $UserClass = LoadClassByAlias('User');
    my $BillingAddressAttributeID = $UserClass->attribute('BillingAddress')->id;
    my $ShippingAddressAttributeID = $UserClass->attribute('ShippingAddress')->id;
    my @Parameter = ($Shop->id, $UserClass->id, $BillingAddressAttributeID, $ShippingAddressAttributeID);
    my $dbResult = $dbi->execute(<<END_SQL, \@Parameter, [ 'Integer' ]); #, [qw(AddressID)]);
SELECT count(1) FROM address a
                          INNER JOIN object ao ON a.addressid=ao.objectid AND ao.siteid=?
                          INNER JOIN object pao ON pao.objectid = ao.parentid AND pao.alias='Addresses'
                          INNER JOIN object uo ON uo.objectid=pao.parentid AND uo.classid=?
WHERE NOT EXISTS (SELECT 1 FROM referenceattribute WHERE referencedobjectid=a.addressid AND attributeid=?)
  AND NOT EXISTS (SELECT 1 FROM referenceattribute WHERE referencedobjectid=a.addressid AND attributeid=?)
END_SQL
    return $dbResult->[0][0] > 0 ? 1 : 0;
}

sub existsBookingHours {
    my ($Shop) = @_;

    my @Objects = @{LoadClassByAlias('BookingPeriod')->findObjects('ByCalendarID', [ $Shop->businessHours->id ])};
    return scalar @Objects ? 1 :
        0;
}
sub existsShopObjectsbyClassAlias {
    my ($Shop, $ClassAlias) = @_;
    my $Class = LoadClassByAlias($ClassAlias);
    my @Objects;
    if ($Class->baseClass->alias eq 'BaseObject') {
        @Objects = @{$Class->findObjects('BySiteID', [ $Shop->id ])};
    } else {
        @Objects = @{$Class->findObjects('BySiteIDClassID', [ $Shop->id, $Class->id ])};
    }
    return scalar @Objects ? 1 : 0;
}

sub getCurrentValues {
    my ($Shop) = @_;

    my $LanguageID = GetPKeyLanguageByCode('en');
    my $Expected = getTestData()->{Reset};
    my $Values = $Shop->get([ keys %$Expected ], $LanguageID);
    # Workaround AD-9837
    $Values->{AllowRecommendations} = $Shop->get('AllowRecommendations');
    $Values->{AllowInquiries} = $Shop->get('AllowInquiries');
    return $Values;
}

sub hashDiff {
    my ($a, $b) = @_;

    my $Differ = {};
    foreach my $Key (sort keys %$a) {
        unless ($b->{$Key} eq $a->{$Key}) {
            $Differ->{$Key} = { a => $a->{$Key}, b => $b->{$Key} };
        }
    }
    foreach my $Key (sort keys %$b) {
        next if exists $Differ->{$Key};
        unless ($b->{$Key} eq $a->{$Key}) {
            $Differ->{$Key} = { a => $a->{$Key}, b => $b->{$Key} };
        }
    }
    return $Differ;
}

sub getTestData {
    my $TestHash = {
        Reset => {
            NextAppointmentNo                                            => undef,
            ConfianzaOnlineCode                                          => undef,
            AutoCrossSellingIsVisible                                    => 0,
            AutoCrossSellingCollectData                                  => 0,
            ShowDisplayLink                                              => 0,
            Slogan                                                       => undef,
            eKomiInterfaceID                                             => undef,
            eKomiInterfaceID                                             => undef,
            eKomiInterfacePW                                             => undef,
            eKomiDuration                                                => undef,
            eKomiCertificateURL                                          => undef,
            eKomiWidgetCoordinatesString                                 => undef,
            eKomiMessagesString                                          => undef,
            eKomiInactive                                                => undef,
            eKomiSentRating                                              => undef,
            AlloweKomiEmail                                              => 0,
            eKomiAward                                                   => undef,
            ERPShop                                                      => 0,
            EtrackerClientID                                             => undef,
            EtrackerCode                                                 => undef,
            EtrackerSecureKey                                            => undef,
            EtrackerPassword                                             => undef,
            EtrackerSearchEngineCampaign                                 => undef,
            FacebookStartPageProducts                                    => 0,
            IsFacebookPageUsed                                           => 0,
            'GoogleAdWordsID'                                            => undef,
            JanolawUserID                                                => undef,
            JanolawShopID                                                => undef,
            JanolawLastUpdateDate                                        => undef,
            CachedPagesString                                            => 'Shop;1430;ContentObject;10080;Product;1430;Category;10080',
            LoginRequired                                                => 0,
            LoginCookie                                                  => 0,
            MailEncoding                                                 => undef,
            DefaultPageSize                                              => 10,
            IsPageCacheUsed                                              => 0,
            MaxAgePagesString                                            => 'Category;60;ContentObject;60;Product;60;Shop;60',
            PageCacheMaxAge                                              => '10080',
            UseMobileSF                                                  => '1',
            PageCacheLastInvalidation                                    => undef,
            AddToBasketView                                              => 0,
            AddToBasketOption                                            => 0,
            IsRatingInvitationAcceptanceRequired                         => 0,
            CanSelectTaxAreaOnBasket                                     => 0,
            MinOrderSubTotalAttributeName                                => 'DiscountsSubTotal',
            BasketOverText                                               => undef,
            OrderDocumentsHeader                                         => undef,
            OrderDocumentsInvoiceText2                                   => undef,
            OrderDocumentsPackingSlipText2                               => undef,
            OrderDocumentsCreditNoteText2                                => undef,
            OrderDocumentsDisplayHeaderFooterOnPDF                       => 1,
            OrderDocumentsDisplayHeaderFooterOnEachPage                  => 1,
            OrderDocumentsAttachCustomerInformationPDFToInvoiceEmail     => 0,
            OrderDocumentsAttachCustomerInformationPDFToPackingSlipEmail => 0,
            OrderDocumentsAttachCustomerInformationPDFToCreditNoteEmail  => 0,
            ViewBasketText                                               => undef,
            AddToBasketText                                              => undef,
            ShowThumbnailsInBasket                                       => 1,
            DeliveryPaymentMethodView                                    => undef,
            OrderProcess                                                 => 1,
            OrderProcessHeaderText                                       => undef,
            OrderProcessBasketText                                       => undef,
            ExclusiveAddToBasketButton                                   => 1,
            ProductLowOnStockAction                                      => 2,
            ProductLongerDeliveryTimeText                                => undef,
            ProductNoStockDataText                                       => undef,
            ProductQuantityToShowProductsList                            => 50,
            ShowCustomerSpecificPrices                                   => 0,
            IsProductNoVisible                                           => 0,
            ShowPriceSavings                                             => 0,
            PriceTagInfoSingleResourceNet                                => undef,
            PriceTagInfoSingleProductTaxFree                             => undef,
            PriceTagInfoMultiProductTaxFree                              => undef,
            PriceTagInfoSingleResourceTaxFree                            => undef,
            PriceTagInfoSingleResourceGross                              => undef,
            CancelReferencePrice                                         => 1,
            HideProductWeigt                                             => 0,
            AllowDownloadOfUnpaidProducts                                => 0,
            ShowAvailabilityIcons                                        => 1,
            ProductZoomHeight                                            => 400,
            ProductZoomWidth                                             => 400,
            DisplayDuration                                              => 0,
            DisplayThumbsInZoomView                                      => 1,
            ProductAvailabilityNotificationEmail                         => 0,
            ShowProductMainCharacteristics                               => 'InputField',
            AllowProductComparisonByCustomer                             => 1,
            ShowProductRating                                            => 0,
            ProductRatingImmediatelyVisible                              => 0,
            ProductRatingCustomerRequired                                => 0,
            AllowProductRatingInvitation                                 => 0,
            ProductRatingInvitationPeriod                                => undef,
            AnonymousRecommendation                                      => 0,
            MaxRecommendations                                           => 10,
            AllowRecommendations                                         => 0,
            AllowInquiries                                               => 0,
            InquiriesHidePhoneNo                                         => 0,
            InitialMsgProductRecommendation                              => 1,
            RemoteNoResultMessage                                        => undef,
            FacetedSearchStartDepth                                      => 0,
            FacetedSearchDefaultPageSize                                 => 40,
            FacetedSearchShowFacets                                      => 0,
            SafeBuyURL                                                   => undef,
            AddProductStyle                                              => 0,
            AcceptPrivacyPolicy                                          => 0,
            LoginRequired2Order                                          => 0,
            LoginRequired2InstantlyOrder                                 => 0,
            MobileSFModeShop                                             => 1,
            AdditionalHtmlHead                                           => undef,
            AdditionalHtmlHeadMobileSF                                   => undef,
            IsPriceOnlyForRegisteredUser                                 => 0,
            IsDoubleOptInRegistrationDisabled                            => 0,
            ShowPrintLink                                                => 0,
            ShowCookieLawText                                            => 0,
            AllowShippingAddress                                         => 1,
            ShopSurveyWaitForMerchant                                    => 1,
            AcceptWithReservationText                                    => undef,
            MakeAppointmentLinkText                                      => undef,
            CloseAppointmentLinkText                                     => undef,
            CustomerMustLoginMessage                                     => undef,
            CookieLawMessage                                             => undef,
            MailFromName                                                 => undef,
            ShoppingListVisible                                          => 1,
            TrustedRatingID                                              => undef,
            TrustedRatingDuration                                        => undef,
            TrustedRatingIsActive                                        => undef,
            UnityMBOGuidedTourEnabled                                    => 0,
            ShippingOptionLinkText                                       => undef,
            NextProductNo                                                => '1001',
        },
        Init  => {
            NextAppointmentNo                                            => 9999,
            ConfianzaOnlineCode                                          => 'validCode',
            AutoCrossSellingIsVisible                                    => 1,
            AutoCrossSellingCollectData                                  => 1,
            ShowDisplayLink                                              => 1,
            Slogan                                                       => 'english slogan',
            eKomiInterfaceID                                             => 'validID',
            eKomiInterfaceID                                             => 'eKomiInterfaceID',
            eKomiInterfacePW                                             => 'eKomiInterfacePW',
            eKomiDuration                                                => 123,
            eKomiCertificateURL                                          => 'eKomiCertificateURL',
            eKomiWidgetCoordinatesString                                 => 'eKomiWidgetCoordinatesString',
            eKomiMessagesString                                          => 'eKomiMessagesString',
            eKomiInactive                                                => 'eKomiInactive',
            eKomiSentRating                                              => 'eKomiSentRating',
            AlloweKomiEmail                                              => 1,
            eKomiAward                                                   => 'eKomiAward',
            ERPShop                                                      => 0,
            EtrackerClientID                                             => 'EtrackerClientID',
            EtrackerCode                                                 => 'EtrackerCode',
            EtrackerSecureKey                                            => 'EtrackerSecureKey',
            EtrackerPassword                                             => 'EtrackerPassword',
            EtrackerSearchEngineCampaign                                 => '123456',
            FacebookStartPageProducts                                    => 1,
            IsFacebookPageUsed                                           => 1,
            GoogleAdWordsID                                              => '123456',
            JanolawUserID                                                => '123456',
            JanolawShopID                                                => '123456',
            JanolawLastUpdateDate                                        => '2020-12-01T00:00:00',
            CachedPagesString                                            => 'Shop;9999;',
            LoginRequired                                                => 1,
            LoginCookie                                                  => 1,
            MailEncoding                                                 => 1,
            DefaultPageSize                                              => 20,
            IsPageCacheUsed                                              => 1,
            MaxAgePagesString                                            => 'Shop;120',
            PageCacheMaxAge                                              => '9999',
            UseMobileSF                                                  => '0',
            PageCacheLastInvalidation                                    => '2020-12-01T00:00:00',
            AddToBasketView                                              => 1,
            AddToBasketOption                                            => 1,
            IsRatingInvitationAcceptanceRequired                         => 1,
            CanSelectTaxAreaOnBasket                                     => 1,
            MinOrderSubTotalAttributeName                                => 'SumDelivery',
            BasketOverText                                               => 'someText',
            OrderDocumentsHeader                                         => 'someText',
            OrderDocumentsInvoiceText2                                   => 'someText',
            OrderDocumentsPackingSlipText2                               => 'someText',
            OrderDocumentsCreditNoteText2                                => 'someText',
            OrderDocumentsDisplayHeaderFooterOnPDF                       => 0,
            OrderDocumentsDisplayHeaderFooterOnEachPage                  => 0,
            OrderDocumentsAttachCustomerInformationPDFToInvoiceEmail     => 1,
            OrderDocumentsAttachCustomerInformationPDFToPackingSlipEmail => 1,
            OrderDocumentsAttachCustomerInformationPDFToCreditNoteEmail  => 1,
            ViewBasketText                                               => 'someText',
            AddToBasketText                                              => 'someText',
            ShowThumbnailsInBasket                                       => 0,
            DeliveryPaymentMethodView                                    => 1,
            OrderProcess                                                 => 0,
            OrderProcessHeaderText                                       => 'someText',
            OrderProcessBasketText                                       => 'someText',
            ExclusiveAddToBasketButton                                   => 0,
            ProductLowOnStockAction                                      => 1,
            ProductLongerDeliveryTimeText                                => 'someText',
            ProductNoStockDataText                                       => 'someText',
            ProductQuantityToShowProductsList                            => 51,
            ShowCustomerSpecificPrices                                   => 1,
            IsProductNoVisible                                           => 1,
            ShowPriceSavings                                             => 1,
            PriceTagInfoSingleResourceNet                                => 'someText',
            PriceTagInfoSingleProductTaxFree                             => 'someText',
            PriceTagInfoMultiProductTaxFree                              => 'someText',
            PriceTagInfoSingleResourceTaxFree                            => 'someText',
            PriceTagInfoSingleResourceGross                              => 'someText',
            CancelReferencePrice                                         => 0,
            HideProductWeigt                                             => 1,
            AllowDownloadOfUnpaidProducts                                => 1,
            ShowAvailabilityIcons                                        => 0,
            ProductZoomHeight                                            => 500,
            ProductZoomWidth                                             => 500,
            DisplayDuration                                              => 1,
            DisplayThumbsInZoomView                                      => 0,
            ProductAvailabilityNotificationEmail                         => 1,
            ShowProductMainCharacteristics                               => 'ShortDescription',
            AllowProductComparisonByCustomer                             => 0,
            ShowProductRating                                            => 1,
            ProductRatingImmediatelyVisible                              => 1,
            ProductRatingCustomerRequired                                => 1,
            AllowProductRatingInvitation                                 => 1,
            ProductRatingInvitationPeriod                                => 30,
            AnonymousRecommendation                                      => 1,
            MaxRecommendations                                           => 5,
            AllowRecommendations                                         => 1,
            AllowInquiries                                               => 1,
            InquiriesHidePhoneNo                                         => 1,
            InitialMsgProductRecommendation                              => 0,
            RemoteNoResultMessage                                        => 'someText',
            FacetedSearchStartDepth                                      => 1,
            FacetedSearchDefaultPageSize                                 => 20,
            FacetedSearchShowFacets                                      => 1,
            SafeBuyURL                                                   => 'http://some.url',
            AddProductStyle                                              => 1,
            AcceptPrivacyPolicy                                          => 1,
            LoginRequired2Order                                          => 1,
            LoginRequired2InstantlyOrder                                 => 1,
            MobileSFModeShop                                             => 0,
            AdditionalHtmlHead                                           => 'someText',
            AdditionalHtmlHeadMobileSF                                   => 'someText',
            IsPriceOnlyForRegisteredUser                                 => 1,
            IsDoubleOptInRegistrationDisabled                            => 1,
            ShowPrintLink                                                => 1,
            ShowCookieLawText                                            => 1,
            AllowShippingAddress                                         => 0,
            ShopSurveyWaitForMerchant                                    => 0,
            AcceptWithReservationText                                    => 'someText',
            MakeAppointmentLinkText                                      => 'someText',
            CloseAppointmentLinkText                                     => 'someText',
            CustomerMustLoginMessage                                     => 'someText',
            CookieLawMessage                                             => 'someText',
            MailFromName                                                 => 'some@where.com',
            ShoppingListVisible                                          => 0,
            TrustedRatingID                                              => '12345',
            TrustedRatingDuration                                        => 10,
            TrustedRatingIsActive                                        => 1,
            UnityMBOGuidedTourEnabled                                    => 1,
            ShippingOptionLinkText                                       => 'someText',
            NextProductNo                                                => undef,
        },
    };
    return $TestHash;
}

sub initShop {
    my ($Shop) = @_;

    ImportObjects('ImportFiles/DataForResetShopHandler.xml', $TestData{TestShop}->pathString());
    SetPageSize($Shop, 'SF_ProductsTwoColumns', 10); ## no critic(ValuesAndExpressions::ProhibitMagicNumbers)
    my $ShopData = getTestData()->{Init};
    my $LanguageID = GetPKeyLanguageByCode('en');
    $Shop->set($ShopData, $LanguageID);
    note(sprintf('Importing files to shop %s', $TestData{TestShop}->pathString()));
    my $User = $Shop->child('Users')->child('admin2');
    my $hProductRecommendationUser = {
        'ShopID'   => $Shop->id,
        'UserID'   => $User->id,
        'Number'   => 1,
        'LastDate' => GetCurrentDateTime(),
    };
    InsertProductRecommendationUser($hProductRecommendationUser);
    InsertGoogleAdWordsCode({ 'CouponCode' => '12345', 'IsUsed' => 0, 'ShopID' => $Shop->id });
    my $FileContent = 'test';
    $Shop->set({ 'SafeBuyShield' => 'SafeBuyShield', 'Logo' => 'Logo.gif', 'FavIcon' => 'favicon.ico' });
    WriteFile(catfile($Shop->get('PublicPath'), $Shop->get('SafeBuyShield')), \$FileContent, { create_directory => 1 });
    my $CCAgencyConfigFolder = $Shop->folder('CreditCheckAgencyConfigs');
    my $CCProviderConfigFolder = $Shop->folder('CreditCheckProviderConfigs');
    my $CreditCheckAgency = LoadObjectByPath('/CreditCheckProviders/Mediafinanz/SchufaIdentCheck');
    my $CreditCheckProvider = $CreditCheckAgency->get('CreditCheckProvider');
    unless ($CCProviderConfigFolder->existsChild($CreditCheckProvider->alias)) {
        $CCProviderConfigFolder->insertChild($CreditCheckProvider->get('ConfigClass') // 'CreditCheckProviderConfig', {
            'Alias'               => $CreditCheckProvider->alias,
            'CreditCheckProvider' => $CreditCheckProvider
        });
    }
    unless ($CCAgencyConfigFolder->existsChild($CreditCheckAgency->alias)) {
        $CCAgencyConfigFolder->insertChild($CreditCheckAgency->get('ConfigClass') // 'CreditCheckAgencyConfig', {
            'Alias'             => $CreditCheckAgency->alias,
            'IsActive'          => 0,
            'CreditCheckAgency' => $CreditCheckAgency
        });
    }
    my $Payment = $Shop->folder('PaymentMethods')->child('MoneybookersCreditCard');
    $Payment->set({ 'PaymentType' => LoadObjectByPath('/PaymentTypes/MoneybookersCreditCard') });
    $Payment = $Shop->folder('PaymentMethods')->child('Acceptance');
    $Payment->set({ 'PaymentType' => LoadObjectByPath('/PaymentTypes/Acceptance') });
    $Payment = $Shop->folder('PaymentMethods')->child('PayPalIntegralEvolution');
    $Payment->set({ 'PaymentType' => LoadObjectByPath('/PaymentTypes/PayPalIntegralEvolution') });
    my $MediaGallery = $Shop->child('MediaGallery');
    my $Content = 'Lorem ipsum';
    $MediaGallery->createDirectory('TestDir', 'MyFiles/SomeDir/');
    $MediaGallery->insertFile('TestDir/test.txt', \$Content);
}


sub init {
    my $self = shift;

    $self->SUPER::init;
    my $ShopAlias = $TestData{TestShopAlias};
    my $System = LoadRootObject();
    my $Folder = $System->folder('Shops');
    my $Shop;
    my $ShopTypeRefFolder = $System->folder('ShopTypeRefs');
    my $Class = LoadClassByAlias('ShopTypeRef');
    $Class->insertObject({ Alias => 'ECom5000', 'Parent' => $ShopTypeRefFolder }) unless ($ShopTypeRefFolder->existsChild('ECom5000'));
    if ($Folder->existsChild($ShopAlias)) {
        note(sprintf('Using existing %s', $ShopAlias));
        $Shop = $Folder->child($ShopAlias);
    } else {
        note(sprintf('Creating new shop %s', $ShopAlias));
        $Shop = $Folder->insertChild('Shop', {
            Alias                   => $ShopAlias,
            ShopTypeRef             => $ShopTypeRefFolder->child('ECom5000'),
            WebServerScriptNamePart => $ShopAlias,
            MailFrom                => 'blackhole@epages.com',
        });
    }
    $TestData{TestShop} = $Shop;
    SetShopFeatureValues($Shop, [
        { Alias => 'Languages', MaxValue => '16' },
        { Alias => 'Products', MaxValue => '1500' },
        { Alias => 'Pages', MaxValue => '1500' },
        { Alias => 'ProductTypes', MaxValue => '15' },
        { Alias => 'ProductTypeAttributes', MaxValue => '20' },
        { Alias => 'Variations', MaxValue => '20' },
        { Alias => 'Currencies', MaxValue => '10' },
        { Alias => 'PromotionProducts', MaxValue => '1' },
        { Alias => 'CustomerGroups', MaxValue => '5' },
        { Alias => 'ShippingOption', MaxValue => '5' },
        { Alias => 'PriceLists', MaxValue => '5' },
        { Alias => 'ShippingMethods', MaxValue => '50' },
        { Alias => 'PaymentMethods', MaxValue => '100' },
        { Alias => 'Coupon', MaxValue => '1' },
        { Alias => 'Newsletter', MaxValue => '1' },
        { Alias => 'NewsletterAddressees', MaxValue => '1000' },
        { Alias => 'Mediafinanz', MaxValue => '1' },
        { Alias => 'SiteMap', MaxValue => '1' },
        { Alias => 'Mediafinanz', MaxValue => '1' },
        { Alias => 'Forum', MaxValue => '1' },
        { Alias => 'Blog', MaxValue => '1' },
        { Alias => 'Guestbook', MaxValue => '1' },
        { Alias => 'ImageGallery', MaxValue => '1' },
        { Alias => 'CustomForm', MaxValue => '1' },
        { Alias => 'Moneybookers', MaxValue => '1' },
        { Alias => 'DirectCreditCard', MaxValue => '1' },
        { Alias => 'Ogone', MaxValue => '1' },
        { Alias => 'PayPal', MaxValue => '1' },
        { Alias => 'PayPalExpress', MaxValue => '1' },
        { Alias => 'PayPalPlus', MaxValue => '1' },
        { Alias => 'PayPalPro', MaxValue => '1' },
        { Alias => 'WorldPay', MaxValue => '1' },
        { Alias => 'SipsAtos', MaxValue => '1' },
        { Alias => 'SaferpayBusiness', MaxValue => '1' },
        { Alias => 'Saferpay', MaxValue => '1' },
        { Alias => '4B', MaxValue => '1' },
        { Alias => 'DocdataPay', MaxValue => '1' },
        { Alias => 'AuthorizeNet', MaxValue => '1' },
        { Alias => 'RealexPayment', MaxValue => '1' },
        { Alias => 'Klarna', MaxValue => '1' },
        { Alias => 'CyberMUT', MaxValue => '1' },
        # { Alias => 'SPSBillingOnline', MaxValue => '1' },
        { Alias => 'DHL', MaxValue => '1' },
        { Alias => 'DHLIntraship', MaxValue => '1' },
        { Alias => 'SDA', MaxValue => '1' },
        { Alias => 'SEUR', MaxValue => '1' },
        { Alias => 'UPS', MaxValue => '1' },
        # { Alias => 'Envialia', MaxValue => '1' },
        { Alias => 'Kelkoo', MaxValue => '1' },
        { Alias => 'Pangora', MaxValue => '1' },
        { Alias => 'Preissuchmaschine', MaxValue => '1' },
        { Alias => 'PriceMinister', MaxValue => '1' },
        { Alias => 'Shopzilla', MaxValue => '1' },
        { Alias => 'Toppreise', MaxValue => '1' },
        { Alias => 'Twenga', MaxValue => '1' },
        { Alias => 'GallerySize', MaxValue => '102400' },
    ]);
    initShop($Shop);
    return;
}

sub cleanup {
    my $self = shift;

    my $System = LoadRootObject();
    my $Folder = $System->folder('Shops');
    my $TestShopAlias = $TestData{TestShopAlias};
    if ($Folder->existsChild($TestShopAlias)) {
        note(sprintf('Cleaning up %s', $TestShopAlias));
        my $Shop = $Folder->child($TestShopAlias);
        $Shop->delete();
    }
    return;
}

__PACKAGE__->runScript();

1;
