#========================================================================================
# package      DE_EPAGES::ShopMigration::t::ShopDataExportImport
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::ShopMigration::API::JSONLines::ShopDataExportImport
#========================================================================================
package DE_EPAGES::ShopMigration::t::ShopDataExportImport;
use base DE_EPAGES::Test::API::StoreUnitTest;

use strict;
use utf8;
use warnings;
use File::Spec::Functions;
use DE_EPAGES::Core::API::File qw(GetFileContent GetFileContentLines ExistsFile DeleteDirectory DeleteFile Text2Unix GetFileNames);
use DE_EPAGES::Object::API::Factory qw(LoadRootObject LoadClassByAlias LoadObjectByPath);
use DE_EPAGES::Object::API::Language qw(GetPKeyLanguageByCode);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw(ImportObjects);
use DE_EPAGES::ShopMigration::API::JSON qw(DecodeJSON);
use DE_EPAGES::ShopMigration::API::JSONLines::ShopDataExportImport qw(ExportShopDataJSON ImportShopDataJSON);
use DE_EPAGES::ShopMigration::API::MigrationShop qw(CreatePrepShop ExistsPrepShop SetShopFeatureValues);
use File::Temp qw(tempdir);
use Test::More;

my $TmpDir = tempdir();

if ($ENV{SKIP_IN_PR_TESTS}) {
    plan skip_all => 'Skipping it because SKIP_IN_PR_TESTS == 1';
}

sub test {
    testExportWithShopDataExportImportBase2Prep();
    testExportWithShopDataExportImportPrep2Base();
    testImportWithShopDataExportImportBase2Prep();
    testImportWithShopDataExportImportPrep2Base();
    done_testing();
}

my %TestData = (
    TestShop      => undef,
    TestShopAlias => 'TestShop',
    ImportPath    => '/Test/Shops/TestShop',
);

# This is a bit hacky, the json exporter *can* export a numeric value either as 1 or as "1" (string) depending on how it was used before
# I got random results, so we just check both (as it just concerns the "IsActive" key)
sub testExportWithShopDataExportImportBase2Prep {
    my $Shop = $TestData{TestShop};

    note('test ShopDataExportDriver (Base2Prep)');
    isnt($Shop, undef, 'Testshop exists');
    is(scalar @{$Shop->get('Languages')}, 1, 'Testshop has now one Language');
    is(scalar @{$Shop->get('SiteLocales')}, 1, 'Testshop has one SiteLocale');
    my $Exporter = ExportShopDataJSON($Shop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, 1, 'Export file has 1 line');
    ok($Exporter->fileName =~ /shop.json/, 'export file name matches shop.json');
    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(
        catfile('ShopData.d', 'export', 'TestShop', 'base2prep_shop.json'), { encoding => 'utf8' });
    is($$sFileContent, $$sExpected, 'file content matches');
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');

    foreach my $Lang (qw(de en ru)) {
        my $LanguageID = GetPKeyLanguageByCode($Lang);
        $Shop->addLanguage($LanguageID) unless $Shop->existsLanguage($LanguageID);
    }
    foreach my $Locale (qw(ru_RU de_DE en_GB)) {
        $Shop->addLocale($Locale) unless $Shop->existsLocale($Locale);
    }
    is(scalar @{$Shop->get('Languages')}, 3, 'Testshop has now three Languages');
    is(scalar @{$Shop->get('SiteLocales')}, 3, 'Testshop has now three SiteLocales');
    $Exporter = ExportShopDataJSON($Shop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, 1, 'Export file has 1 line');
    my $JSONContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $hValues = DecodeJSON($$JSONContent);
    ok(exists $hValues->{LocalizedAttributes}{ru}, 'Export file has localized attributes for language ru');
    ok(exists $hValues->{LocalizedAttributes}{de}, 'Export file has localized attributes for language de');
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub testExportWithShopDataExportImportPrep2Base {
    my $PrepShop = $TestData{PrepShop};

    note('test ShopDataExportDriver (Prep2Base)');
    isnt($PrepShop, undef, 'PrepShop exists');
    is(scalar @{$PrepShop->get('Languages')}, 1, 'Testshop has now one Language');
    is(scalar @{$PrepShop->get('SiteLocales')}, 1, 'Testshop has one SiteLocale');
    ok($PrepShop->get('IsMigrationPrepShop'), 'PrepShop has IsMigrationPrepShop=1');
    $PrepShop->set({
        UnitySeenAppIDs                 => { 'customer' => [ 'E659487B-F898-4C7B-AB6C-7E291DABBBD0' ] },
        LastOrderUnitForProductCreation => LoadObjectByPath('/Units/byte/terrabyte')
    });
    my $Exporter = ExportShopDataJSON($PrepShop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists ' . $Exporter->fileName);
    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, 1, 'Export file has 1 line');
    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(
        catfile('ShopData.d', 'export', 'TestShop', 'prep2base_shop.json'), { encoding => 'utf8' });
    is($$sFileContent, $$sExpected, 'file content matches');
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');

    foreach my $Lang (qw(de en ru)) {
        my $LanguageID = GetPKeyLanguageByCode($Lang);
        $PrepShop->addLanguage($LanguageID) unless $PrepShop->existsLanguage($LanguageID);
    }
    foreach my $Locale (qw(ru_RU de_DE en_GB)) {
        $PrepShop->addLocale($Locale) unless $PrepShop->existsLocale($Locale);
    }
    is(scalar @{$PrepShop->get('Languages')}, 3, 'PrepShop has now three Languages');
    is(scalar @{$PrepShop->get('SiteLocales')}, 3, 'PrepShop has now three SiteLocales');
    $Exporter = ExportShopDataJSON($PrepShop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, 1, 'Export file has 1 line');
    my $JSONContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $hValues = DecodeJSON($$JSONContent);
    ok(exists $hValues->{LocalizedAttributes}{ru}, 'Export file has localized attributes for language ru');
    ok(exists $hValues->{LocalizedAttributes}{de}, 'Export file has localized attributes for language de');
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub testImportWithShopDataExportImportBase2Prep {
    my $ExpectedValues = {
        'Attributes'          => {
            'AcceptCustomerInformation' => 0,
            'AcceptTac'                 => 0,
            'DeliveryDimensionUnitInch' => 0,
            'MailFrom'                  => 'someone@somewhere.com',
            'UseProductZoom'            => 0,
            'VATreleased'               => 0,
            'MinOrderAmount'            => 197,
        },
        'LocalizedAttributes' => {
            'en' => {
                'AcceptCustomerInformationText'  => 'text1',
                'AcceptTacText'                  => 'text2',
                'Name'                           => 'text3',
                'OrderDocumentsCreditNoteText1'  => 'text4',
                'OrderDocumentsFooter'           => 'text5',
                'OrderDocumentsInvoiceText1'     => 'text6',
                'OrderDocumentsPackingSlipText1' => 'text7',
                'OrderProcessAddressText'        => 'text8',
                'OrderProcessCheckText'          => 'text9',
                'OrderProcessPaymentText'        => 'text10',
                'OrderProcessShippingText'       => 'text11',
                'Signature'                      => 'text12',
                'VATreleasedMessage'             => 'text13',
                'OrderConfirmationText'          => 'text14',
                'PriceTagInfoSingleProductNet'   => 'text15',
                'PriceTagInfoMultiProductGross'  => 'text16',
                'PriceTagInfoMultiProductNet'    => 'text17',
                'ReferencePriceText'             => 'text18',
                'PriceTagInfoSingleProductGross' => 'text19',
            },
        },
    };

    my $PrepShop = $TestData{PrepShop};
    my $DirName = catdir('ShopData.d', 'import');
    note('test ShopDataImportHandler (Base2Prep)');
    isnt($PrepShop, undef, 'Prepshop exists');
    my $Importer = ImportShopDataJSON($PrepShop, $DirName);
    ok($Importer->fileName() =~ /TestShop.base2prep_shop[.]json/, 'ImportFileName was constructed properly');
    foreach my $Attribute (keys %{$ExpectedValues->{Attributes}}) {
        if ($Attribute eq 'MinOrderAmount') {
            my $MinOrderAmount;
            foreach my $Entry (@{$PrepShop->get('MinOrderAmounts')}) {
                $MinOrderAmount = $Entry->{Value} if ($Entry->{CurrencyID} eq $PrepShop->get('CurrencyID'));
            }
            is($MinOrderAmount, $ExpectedValues->{Attributes}{$Attribute}, "Shop attribute $Attribute matches")
        } else {
            is($PrepShop->get($Attribute), $ExpectedValues->{Attributes}{$Attribute}, "Shop attribute $Attribute matches")
        }
    }
    foreach my $LanguageCode (keys %{$ExpectedValues->{LocalizedAttributes}}) {
        my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
        foreach my $Attribute (keys %{$ExpectedValues->{LocalizedAttributes}{$LanguageCode}}) {
            is($PrepShop->get($Attribute, $LanguageID),
                $ExpectedValues->{LocalizedAttributes}{$LanguageCode}{$Attribute},
                "Shop localized attribute $Attribute matches")
        }
    }
}

sub testImportWithShopDataExportImportPrep2Base {
    my $ExpectedValues = {
        'Attributes'                      => {
            'AcceptCustomerInformation'            => 0,
            'AcceptTac'                            => 0,
            'BillingAddressSetMandatoryFields'     => 'FirstName,LastName,Street,Zipcode,City,CountryID,EMail',
            'BillingAddressSetVisibleFields'       => 'FirstName,LastName,Street,Company,Zipcode,City,CountryID,EMail',
            'DeliveryDimensionUnitInch'            => 0,
            'EmailEventsDisplayLogoUnity'          => 1,
            'MailFrom'                             => 'merchant@somewhere.com',
            'NewMemberCustomerGroup'               => undef,
            'NonMemberCustomerGroup'               => undef,
            'OrderDocumentsDisplayLogoUnity'       => 1,
            'RegistrationSetMandatoryFields'       => 'FirstName,LastName,Street,Zipcode,City,CountryID,EMail',
            'RegistrationSetVisibleFields'         => 'FirstName,LastName,Street,Company,Zipcode,City,CountryID,EMail',
            'ShippingAddressSetMandatoryFields'    => 'FirstName,LastName,Street,Zipcode,City,CountryID',
            'ShippingAddressSetVisibleFields'      => 'FirstName,LastName,Street,Company,Zipcode,City,CountryID,EMail',
            'UnityFacetedSearchFilterCategory'     => 1,
            'UnityFacetedSearchFilterManufacturer' => 1,
            'UnityFacetedSearchFilterPriceRange'   => 1,
            'UnitySeenAppIDs'                      => {
                'customer' => [ 'E659487B-F898-4C7B-AB6C-7E291DABBBD0' ]
            },
            'UnityThemeStylePath'                  => undef,
            'UseProductZoom'                       => 1,
            'MinOrderAmount'                       => 198,
        },
        'LocalizedAttributes'             => {
            'en' =>
                {
                    'AcceptCustomerInformationText'  => 'text0',
                    'AcceptTacText'                  => 'text1',
                    'Name'                           => 'text2',
                    'OrderDocumentsCreditNoteText1'  => 'text3',
                    'OrderDocumentsFooter'           => 'text4',
                    'OrderDocumentsInvoiceText1'     => 'text5',
                    'OrderDocumentsPackingSlipText1' => 'text6',
                    'OrderProcessAddressText'        => 'text7',
                    'OrderProcessCheckText'          => 'text8',
                    'OrderProcessPaymentText'        => 'text9',
                    'OrderProcessShippingText'       => 'text10',
                    'ProductNoAddToBasketText'       => 'text11',
                    'ProductStockAvailableText'      => 'text12',
                    'ProductStockCriticalText'       => 'text13',
                    'Signature'                      => 'text14',
                    'UnityCrossSellingBoxHeadline'   => 'text15',
                    'VATreleasedMessage'             => 'text16',
                    'BasketBelowText'                => 'text17',
                    'OrderConfirmationText'          => 'text18',
                    'PriceTagInfoSingleProductNet'   => 'text19',
                    'PriceTagInfoSingleProductGross' => 'text20',
                    'PriceTagInfoMultiProductNet'    => 'text21',
                    'PriceTagInfoMultiProductGross'  => 'text22',
                    'ReferencePriceText'             => 'text23',
                }
        },
        'LastOrderUnitForProductCreation' => LoadObjectByPath('/Units/byte/terrabyte'),
    };

    my $Shop = $TestData{TestShop};
    my $DirName = catdir('ShopData.d', 'import');
    my $Importer = ImportShopDataJSON($Shop, $DirName);
    note('test ShopDataImportHandler (Prep2Base)');
    isnt($Shop, undef, 'Testshop exists');
    ok($Importer->fileName() =~ /TestShop.prep2base_shop[.]json/, 'ImportFileName was constructed properly');
    foreach my $Attribute (keys %{$ExpectedValues->{Attributes}}) {
        if ($Attribute eq 'UnitySeenAppIDs') {
            is_deeply($Shop->get($Attribute), $ExpectedValues->{Attributes}{$Attribute}, "Shop attribute $Attribute matches")
        } elsif ($Attribute eq 'MinOrderAmount') {
            my $MinOrderAmount;
            foreach my $Entry (@{$Shop->get('MinOrderAmounts')}) {
                $MinOrderAmount = $Entry->{Value} if ($Entry->{CurrencyID} eq $Shop->get('CurrencyID'));
            }
            is($MinOrderAmount, $ExpectedValues->{Attributes}{$Attribute}, "Shop attribute $Attribute matches")
        } else {
            is($Shop->get($Attribute), $ExpectedValues->{Attributes}{$Attribute}, "Shop attribute $Attribute matches")
        }
    }
    foreach my $LanguageCode (keys %{$ExpectedValues->{LocalizedAttributes}}) {
        my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
        foreach my $Attribute (keys %{$ExpectedValues->{LocalizedAttributes}{$LanguageCode}}) {
            is($Shop->get($Attribute, $LanguageID),
                $ExpectedValues->{LocalizedAttributes}{$LanguageCode}{$Attribute},
                "Shop localized attribute $Attribute matches")
        }
    }
    my $LastOrderUnit = $Shop->get('LastOrderUnitForProductCreation');
    is($LastOrderUnit->id, $ExpectedValues->{LastOrderUnitForProductCreation}->id,
        'Shop LastOrderUnitForProductCreation match');
}


# We need to create a fake ShopTypeRef UnityDE + ECom5000 in case they don't exists
# otherwise the Mapper won't work
sub init {
    my $self = shift;

    $self->SUPER::init;
    my $ShopAlias = $TestData{TestShopAlias};
    my $System = LoadRootObject();
    my $Shop;
    my $TestFolder = $System->folder('Test')->folder('Shops');
    my $ShopTypeRefFolder = $System->folder('ShopTypeRefs');
    my $Class = LoadClassByAlias('ShopTypeRef');
    $Class->insertObject({ Alias => 'UnityDE', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('UnityDE'));
    $Class->insertObject({ Alias => 'ECom5000', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('ECom5000'));
    if ($TestFolder->existsChild($ShopAlias)) {
        $Shop = $TestFolder->child($ShopAlias);
    } else {
        $Shop = $TestFolder->insertChild('Shop', {
            Alias                   => $ShopAlias,
            ShopTypeRef             => $ShopTypeRefFolder->child('ECom5000'),
            WebServerScriptNamePart => 'dummyScriptNamePart'
        });
        SetShopFeatureValues($Shop,
            [ { Alias => 'Regions', MaxValue => '2' }, { Alias => 'Languages', MaxValue => '15' } ])
    }
    $TestData{TestShop} = $Shop;
    $TestData{PrepShop} = ExistsPrepShop($Shop) ? $Shop->get('MigrationPrepShop') : CreatePrepShop($Shop);
    SetShopFeatureValues($TestData{PrepShop}, [ { Alias => 'Languages', MaxValue => '15' } ]);
    foreach my $JSONFile (@{GetFileNames('ShopData.d', { 'recursive' => 1 })}) {
        Text2Unix($JSONFile) if ($^O eq 'linux');
    }
    ImportObjects('ImportFiles/ShopDataBase.xml', $TestData{TestShop}->pathString());
    ImportObjects('ImportFiles/ShopDataNow.xml', $TestData{PrepShop}->pathString());
    # MinOrderAmounts won't import via XML so..
    $TestData{TestShop}->set({ MinOrderAmounts => [ { CurrencyID => 'GBP', Value => '97' } ] });
    $TestData{PrepShop}->set({ MinOrderAmounts => [ { CurrencyID => 'GBP', Value => '98' } ] });
    return;
}

sub cleanup {
    my $self = shift;

    my $ShopAlias = $TestData{TestShopAlias};
    my $TestFolder = LoadRootObject()->folder('Test')->folder('Shops');
    if ($TestFolder->existsChild($ShopAlias)) {
        my $Shop = $TestFolder->child($ShopAlias);
        my $PrepShop = $Shop->get('MigrationPrepShop');
        $PrepShop->delete() if (defined $PrepShop);
        $Shop->delete();
        LoadRootObject()->folder('Test')->delete();
    }
    DeleteDirectory($TmpDir, { 'recursive' => 1 });
    return;
}

__PACKAGE__->runScript();

1;
