#==============================================================================
# package      DE_EPAGES::ShopTransfer::API::DBI
# state        public
# description  Database helper functions with DBMS-independend implementation.
#               The DBMS specifics are implemented in sub classes.
#==============================================================================
package DE_EPAGES::ShopTransfer::API::DBI;

use strict;
use version;
use DateTime;
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle);
use DE_EPAGES::ShopTransfer::API::Constants qw( IGNORE_TABLES_EXPORT );

sub new {
    my $class = shift;
    my $self = bless {}, $class;
    my $dbi = GetCurrentDBHandle();
    if (version->parse($dbi->databaseVersion) >= version->parse(8)) {
        $dbi->do('set information_schema_stats_expiry=0');
    }
    return $self;
}

#========================================================================================
# function     getTables
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aTables = $dbi->getTables();
#----------------------------------------------------------------------------------------
# description  Returns list of all table names, in correct order of delete.
#----------------------------------------------------------------------------------------
# return       $aTables | table names | ref.array.string
#========================================================================================
sub getTables {
    my $self = shift;
    my $dbi = GetCurrentDBHandle();

    return $dbi->sortDeleteAllTables( $dbi->getAllTableReferences );
}

#========================================================================================
# function     exportableTables
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aTables = $dbi->exportableTables();
#----------------------------------------------------------------------------------------
# description  Returns list of tables names to consider for export, in correct order of
#               delete.
#               Exceptions are: IGNORE_TABLES_EXPORT and those ending in '_nextid'.
#----------------------------------------------------------------------------------------
# return       $aTables | table names | ref.array.string
#========================================================================================
sub exportableTables {
    my $self = shift;

    my %Exceptions = map { $_ => undef } IGNORE_TABLES_EXPORT;
    return [
        grep { !/_nextid$/ }
        grep { !exists $Exceptions{$_} }
        @{ $self->getTables }
    ];
}

#========================================================================================
# function     systemTables
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aTables = $dbi->systemTables();
#----------------------------------------------------------------------------------------
# description  Returns the tables to consider for export of the system.
#----------------------------------------------------------------------------------------
# return       $aTables | list of table names | ref.array.string
#========================================================================================
sub systemTables {
    my $self = shift;

    return [
        grep { !$self->existsColumn( $_, 'shopid') }
        grep { !$self->existsColumn( $_, 'productid') }
        @{ $self->exportableTables }
    ];
}

#========================================================================================
# function     shopTables
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aTables = $dbi->shopTables();
#----------------------------------------------------------------------------------------
# description  Returns the tables to consider for export of shops.
#----------------------------------------------------------------------------------------
# return       $aTables | list of table names | ref.array.string
#========================================================================================
sub shopTables {
    my $self = shift;

    return [
        grep { defined $self->getObjectColumn($_) }
        @{ $self->exportableTables }
    ];
}

#========================================================================================
# function     dynamicIDTables
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aTables = $dbi->dynamicIDTables();
#----------------------------------------------------------------------------------------
# description  Returns exportable tables, which have only one primary key, no foreign
#               key on it and no autoincrement.
#----------------------------------------------------------------------------------------
# return       $aTables | list of table names | ref.array.string
#========================================================================================
sub dynamicIDTables {
    my $self = shift;

    my @Tables;
    foreach my $Table ( @{$self->exportableTables} ) {
        my @PKeyColumns = $self->getPKeyOfTable($Table);
        my $isColumnObjectID = $self->isColumnObjectID($Table,$PKeyColumns[0]);
        my $isColumnAutoIncrement = $self->isColumnAutoIncrement($Table);

        #print "$Table => PKeys:" . scalar @PKeyColumns . "; Column: $PKeyColumns[0]; isColumnObjectID: $isColumnObjectID; isColumnAutoIncrement: $isColumnAutoIncrement\n";
        next if $isColumnAutoIncrement or scalar @PKeyColumns > 1 or $isColumnObjectID;

        push @Tables, $Table;
    }
    return \@Tables;
}

#========================================================================================
# function     quoteParameter
#----------------------------------------------------------------------------------------
# syntax       $Parameter = $dbi->quoteParameter($Type, $Value);
#----------------------------------------------------------------------------------------
# example      $Parameter = $dbi->quoteParameter('String', "don't");
#----------------------------------------------------------------------------------------
# description  Tests and quotes the parameters according to the data type spefification.
#----------------------------------------------------------------------------------------
# input        $Type | data type specification for parameters | ref.hash
# input        $Value | value | *
# return       $Parameter | parameter for database | string
#========================================================================================
sub quoteParameter {
    my $self = shift;
    my ($Type, $Value) = @_;

    return 'null' unless defined $Value;

    if ($Type =~ /^DateTime|Time|Date$/) {
        if( $Value =~ /(\d{4})\/(\d{2})\/(\d{2})\ (\d{2}):(\d{2}):(\d{2})/x ) {
            # set default values on undef
            my $year   = $1 ? $1 : 0;
            my $month  = $2 ? $2 : 1;
            my $day    = $3 ? $3 : 1;
            my $hour   = $4 ? $4 : 0;
            my $minute = $5 ? $5 : 0;
            my $sec    = $6 ? $6 : 0;
            $Value = DateTime->new( year   => $year,
                              month  => $month,
                              day    => $day,
                              hour   => $hour,
                              minute => $minute,
                              second => $sec
                            );
        }
        else {
            die "invalid DateTime value: $Value";
        }
    }
    my $dbi  = GetCurrentDBHandle();
    return $dbi->quoteParameter($Type, $Value);
}
#========================================================================================
# function     getDBName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbName = $dbi->getDBName();
#----------------------------------------------------------------------------------------
# description  returns the current db name.
#----------------------------------------------------------------------------------------
# return       $DbName | name of current database | string
#========================================================================================
sub getDBName {
    return GetCurrentDBHandle()->databaseName;
}

#========================================================================================
# function     getAllPrimaryKeys
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hTablePrimaryKeys = $dbi->getAllPrimaryKeys;
#----------------------------------------------------------------------------------------
# description  Returns all primary keys of all tables in the current database.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       $hTableReferences |
#                   {
#                       $TableName => [ $ColumnName, ... ],
#                       ...
#                   }
#               | ref.hash.hash.string/array
#========================================================================================
sub getAllPrimaryKeys { ## no critic qw(RequireCommentForExportedFunctions)
    die 'Abstract method getAllPrimaryKeys - must be implemented in subclass!';
}

#========================================================================================
# function     getPKeyOfTable
# state        public
#----------------------------------------------------------------------------------------
# syntax       @PrimaryKeyColumns = $dbi->getPKeyOfTable($Table);
# syntax       $PrimaryKeyColumn = $dbi->getPKeyOfTable($Table);
#----------------------------------------------------------------------------------------
# description  Returns the primary key of a table.
#               Returns an empty array if the table does not have a primary key.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       @PrimaryKeyColumns | column names | array.string
# return       $PrimaryKeyColumn | first column name | string
#========================================================================================
sub getPKeyOfTable {
    my $self = shift;
    my ($Table) = @_;

    my $aColumns = $self->getAllPrimaryKeys->{$Table};
    return unless defined $aColumns;
    return wantarray ? @$aColumns : $aColumns->[0];
}

#========================================================================================
# function     getMasterOfReference
# state        public
#----------------------------------------------------------------------------------------
# syntax       ($MasterTable, $MasterColumn) = $dbi->getMasterOfReference($Table, $Column);
#----------------------------------------------------------------------------------------
# description  Returns the master of foreign key.
#               Returns an empty array if there is no reference.
#----------------------------------------------------------------------------------------
# input        $Table | slave table name | string
# input        $Column | slave column name | string
# return       $MasterTable | master table name | string
# return       $MasterColumn | master column name | string
#========================================================================================
sub getMasterOfReference {
    my $self = shift;
    my ($Table, $Column) = @_;

    my $hReference = $self->getTableReferences($Table)->{$Column};
    return unless defined $hReference;

    return $hReference->{'MasterTableName'}, $hReference->{'MasterColumnName'};
}

#========================================================================================
# function     getMasterColumn
# state        public
#----------------------------------------------------------------------------------------
# syntax       ($MasterTable, $MasterColumn) = $dbi->getMasterColumn($Table, $Column);
#----------------------------------------------------------------------------------------
# description  Returns the masterMaster of a foreign key chain. Returns undef unless
#               there is no reference.
#----------------------------------------------------------------------------------------
# input        $Table | slave table name | string
# input        $Column | slave column name | string
# return        $MasterTable | masterMaster table name | string
# return        $MasterColumn | masterMaster column name | string
#========================================================================================
sub getMasterColumn {
    my $self = shift;
    my ($Table, $Column) = @_;

    my @Master = $self->getMasterOfReference($Table, $Column);
    my @LastMaster = @Master;

    while (scalar @Master) {
        @LastMaster = @Master;
        @Master = $self->getMasterOfReference($Master[0],$Master[1]);
    }
    return @LastMaster;
}

#========================================================================================
# function     getAllReferences
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hTableReferences = $dbi->getAllReferences;
#----------------------------------------------------------------------------------------
# description  Returns all foreign keys of all tables in the current database.
#----------------------------------------------------------------------------------------
# return       $hTableReferences |
#                   {
#                       $TableName => {
#                           $ColumnName => {
#                               ConstraintName => $ConstraintName,
#                               MasterTableName => $MasterTableName,
#                               MasterColumnName => $MasterColumnName,
#                           },...
#                       }, ...
#                   }
#               | ref.hash.hash.hash.string
#========================================================================================
sub getAllReferences {
    die 'Abstract method - must be implemented in subclass!';
}

#========================================================================================
# function     getTableReferences
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hReferences = $dbi->getTableReferences($Table);
#----------------------------------------------------------------------------------------
# description  Returns the reference info of a table.
#               Returns an empty hash if the table has no references.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       $hReferences |
#                       {
#                           $ColumnName => {
#                               ConstraintName => $ConstraintName,
#                               MasterTableName => $MasterTableName,
#                               MasterColumnName => $MasterColumnName,
#                           },...
#                       }
#               | ref.hash.hash.string
#========================================================================================
sub getTableReferences {
    my $self = shift;
    my ($Table) = @_;

    return $self->getAllReferences->{$Table} || {};
}

#========================================================================================
# function     getReferenceColumns
# state        public
#----------------------------------------------------------------------------------------
# syntax       @Columns = $dbi->getReferenceColumns($Table);
#----------------------------------------------------------------------------------------
# description  Returns the columns of a table, which have foreign keys.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       @Columns | column names | array.string
#========================================================================================
sub getReferenceColumns {
    my $self = shift;
    my ($Table) = @_;

    return keys %{ $self->getTableReferences($Table) };
}

#========================================================================================
# function     IsColumnObjectID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsObjectColumn = $dbi->isColumnObjectID($Table, $Column);
#----------------------------------------------------------------------------------------
# description  Returns true the columns is a object identifier.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $Column | column name | string
# return       $IsObjectColumn | column an object identifier | boolean
#========================================================================================
sub isColumnObjectID {
    my $self = shift;
    my ($Table, $Column) = @_;
    my @Master = $self->getMasterColumn($Table, $Column);

    return (scalar @Master and $Master[0] eq 'object' and $Master[1] eq 'objectid') ? 1 : 0;
}

#========================================================================================
# function     GetIDByPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ObjectID = $dbi->getIDByPath(@Path);
# example      $ShopID = $dbi->getIDByPath('Shops',$ShopAlias);
#----------------------------------------------------------------------------------------
# description  Returns the identifier of an object without loading objects.
#----------------------------------------------------------------------------------------
# input        @Path | path names | array.string
# return       $ObjectID | indentifier | integer
#========================================================================================
sub getIDByPath {
    my $self = shift;
    my @Path = @_;

    my $dbi = GetCurrentDBHandle();
    my $sql = 'SELECT objectid from object where parentid = ? and alias = ?';
    my $ID = 1;
    foreach my $Alias (@Path) {
        my $aaResult = $dbi->execute($sql, [$ID, $Alias]);
        $ID = $aaResult->[0][0];
    }
    return $ID;
}

#========================================================================================
# function     getAliasByID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Alias = $dbi->getAliasByID( $ObjectID );
#----------------------------------------------------------------------------------------
# description  Returns the alias of an object without loading objects.
#               Returns undef if the object does not exist.
#----------------------------------------------------------------------------------------
# input        $ObjectID | indentifier | integer
# return       $Alias | alias | string
#========================================================================================
sub getAliasByID {
    my $self = shift;
    my ($ObjectID) = @_;

    my $dbi = GetCurrentDBHandle();
    return $dbi->execute('SELECT alias FROM object WHERE objectid = ?', [$ObjectID])->[0]->[0];
}

#========================================================================================
# function     getObjectGUID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $GUID = $dbi->getObjectGUID( $ObjectID );
#----------------------------------------------------------------------------------------
# description  Returns the GUID of an object without loading objects.
#               Returns undef if the object does not exist.
#----------------------------------------------------------------------------------------
# input        $ObjectID | indentifier | integer
# return       $GUID | object GUID | string
#========================================================================================
sub getObjectGUID {
    my $self = shift;
    my ($ObjectID) = @_;

    my $dbi = GetCurrentDBHandle();
    return $dbi->execute('SELECT guid FROM object WHERE objectid = ?', [$ObjectID])->[0]->[0];
}

#========================================================================================
# function     getObjectColumn
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Column = $dbi->getObjectColumn($Table);
#----------------------------------------------------------------------------------------
# description  Returns the column which can be used for siteid assignment.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       $Column | column name | string
#========================================================================================
sub getObjectColumn {
    my $self = shift;
    my ($Table) = @_;

    # direct mapping to shop/site
    return 'siteid' if $self->existsColumn( $Table, 'siteid');
    return 'shopid' if $self->existsColumn( $Table, 'shopid');

    # indirect mapping to shop/site_ use a part of the primary key
    my @PKeyColumns = grep { $self->isColumnObjectID($Table, $_) } $self->getPKeyOfTable($Table);
    if (scalar @PKeyColumns) {
        ## EPG-26257: get 'trusteeid' instead of 'groupid' in order to fetch SystemMerchants (not a site object)
        return $PKeyColumns[$self->getColumnPosition($Table, 'trusteeid')] if $Table eq 'groupmember';
        return $PKeyColumns[0];
    }

    # for mapping tables use the first object column
    my @ObjectRefColumns = grep { $self->isColumnObjectID($Table, $_) } $self->getReferenceColumns($Table);
    my %ObjectRefColumns = map { $_ => undef } @ObjectRefColumns;
    return ( grep { exists $ObjectRefColumns{$_} } $self->getColumns($Table) )[0];
}

#========================================================================================
# function     getAllColumns
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hTableColumns = $dbi->getAllColumns;
#----------------------------------------------------------------------------------------
# description  Returns all columns of all tables in the current database.
#----------------------------------------------------------------------------------------
# return       $hTableReferences |
#                   {
#                       $TableName => {
#                           $ColumnName => {
#                               Position => $Position,
#                           },...
#                       }, ...
#                   }
#               | ref.hash.hash.hash.string
#========================================================================================
sub getAllColumns {
    die 'Abstract method - must be implemented in subclass!';
}

#========================================================================================
# function     getColumns
# state        public
#----------------------------------------------------------------------------------------
# syntax       @Columns = $dbi->getColumns($Table);
#----------------------------------------------------------------------------------------
# description  Returns all columns.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       @Columns | column names | array.string
#========================================================================================
sub getColumns {
    my $self = shift;
    my ($Table) = @_;

    my $hColumns = $self->getAllColumns->{$Table};
    die "Table $Table doesn't exist" unless defined $hColumns;

    return(
        sort { $hColumns->{$a}->{'Position'} <=> $hColumns->{$b}->{'Position'} }
        keys %$hColumns
    );
}

#========================================================================================
# function     existsColumn
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $dbi->existsColumn($Table, $Column)
#----------------------------------------------------------------------------------------
# description  Returns true if column exists in table.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $Column | column name | string
# return       $Exists | true if column exists | boolean
#========================================================================================
sub existsColumn {
    my $self = shift;
    my ($Table, $Column) = @_;

    my $hColumns = $self->getAllColumns->{$Table};
    return 0 unless defined $hColumns;
    return exists $hColumns->{$Column} ? 1 : 0;
}

#========================================================================================
# function     getUniqueIndexes
# state        public
#----------------------------------------------------------------------------------------
# syntax       @Columns = $dbi->getUniqueIndexes($Table);
#----------------------------------------------------------------------------------------
# description  Returns first columns of all unique indexes.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       @Columns | column names | array.string
#========================================================================================
sub getUniqueIndexes {
    my $self = shift;
    my ($Table) = @_;

    die 'Abstract method - must be implemented in subclass!';
}

#========================================================================================
# function     getTypesOfColumns
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hTypes= $dbi->getTypesOfColumns($Table);
#----------------------------------------------------------------------------------------
# description  Returns types of columns. Currently used types: bit,char,datetime,float,
#               int,money,nchar,numeric,nvarchar,text,tinyint,varchar
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       $hTypes | column types key: columnname value: type | ref.hash.string
#========================================================================================
sub getTypesOfColumns {
    my $self = shift;
    my ($Table) = @_;

    die 'Abstract method - must be implemented in subclass!';
}

#========================================================================================
# function     getGeneralizeDateTimeStatement
# state        public
#----------------------------------------------------------------------------------------
# syntax       $statement = $dbi->getGeneralizeDateTimeStatement($Column, $TablePrefix);
#----------------------------------------------------------------------------------------
# description  Returns a statement for querying datetime columns in a generalized
#               format. Instead of returning the plain column name, a convert statement
#               like 'convert(char(19), $TablePrefix$Column, 117)' is returned and
#               can be integrated into a 'SELECT $statement from table where bla' query.
#----------------------------------------------------------------------------------------
# input        $Column | column name | string
# input        $TablePrefix | prefix, which should be inserted before the column name | string
# return       $statement | the new column statement | string
#========================================================================================
sub getGeneralizeDateTimeStatement {
    my $self = shift;
    my ($Column, $TablePrefix) = @_;

    die 'Abstract method - must be implemented in subclass!';
}

#========================================================================================
# function     getColumnPosition
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Position = $dbi->getColumnPosition($Table, $Column);
#----------------------------------------------------------------------------------------
# description  Returns the position of the column (start with 0).
#               Returns undef if column does not exist.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $Column | column name | string
# return       $Position | position of column | integer
#========================================================================================
sub getColumnPosition {
    my $self = shift;
    my ($Table, $Column) = @_;

    my $hColumns = $self->getAllColumns->{$Table};
    die "Table $Table doesn't exist" unless defined $hColumns;

    my $hColumn = $hColumns->{$Column};
    die "Column $Column does not exist in table $Table" unless defined $hColumn;

    return $hColumn->{'Position'};
}

#========================================================================================
# function     isColumnAutoIncrement
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isAutoInc = $dbi->isColumnAutoIncrement($Table);
#----------------------------------------------------------------------------------------
# description  Returns 1 if the table has an auto increment column or an _nextid table
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       $Parameter | binary value | int
#========================================================================================
sub isColumnAutoIncrement {
    my $self = shift;
    my ($Table) = @_;

    die 'Abstract method - must be implemented in subclass!';
}

#========================================================================================
# function     disableForeignKeyChecks
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->disableForeignKeyChecks($Code);
#----------------------------------------------------------------------------------------
# description  Disables foreign key checks during execution of $Code.
#----------------------------------------------------------------------------------------
# input        $Code | code reference | code
#========================================================================================
sub disableForeignKeyChecks {
    my $self = shift;
    my ($Code) = @_;

    # the general implementation does not disable foreign keys, because it's not
    # possible for all DBMS.
    $Code->();
    return;
}

#========================================================================================
# function     doLimit
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->doLimit($SQL, $RowCount, $aParams);
#----------------------------------------------------------------------------------------
# description  Runs the SQL statement with $dbi->do(), but limit the number of affected
#               rows.
#----------------------------------------------------------------------------------------
# input        $SQL | SQL statement (DELETE or UPDATE) | string
# input        $RowCount | maximum number of affected rows | integer
# input        $aParams | parameters for the SQL | ref.array
#========================================================================================
sub doLimit {
    my $self = shift;
    my ($RowCount, $SQL, $aParams) = @_;

    # default: don't limit the number of rows
    my $dbi  = GetCurrentDBHandle();
    $dbi->do($SQL, $aParams);
    return;
}

1;
