#========================================================================================
# package      DE_EPAGES::Spreedly::API::StripeUrlBuilder
# state        public
#----------------------------------------------------------------------------------------
# description  Build return URL for Backlink
#========================================================================================
package DE_EPAGES::Spreedly::API::StripeUrlBuilder;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Url qw ( ParseQueryString );
use URI;
use URI::URL;
use DE_EPAGES::Core::API::Log qw ( GetLog );

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Spreedly::API::StripeUrlBuilder
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}
#========================================================================================
# function     buildStripeBacklinkUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildStripeBacklinkUrl( $hQueryParameters );
#----------------------------------------------------------------------------------------
# description  Build BacklinkUrl with return data from Stripe
#----------------------------------------------------------------------------------------
# input        $hQueryParameters | Hashref which is returned from Stripe       | hashref
# return       $URL->as_string() | BacklinkUrl with AuthorizationCode or Error | string
#========================================================================================
sub buildStripeBacklinkUrl {
    my $self = shift;
    my ($hQueryParameters) = @_;
    my $aBacklinkUrl = $hQueryParameters->{'state'};

    GetLog()->info('In backlinkUrl builder');

    if ( (!isValid($aBacklinkUrl)) or (!isValid($aBacklinkUrl->[0])) ) {
        Error('NoURLFound');
    }

    delete($hQueryParameters->{'ChangeAction'}); # UI_Stripe is already called by ChangeAction and we don't need it anymore
    return $self->_urlBuilder($hQueryParameters);
}
#----------------------------------------------------------------------------------------
sub _urlBuilder {
    my $self = shift;
    my ($hQueryParameters) = @_;

    my $BackLinkUrl = delete($hQueryParameters->{'state'}); #State is Long URL which is backlink to MBO. No need to send it with QueryString again
    my $URL = URI->new($BackLinkUrl->[0]);
    my $hBacklinkUrlParameters = ParseQueryString($URL->query(), 'utf-8');

    my $hQueryParam = {%{$hBacklinkUrlParameters}, %{$hQueryParameters}};
    $URL->query_form($hQueryParam);

    return $URL->as_string();
}
#----------------------------------------------------------------------------------------
sub buildConnectAndReconnectUrl {
    my $self = shift;
    my ($PaymentMethod, $StateURL) = @_;

    use DE_EPAGES::Core::API::Log qw ( GetLog );
    GetLog()->info('In backlinkUrl builder, PaymentMethod::PaymentType');
    GetLog()->info($PaymentMethod->get('PaymentType'));

    my $hQueryParameters = $self->_buildQueryParameters($PaymentMethod, $StateURL);
    $hQueryParameters = $self->_addOptionalAddressQueryParams($PaymentMethod, $hQueryParameters);

    my $Url = $self->_buildEncodedUrl($hQueryParameters, $PaymentMethod);

    return $Url;
}
#----------------------------------------------------------------------------------------
sub _buildEncodedUrl {
    my $self = shift;
    my ($hQueryParameters, $PaymentMethod) = @_;

    my $PaymentType = $PaymentMethod->get('PaymentType');
    my $OnBoardingUrl = $PaymentType->get('OnBoardingUrl');

    my $Url = URI::URL->new($OnBoardingUrl);
    $Url->query_form($hQueryParameters);
    my $hBacklinkUrlParameters = ParseQueryString($Url->equery(), 'utf-8');
    $Url->query_form( $hBacklinkUrlParameters );

    return $Url;
}
#----------------------------------------------------------------------------------------
sub _buildQueryParameters {
    my $self = shift;
    my ($PaymentMethod, $StateURL) = @_;
    my $Shop = $PaymentMethod->getSite();
    my $IsInternalTestShop = $Shop->get('IsInternalTestShop');
    my $PaymentType = $PaymentMethod->get('PaymentType');
    my $ClientID = ($IsInternalTestShop) ? $PaymentType->get('TestAccountName') : $PaymentType->get('LiveAccountName');

    my $RedirectUrl = $self->_getSiteUrlFromEpagesConf();
    $RedirectUrl .= '?ChangeAction=BacklinkSpreedlyStripeOnboarding';

    return {
        'response_type' => 'code',
        'client_id' => $ClientID,
        'scope' => 'read_write',
        'redirect_uri' => $RedirectUrl,
        'state' => $StateURL,
    };
}
#----------------------------------------------------------------------------------------
sub _addOptionalAddressQueryParams {
    my $self = shift;
    my ($PaymentMethod, $hQueryParameters) = @_;

    my $Shop = $PaymentMethod->getSite();
    my $Address = $Shop->get('Address');

    my $CountryCode2;
    if($Address->get('Country') && ref($Address->get('Country')) eq 'HASH'){
        $CountryCode2 = $Address->get('Country')->{'Code2'};
    }
    my $hAddressParams = {
        'stripe_user[country]' => $CountryCode2,
        'stripe_user[first_name]' => $Address->get('FirstName'),
        'stripe_user[last_name]' => $Address->get('LastName'),
        'stripe_user[street_address]' => $Address->get('Street'),
        'stripe_user[city]' => $Address->get('City'),
        'stripe_user[state]' => $Address->get('State'),
        'stripe_user[zip]' => $Address->get('Zipcode'),
        'stripe_user[phone_number]' => $Address->get('Phone'),
        'stripe_user[email]' => $Address->get('EMail'),
    };

    foreach my $Key (keys %{$hAddressParams}){
        if($hAddressParams->{$Key}){
            $hQueryParameters->{$Key} = $hAddressParams->{$Key};
        }
    }

    return $hQueryParameters;
}
#----------------------------------------------------------------------------------------
sub _getSiteUrlFromEpagesConf {
    my $self = shift;

    my $EpagesConf = DE_EPAGES::Core::API::IniConfig->new( 'FileName' => $ENV{'EPAGES_CONFIG'}.'/epages.conf', 'CaseSensitive' => 1 );
    return $EpagesConf->get( 'DE_EPAGES::Spreedly', 'SiteUrl');
}
1;