use strict;
use warnings;
use Getopt::Long;
use Time::HiRes qw( time );

use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadObject );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );
use DE_EPAGES::Spreedly::API::Cartridge;
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );


sub Main {
    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }

    my $time1 = time;
    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { InitializeSpreedlyStripePaymentIntentsNotification(); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

#========================================================================================
# function         InitializeSpreedlyStripePaymentIntentsNotification
#----------------------------------------------------------------------------------------
# syntax           InitializeSpreedlyStripePaymentIntentsNotification();
#----------------------------------------------------------------------------------------
# description      Adds and activates the notification for the Spreedly-Stripe-Payment-Intents
#                   payment method to all Unity shops that have the related feature enabled.
#========================================================================================
sub InitializeSpreedlyStripePaymentIntentsNotification {
    print "Reading all Unity shops\n";
    my $aShopIDs = GetUnityShopIDs();

    my $Cartridge = DE_EPAGES::Spreedly::API::Cartridge->new();

    foreach my $ShopID ( @$aShopIDs ) {
        my $Shop = LoadObject($ShopID);
        next unless $Shop->featureMaxValue('SpreedlyStripePaymentIntents');
        if (isPaymentMethodSpreedlyStripeInstalledAndActivated($Shop)) {
            print "PaymentMethod 'Spreedly-Stripe' is activated for ShopID: " . $ShopID . ", Shop path:  " . $Shop->pathString() . " \n ";
            if ($Shop->folder('ShopNotifications')->existsChild('SpreedlyStripePaymentIntents')) {
                print "Notification NOT imported (already exists). ShopID: " . $ShopID . ", Shop path:  " . $Shop->pathString() . " \n ";
            }
            else {
                eval {
                    # import shop notifications
                    ImportObjects($Cartridge->xmlDirectory() . '/ShopNotificationSpreedlyStripePaymentIntents.xml', $Shop->pathString());
                };
                if (ExistsError()) {
                    warn('Patch failed in shop ' . $ShopID . " with error \"" . GetError() . "\".")
                } else {
                    print "Notification imported. ShopID: " . $ShopID . ", Shop path:  " . $Shop->pathString() . " \n ";
                }
            }
        }
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();
}

sub isPaymentMethodSpreedlyStripeInstalledAndActivated {
    my ($Shop) = @_;

    my $PaymentMethods = $Shop->get('PaymentMethods');
    foreach my $PaymentMethod (@$PaymentMethods){
        if ($PaymentMethod->class()->alias() eq 'PaymentMethodSpreedlyStripe') {
            if ( $PaymentMethod->get('IsActivated') ) {
                return 1;
            }
        }
    }

    return 0;
}

sub usage {
    print <<END_USAGE;
Description:
    Adds and activates the notification for the Spreedly-Stripe-Payment-Intents
    payment method to all existing Unity shops.
Usage:
    perl $0 [options] [flags] datafiles
options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
flags:
    -help       show the command line options
Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

#========================================================================================
# SCRIPT RUN
#========================================================================================
RunScript('Sub' => \&Main);